@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	20 December 1989
- * Last Edited:	21 May 1996
+ * Last Edited:	15 October 1996
  *
  * Copyright 1996 by the University of Washington
  *
@@ -1142,7 +1142,7 @@ int bezerk_lock (char *file,int flags,int mode,char *lock,int op)
   time_t t;
   struct stat sb;
 				/* build lock filename */
-  if (chk_notsymlink (strcat (dummy_file (lock,file),".lock"))) do {
+  if (chk_notsymlink (strcat (dummy_file (lock,file),".lock"),NIL)) do {
     t = time (0);		/* get the time now */
 #ifdef NFSKLUDGE
   /* SUN-OS had an NFS, As kludgy as an albatross;
