@@ -10,9 +10,9 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	24 June 1992
- * Last Edited:	22 August 1994
+ * Last Edited:	14 March 1996
  *
- * Copyright 1994 by the University of Washington
+ * Copyright 1996 by the University of Washington
  *
  *  Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -41,11 +41,12 @@
 #include "mail.h"
 #include "osdep.h"
 #include <time.h>
-#include <sys\stat.h>
+#include <sys/stat.h>
 #include <dos.h>
 #include <io.h>
 #include "dawz.h"
 #include "rfc822.h"
+#include "dummy.h"
 #include "misc.h"
 
 /* Dawz mail routines */
@@ -111,27 +112,29 @@ DRIVER *dawz_valid (char *name)
 long dawz_isvalid (char *name)
 {
   int fd;
+  long ret = NIL;
   char *s,tmp[MAILTMPLEN];
   struct stat sbuf;
-  errno = NIL;			/* zap error */
-				/* INBOX is always accepted */
-  if (!strcmp (ucase (strcpy (tmp,name)),"INBOX")) return T;
+  errno = EINVAL;		/* assume invalid argument */
 				/* if file, get its status */
-  if (*name != '{' && mailboxfile (tmp,name) && (stat (tmp,&sbuf) == 0)) {
-				/* allow empty file */
-    if (sbuf.st_size == 0) return LONGT;
-    if ((fd = open (tmp,O_BINARY|O_RDONLY,NIL)) >= 0 && read (fd,tmp,64) >= 0){
-      close (fd);		/* close the file */
-      if ((s = strchr (tmp,'\015')) && s[1] == '\012') {
+  if ((*name != '{') && !((*name == '*') && (name[1] == '{')) &&
+      mailboxfile (tmp,name) && !stat (tmp,&sbuf)) {
+    if (!sbuf.st_size)errno = 0;/* empty file */
+    else if ((fd = open (tmp,O_BINARY|O_RDONLY,NIL)) >= 0) {
+      memset (tmp,'\0',MAILTMPLEN);
+      if ((read (fd,tmp,64) >= 0) && (s = strchr (tmp,'\015')) &&
+	  (s[1] == '\012')) {	/* valid format? */
 	*s = '\0';		/* tie off header */
 				/* must begin with dd-mmm-yy" */
-	  if (((tmp[2] == '-' && tmp[6] == '-') ||
-	       (tmp[1] == '-' && tmp[5] == '-')) &&
-	      (s = strchr (tmp+20,',')) && strchr (s+2,';')) return LONGT;
+	ret = (((tmp[2] == '-' && tmp[6] == '-') ||
+		(tmp[1] == '-' && tmp[5] == '-')) &&
+	       (s = strchr (tmp+20,',')) && strchr (s+2,';')) ? T : NIL;
       }
+      else errno = -1;		/* bogus format */
+      close (fd);		/* close the file */
     }
   }
-  return NIL;			/* failed miserably */
+  return ret;			/* return what we should */
 }
 
 
@@ -186,9 +189,7 @@ void dawz_find_all (MAILSTREAM *stream,char *pat)
 
 long dawz_subscribe (MAILSTREAM *stream,char *mailbox)
 {
-  char tmp[MAILTMPLEN];
-  if (mailboxfile (tmp,mailbox)) return sm_subscribe (mailbox);
-  else return dawz_badname (tmp,mailbox);
+  return dummy_subscribe (stream,mailbox);
 }
 
 
@@ -200,9 +201,7 @@ long dawz_subscribe (MAILSTREAM *stream,char *mailbox)
 
 long dawz_unsubscribe (MAILSTREAM *stream,char *mailbox)
 {
-  char tmp[MAILTMPLEN];
-  if (mailboxfile (tmp,mailbox)) return sm_unsubscribe (mailbox);
-  else return dawz_badname (tmp,mailbox);
+  return dummy_unsubscribe (stream,mailbox);
 }
 
 
@@ -225,17 +224,7 @@ long dawz_subscribe_bboard (MAILSTREAM *stream,char *mailbox)
 
 long dawz_create (MAILSTREAM *stream,char *mailbox)
 {
-  char tmp[MAILTMPLEN];
-  int fd;
-  if (!mailboxfile (tmp,mailbox)) return dawz_badname (tmp,mailbox);
-  if ((fd = open (tmp,O_WRONLY|O_CREAT|O_EXCL,S_IREAD|S_IWRITE)) < 0) {
-				/* failed */
-    sprintf (tmp,"Can't create mailbox %s: %s",mailbox,strerror (errno));
-    mm_log (tmp,ERROR);
-    return NIL;
-  }
-  close (fd);			/* close the file */
-  return LONGT;			/* return success */
+  return dummy_create (stream,mailbox);
 }
 
 
@@ -247,7 +236,7 @@ long dawz_create (MAILSTREAM *stream,char *mailbox)
 
 long dawz_delete (MAILSTREAM *stream,char *mailbox)
 {
-  return dawz_rename (stream,mailbox,NIL);
+  return dummy_delete (stream,mailbox);
 }
 
 
@@ -260,18 +249,7 @@ long dawz_delete (MAILSTREAM *stream,char *mailbox)
 
 long dawz_rename (MAILSTREAM *stream,char *old,char *new)
 {
-  char tmp[MAILTMPLEN],file[MAILTMPLEN],lock[MAILTMPLEN],lockx[MAILTMPLEN];
-				/* make file name */
-  if (!mailboxfile (file,old)) return dawz_badname (tmp,old);
-  if (new && !mailboxfile (tmp,new)) return dawz_badname (tmp,new);
-				/* do the rename or delete operation */
-  if (new ? rename (file,tmp) : unlink (file)) {
-    sprintf (tmp,"Can't %s mailbox %s: %s",new ? "rename" : "delete",old,
-	     strerror (errno));
-    mm_log (tmp,ERROR);
-    return NIL;
-  }
-  return LONGT;			/* return success */
+  return dummy_rename (stream,old,new);
 }
 
 /* Dawz mail open
@@ -299,7 +277,7 @@ MAILSTREAM *dawz_open (MAILSTREAM *stream)
   }
   if (!mailboxfile (tmp,stream->mailbox))
     return (MAILSTREAM *) dawz_badname (tmp,stream->mailbox);
-  if (((fd = open (tmp,O_BINARY|(stream->readonly ? O_RDONLY:O_RDWR),NIL)) < 0)
+  if (((fd = open (tmp,O_BINARY|(stream->rdonly ? O_RDONLY:O_RDWR),NIL)) < 0)
       && (strcmp (ucase (stream->mailbox),"INBOX") ||
 	  ((fd = open (tmp,O_BINARY|O_RDWR|O_CREAT|O_EXCL,S_IREAD|S_IWRITE))
 	   < 0))) {		/* open, possibly creating INBOX */
@@ -499,12 +477,11 @@ char *dawz_fetchheader (MAILSTREAM *stream,long msgno)
 {
   unsigned long hdrsize;
   unsigned long hdrpos = dawz_header (stream,msgno,&hdrsize);
-				/* set default gets routine */
-  if (!mailgets) mailgets = mm_gets;
+  mailgets_t mg = (mailgets_t) mail_parameters (NIL,GET_GETS,NIL);
   if (stream->text) fs_give ((void **) &stream->text);
 				/* get to header position */
   lseek (LOCAL->fd,hdrpos,SEEK_SET);
-  return stream->text = (*mailgets) (dawz_read,stream,hdrsize);
+  return stream->text = (mg ? *mg : mm_gets) (dawz_read,stream,hdrsize);
 }
 
 
@@ -519,8 +496,7 @@ char *dawz_fetchtext (MAILSTREAM *stream,long msgno)
   unsigned long hdrsize;
   unsigned long hdrpos = dawz_header (stream,msgno,&hdrsize);
   unsigned long textsize = mail_elt (stream,msgno)->rfc822_size - hdrsize;
-				/* set default gets routine */
-  if (!mailgets) mailgets = mm_gets;
+  mailgets_t mg = (mailgets_t) mail_parameters (NIL,GET_GETS,NIL);
   if (stream->text) fs_give ((void **) &stream->text);
 				/* mark message as seen */
   mail_elt (stream,msgno)->seen = T;
@@ -528,7 +504,7 @@ char *dawz_fetchtext (MAILSTREAM *stream,long msgno)
   dawz_update_status (stream,msgno);
 				/* get to text position */
   lseek (LOCAL->fd,hdrpos + hdrsize,SEEK_SET);
-  return stream->text = (*mailgets) (dawz_read,stream,textsize);
+  return stream->text = (mg ? *mg : mm_gets) (dawz_read,stream,textsize);
 }
 
 /* Dawz fetch message body as a structure
@@ -548,8 +524,7 @@ char *dawz_fetchbody (MAILSTREAM *stream,long m,char *s,unsigned long *len)
   unsigned long offset = 0;
   unsigned long hdrpos = dawz_header (stream,m,&base);
   MESSAGECACHE *elt = mail_elt (stream,m);
-				/* set default gets routine */
-  if (!mailgets) mailgets = mm_gets;
+  mailgets_t mg = (mailgets_t) mail_parameters (NIL,GET_GETS,NIL);
   if (stream->text) fs_give ((void **) &stream->text);
 				/* make sure have a body */
   if (!(dawz_fetchstructure (stream,m,&b) && b && s && *s &&
@@ -581,7 +556,8 @@ char *dawz_fetchbody (MAILSTREAM *stream,long m,char *s,unsigned long *len)
   elt->seen = T;		/* mark message as seen */
   dawz_update_status (stream,m);/* recalculate status */
   lseek (LOCAL->fd,hdrpos + base + offset,SEEK_SET);
-  return stream->text = (*mailgets) (dawz_read,stream,*len = b->size.bytes);
+  return stream->text =
+    (mg ? *mg : mm_gets) (dawz_read,stream,*len = b->size.bytes);
 }
 
 /* Dawz mail read
@@ -841,7 +817,7 @@ void dawz_expunge (MAILSTREAM *stream)
   char tmp[MAILTMPLEN];
 				/* do nothing if stream dead */
   if (!dawz_ping (stream)) return;
-  if (stream->readonly) {	/* won't do on readonly files! */
+  if (stream->rdonly) {		/* won't do on readonly files! */
     mm_log ("Expunge ignored on readonly mailbox",WARN);
     return;
   }
@@ -935,28 +911,29 @@ long dawz_append (MAILSTREAM *stream,char *mailbox,char *flags,char *date,
   int fd,zone;
   char tmp[MAILTMPLEN];
   MESSAGECACHE elt;
-  time_t ti;
-  struct tm *t;
   long i;
   long size = SIZE (message);
   short f = dawz_getflags (stream,flags);
   if (date) {			/* want to preserve date? */
 				/* yes, parse date into an elt */
     if (!mail_parse_date (&elt,date)) {
-      mm_log ("Bad date in append",ERROR);
+      sprintf (tmp,"Bad date in append: %s",date);
+      mm_log (tmp,ERROR);
       return NIL;
     }
   }
-  tzset ();			/* initialize timezone stuff */
-  if (!dawz_isvalid (mailbox)) {/* make sure valid mailbox */
-    if (errno == ENOENT)
-      mm_notify (stream,"[TRYCREATE] Must create mailbox before append",
-		 (long) NIL);
-    else if (mailboxfile (tmp,mailbox)) {
-      sprintf (tmp,"Not a Dawz-format mailbox: %s",mailbox);
-      mm_log (tmp,ERROR);
-    }
-    else dawz_badname (tmp,mailbox);
+  if (!dawz_isvalid (mailbox)) switch (errno) {
+  case ENOENT:			/* no such file? */
+    mm_notify (stream,"[TRYCREATE] Must create mailbox before append",
+	       (long) NIL);
+    return NIL;
+  case 0:			/* merely empty file? */
+    break;
+  case EINVAL:			/* name is bogus */
+    return dawz_badname (tmp,mailbox);
+  default:			/* file exists, but not valid format */
+    sprintf (tmp,"Not a Dawz-format mailbox: %s",mailbox);
+    mm_log (tmp,ERROR);
     return NIL;
   }
 				/* open the destination */
@@ -968,13 +945,9 @@ long dawz_append (MAILSTREAM *stream,char *mailbox,char *flags,char *date,
   }
   mm_critical (stream);		/* go critical */
   fstat (fd,&sbuf);		/* get current file size */
-  ti = time (0);		/* get time now */
-  t = localtime (&ti);		/* output local time */
   zone = -(((int)timezone)/ 60);/* get timezone from TZ environment stuff */
   if (date) mail_date(tmp,&elt);/* use date if given */
-  else sprintf (tmp,"%2d-%s-%d %02d:%02d:%02d %+03d%02d",
-		t->tm_mday,months[t->tm_mon],t->tm_year+1900,
-		t->tm_hour,t->tm_min,t->tm_sec,zone/60,abs (zone) % 60);
+  else internal_date (tmp);	/* else use time now */
 				/* add remainder of header */
   sprintf (tmp + strlen (tmp),",%ld;0000000000%02o\015\012",size,f);
 
@@ -1097,15 +1070,16 @@ long dawz_parse (MAILSTREAM *stream)
 {
   struct stat sbuf;
   MESSAGECACHE *elt = NIL;
-  char c,*s,*t;
-  char tmp[MAILTMPLEN];
+  char c,*s,*t,*x;
+  char lbuf[65],tmp[MAILTMPLEN];
   long i;
   long curpos = LOCAL->filesize;
   long nmsgs = stream->nmsgs;
   long recent = stream->recent;
   fstat (LOCAL->fd,&sbuf);	/* get status */
   if (sbuf.st_size < curpos) {	/* sanity check */
-    mm_log ("Mailbox shrank!",ERROR);
+    sprintf (tmp,"Mailbox shrank from %ld to %ld!",curpos,sbuf.st_size);
+    mm_log (tmp,ERROR);
     dawz_close (stream);
     return NIL;
   }
@@ -1113,32 +1087,43 @@ long dawz_parse (MAILSTREAM *stream)
   while (i = sbuf.st_size - curpos) {
 				/* get to that position in the file */
     lseek (LOCAL->fd,curpos,SEEK_SET);
-    if (!((read (LOCAL->fd,tmp,64) > 0)
-	  && (s = strchr (tmp,'\015')) && (s[1] == '\012'))) {
-      mm_log ("Unable to read internal header line",ERROR);
+    if ((i = read (LOCAL->fd,lbuf,64)) <= 0) {
+      sprintf (tmp,"Unable to read internal header at %ld, size = %ld: %s",
+	       curpos,sbuf.st_size,i ? strerror (errno) : "no data read");
+      mm_log (tmp,ERROR);
+      dawz_close (stream);
+      return NIL;
+    }
+    lbuf[i] = '\0';		/* tie off buffer just in case */
+    if (!((s = strchr (lbuf,'\015')) && (s[1] == '\012'))) {
+      sprintf (tmp,"Unable to find end of line at %ld in %ld bytes, text: %s",
+	       curpos,i,lbuf);
+      mm_log (tmp,ERROR);
       dawz_close (stream);
       return NIL;
     }
     *s = '\0';			/* tie off header line */
-    i = (s + 2) - tmp;		/* note start of text offset */
-    if (!((s = strchr (tmp,',')) && (t = strchr (s+1,';')))) {
-      mm_log ("Unable to parse internal header line",ERROR);
+    i = (s + 2) - lbuf;		/* note start of text offset */
+    if (!((s = strchr (lbuf,',')) && (t = strchr (s+1,';')))) {
+      sprintf (tmp,"Unable to parse internal header at %ld: %s",curpos,lbuf);
+      mm_log (tmp,ERROR);
       dawz_close (stream);
       return NIL;
     }
     *s++ = '\0'; *t++ = '\0';	/* tie off fields */
 				/* intantiate an elt for this message */
-    elt = mail_elt (stream,++nmsgs);
+    (elt = mail_elt (stream,++nmsgs))->valid = T;
     elt->data1 = curpos;	/* note file offset of header */
     elt->data2 = i << 24;	/* as well as offset from header of message */
 				/* parse the header components */
-    if (!(mail_parse_date (elt,tmp) &&
-	  (elt->rfc822_size = strtol (s,&s,10)) && (!(s && *s)) &&
+    if (!(mail_parse_date (elt,lbuf) &&
+	  (elt->rfc822_size = strtol (x = s,&s,10)) && (!(s && *s)) &&
 	  isdigit (t[0]) && isdigit (t[1]) && isdigit (t[2]) &&
 	  isdigit (t[3]) && isdigit (t[4]) && isdigit (t[5]) &&
 	  isdigit (t[6]) && isdigit (t[7]) && isdigit (t[8]) &&
 	  isdigit (t[9]) && isdigit (t[10]) && isdigit (t[11]) && !t[12])) {
-      mm_log ("Unable to parse internal header line components",ERROR);
+      sprintf (tmp,"Unable to parse internal header elements at %ld: %s,%s;%s",
+	       curpos,lbuf,x,t);
       dawz_close (stream);
       return NIL;
     }
@@ -1177,15 +1162,18 @@ long dawz_copy_messages (MAILSTREAM *stream,char *mailbox)
   unsigned long i,j,k;
   long ret = LONGT;
   int fd;
-  if (!dawz_isvalid (mailbox)) {/* make sure valid mailbox */
-    if (errno == ENOENT)
-      mm_notify (stream,"[TRYCREATE] Must create mailbox before append",
-		 (long) NIL);
-    else if (mailboxfile (tmp,mailbox)) {
-      sprintf (tmp,"Not a Dawz-format mailbox: %s",mailbox);
-      mm_log (tmp,ERROR);
-    }
-    else dawz_badname (tmp,mailbox);
+  if (!dawz_isvalid (mailbox)) switch (errno) {
+  case ENOENT:			/* no such file? */
+    mm_notify (stream,"[TRYCREATE] Must create mailbox before append",
+	       (long) NIL);
+    return NIL;
+  case 0:			/* merely empty file? */
+    break;
+  case EINVAL:			/* name is bogus */
+    return dawz_badname (tmp,mailbox);
+  default:			/* file exists, but not valid format */
+    sprintf (tmp,"Not a Dawz-format mailbox: %s",mailbox);
+    mm_log (tmp,ERROR);
     return NIL;
   }
 				/* open the destination */
@@ -1231,7 +1219,7 @@ void dawz_update_status (MAILSTREAM *stream,long msgno)
   char tmp[MAILTMPLEN];
   MESSAGECACHE *elt = mail_elt (stream,msgno);
   unsigned long j,k = 0;
-  if (stream->readonly) return;	/* not if readonly you don't */
+  if (stream->rdonly) return;	/* not if readonly you don't */
   j = elt->user_flags;		/* get user flags */
 				/* reverse bits (dontcha wish we had CIRC?) */
   while (j) k |= 1 << 29 - find_rightmost_bit (&j);
@@ -1487,28 +1475,30 @@ search_t dawz_search_flag (search_t f,long *n,MAILSTREAM *stream)
 
 search_t dawz_search_string (search_t f,char **d,long *n)
 {
+  char *end = " ";
   char *c = strtok (NIL,"");	/* remainder of criteria */
-  if (c) {			/* better be an argument */
-    switch (*c) {		/* see what the argument is */
-    case '\0':			/* catch bogons */
-    case ' ':
-      return NIL;
-    case '"':			/* quoted string */
-      if (!(strchr (c+1,'"') && (*d = strtok (c,"\"")) && (*n = strlen (*d))))
-	return NIL;
-      break;
-    case '{':			/* literal string */
-      *n = strtol (c+1,&c,10);	/* get its length */
-      if (*c++ != '}' || *c++ != '\015' || *c++ != '\012' ||
-	  *n > strlen (*d = c)) return NIL;
-      c[*n] = DELIM;		/* write new delimiter */
-      strtok (c,DELMS);		/* reset the strtok mechanism */
-      break;
-    default:			/* atomic string */
-      *n = strlen (*d = strtok (c," "));
+  if (!c) return NIL;		/* missing argument */
+  switch (*c) {			/* see what the argument is */
+  case '{':			/* literal string */
+    *n = strtol (c+1,d,10);	/* get its length */
+    if ((*(*d)++ == '}') && (*(*d)++ == '\015') && (*(*d)++ == '\012') &&
+	(!(*(c = *d + *n)) || (*c == ' '))) {
+      char e = *--c;
+      *c = DELIM;		/* make sure not a space */
+      strtok (c," ");		/* reset the strtok mechanism */
+      *c = e;			/* put character back */
       break;
     }
-    return f;
+  case '\0':			/* catch bogons */
+  case ' ':
+    return NIL;
+  case '"':			/* quoted string */
+    if (strchr (c+1,'"')) end = "\"";
+    else return NIL;
+  default:			/* atomic string */
+    if (*d = strtok (c,end)) *n = strlen (*d);
+    else return NIL;
+    break;
   }
-  else return NIL;
+  return f;
 }
