@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 November 1989
- * Last Edited:	14 July 1996
+ * Last Edited:	15 October 1996
  *
  * Copyright 1996 by the University of Washington
  *
@@ -1208,17 +1208,17 @@ long mail_parse_date (MESSAGECACHE *elt,char *s)
     elt->hours = d; elt->minutes = m; elt->seconds = y;
     switch (*s) {		/* time zone specifier? */
     case ' ':			/* numeric time zone */
+      while (s[1] == ' ') s++;	/* slurp extra whitespace */
       if (!isalpha (s[1])) {	/* treat as '-' case if alphabetic */
 				/* test for sign character */
 	if ((elt->zoccident = (*++s == '-')) || (*s == '+')) s++;
 				/* validate proper timezone */
-	if (!(isdigit (*s) && isdigit (s[1]) && isdigit (s[2]) &&
-	      isdigit (s[3])) || (s[4] && (s[4] != ' '))) return NIL;
-	elt->zhours = (*s - '0') * 10 + (s[1] - '0');
-	elt->zminutes = (s[2] - '0') * 10 + (s[3] - '0');
-	break;
+	if (isdigit(*s) && isdigit(s[1]) && isdigit(s[2]) && isdigit(s[3])) {
+	  elt->zhours = (*s - '0') * 10 + (s[1] - '0');
+	  elt->zminutes = (s[2] - '0') * 10 + (s[3] - '0');
+	}
+	return T;		/* all done! */
       }
-      while (s[1] == ' ') s++;	/* slurp extra whitespace */
 				/* falls through */
 
     case '-':			/* symbolic time zone */
@@ -1227,7 +1227,7 @@ long mail_parse_date (MESSAGECACHE *elt,char *s)
 	ms -= 'A'; ms *= 1024;	/* yes, make compressed three-byte form */
 	ms += ((*s++ - 'A') * 32);
 	if (*s) ms += *s++ - 'A';
-	if (*s) ms = 'Z';	/* more than three characters */
+	if (*s) ms = '\0';	/* more than three characters */
       }
       /* This is not intended to be a comprehensive list of all possible
        * timezone strings.  Such a list would be impractical.  Rather, this
