@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	15 May 1993
- * Last Edited:	13 March 1996
+ * Last Edited:	15 October 1996
  *
  * Copyright 1996 by the University of Washington
  *
@@ -42,7 +42,6 @@ extern int errno;		/* just in case */
 #include "mail.h"
 #include "osdep.h"
 #include <pwd.h>
-#include <sys/file.h>
 #include <sys/stat.h>
 #include "mmdf.h"
 #include "rfc822.h"
@@ -276,9 +275,9 @@ long mmdf_rename (MAILSTREAM *stream,char *old,char *new)
   char tmp[MAILTMPLEN],file[MAILTMPLEN],lock[MAILTMPLEN],lockx[MAILTMPLEN];
   int fd,ld;
 				/* get the c-client lock */
-  if ((ld = open (lockname (lock,dummy_file (file,old)),O_RDWR|O_CREAT,
-		  (int) mail_parameters (NIL,GET_LOCKPROTECTION,NIL)))
-      < 0) {
+  if (!lockname (lock,dummy_file (file,old))) return NIL;
+  if ((ld = open (lock,O_RDWR|O_CREAT,
+		  (int) mail_parameters (NIL,GET_LOCKPROTECTION,NIL))) < 0) {
     sprintf (tmp,"Can't get lock for mailbox %s: %s",old,strerror (errno));
     mm_log (tmp,ERROR);
     return NIL;
@@ -349,9 +348,8 @@ MAILSTREAM *mmdf_open (MAILSTREAM *stream)
    * new name, and mmdf_close() needs the old name.
    */
   LOCAL->name = cpystr (tmp);	/* local copy for recycle case */
-				/* build name of our lock file */
-  LOCAL->lname = cpystr (lockname (tmp,LOCAL->name));
   LOCAL->ld = NIL;		/* no state locking yet */
+  LOCAL->lname = NIL;
   LOCAL->filesize = 0;		/* initialize file information */
   LOCAL->filetime = 0;
   LOCAL->msgs = NIL;		/* no cache yet */
@@ -363,8 +361,9 @@ MAILSTREAM *mmdf_open (MAILSTREAM *stream)
 				/* make lock for read/write access */
   if (!stream->rdonly) while (retry) {
 				/* get a new file handle each time */
-    if ((fd = open (LOCAL->lname,O_RDWR|O_CREAT,
-		    (int) mail_parameters (NIL,GET_LOCKPROTECTION,NIL))) < 0) {
+    if (!lockname (tmp,LOCAL->name) ||
+	((fd = open (tmp,O_RDWR|O_CREAT,
+		    (int) mail_parameters (NIL,GET_LOCKPROTECTION,NIL))) < 0)){
       mm_log ("Can't open mailbox lock, access is readonly",WARN);
       retry = 0;		/* give up */
     }
@@ -388,7 +387,8 @@ MAILSTREAM *mmdf_open (MAILSTREAM *stream)
       }
     }
     else {			/* got the lock, nobody else can alter state */
-      LOCAL->ld = fd;		/* note lock's fd */
+      LOCAL->ld = fd;		/* note lock's fd and name */
+      LOCAL->lname = cpystr (tmp);
 				/* make sure mode OK (don't use fchmod()) */
       chmod (LOCAL->lname,(int) mail_parameters (NIL,GET_LOCKPROTECTION,NIL));
       if (stream->silent) i = 0;/* silent streams won't accept KOD */
@@ -1044,8 +1044,7 @@ int mmdf_lock (char *file,int flags,int mode,char *lock,int op)
   time_t t;
   struct stat sb;
 				/* build lock filename */
-  strcat (dummy_file (lock,file),".lock");
-  do {				/* until OK or out of tries */
+  if (chk_notsymlink (strcat (dummy_file (lock,file),".lock"),NIL)) do {
     t = time (0);		/* get the time now */
 #ifdef NFSKLUDGE
   /* SUN-OS had an NFS, As kludgy as an albatross;
@@ -1534,8 +1533,9 @@ int mmdf_extend (MAILSTREAM *stream,int fd,char *error)
     elt = mail_elt (stream,++i);/* get elt, increment message */
 				/* if not expunging, or not deleted */
     if (!(error && elt->deleted))
-      f += (m->header - m->internal) + m->headersize + m->bodysize + 1 +
-	sizeof (STATUS) - (elt->seen+elt->deleted+elt->flagged+elt->answered);
+      f += MMDFHDRLEN + (m->header - m->internal) + m->headersize +
+	sizeof (STATUS) - (elt->seen+elt->deleted+elt->flagged+elt->answered) +
+	  m->bodysize + 1 + MMDFHDRLEN;
   }
   mm_critical (stream);		/* go critical */
 				/* return now if file large enough */
@@ -1697,7 +1697,8 @@ int mmdf_write_message (int fd,FILECACHE *m)
     iov[i].iov_base=m->internal;/* pointer/counter to headers */
     iov[i].iov_len = siz;	/* length of internal + message headers */
 				/* suppress extra newline if present */
-    if ((iov[i].iov_base)[iov[i].iov_len - 2] == '\n') iov[i++].iov_len--;
+    if (((char *) iov[i].iov_base)[iov[i].iov_len - 2] == '\n')
+      iov[i++].iov_len--;
     else i++;			/* unlikely but... */
   }
   iov[i].iov_base = m->status;	/* pointer/counter to status */
@@ -1705,9 +1706,11 @@ int mmdf_write_message (int fd,FILECACHE *m)
   if (m->bodysize) {		/* only if a non-empty body */
     iov[i].iov_base = m->body;	/* pointer/counter to text body */
     iov[i++].iov_len = m->bodysize;
+    if (m->body[m->bodysize - 1] != '\n') {
+      iov[i].iov_base = "\n";	/* pointer/counter to extra newline */
+      iov[i++].iov_len = 1;
+    }
   }
-  iov[i].iov_base = "\n";	/* pointer/counter to extra newline */
-  iov[i++].iov_len = 1;
   iov[i].iov_base = mmdfhdr;	/* write MMDF trailer */
   iov[i++].iov_len = MMDFHDRLEN;
   return writev (fd,iov,i);
