@@ -10,9 +10,9 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 May 1990
- * Last Edited:	6 October 1994
+ * Last Edited:	15 October 1996
  *
- * Copyright 1994 by the University of Washington
+ * Copyright 1996 by the University of Washington
  *
  *  Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -41,7 +41,6 @@ extern int errno;		/* just in case */
 #include "mail.h"
 #include "osdep.h"
 #include <pwd.h>
-#include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include "mtx.h"
@@ -120,9 +119,9 @@ int mtx_isvalid (char *name,char *tmp)
   errno = EINVAL;		/* assume invalid argument */
 				/* if file, get its status */
   if ((*name != '{') && !((*name == '*') && (name[1] == '{')) &&
-      !stat (mtx_file (file,name),&sbuf)) {
+      (s = mtx_file (file,name)) && !stat (s,&sbuf)) {
     if (!sbuf.st_size) {	/* allow empty file if INBOX */
-      if (!strcmp (ucase (strcpy (tmp,name)),"INBOX")) ret = T;
+      if ((s = mailboxfile (tmp,name)) && !*s) ret = T;
       else errno = 0;		/* empty file */
     }
     else if ((fd = open (file,O_RDONLY,NIL)) >= 0) {
@@ -372,6 +371,7 @@ MAILSTREAM *mtx_open (MAILSTREAM *stream)
   stream->nmsgs = stream->recent = 0;
   if (mtx_ping (stream) && !stream->nmsgs)
     mm_log ("Mailbox is empty",(long) NIL);
+  if (!LOCAL) return NIL;	/* failure if stream died */
   return stream;		/* return stream to caller */
 }
 
@@ -912,7 +912,7 @@ void mtx_snarf (MAILSTREAM *stream)
 		(fFLAGGED * elt->flagged) + (fANSWERED * elt->answered));
 	iov[0].iov_len = strlen (iov[0].iov_base);
 				/* copy message to new mailbox */
-	if (writev (LOCAL->fd,iov,2) < 0) {
+	if ((writev (LOCAL->fd,iov,3) < 0) || fsync (LOCAL->fd)) {
 	  sprintf (LOCAL->buf,"Can't copy new mail: %s",strerror (errno));
 	  mm_log (LOCAL->buf,ERROR);
 	  ftruncate (LOCAL->fd,sbuf.st_size);
@@ -920,7 +920,6 @@ void mtx_snarf (MAILSTREAM *stream)
 	}
 	fs_give ((void **) &iov[1].iov_base);
       }
-      fsync (LOCAL->fd);	/* make sure changes dumped out */
       fstat (LOCAL->fd,&sbuf);	/* yes, get current file size */
       LOCAL->filetime = sbuf.st_mtime;
       if (r) {			/* delete all the messages we copied */
@@ -942,6 +941,7 @@ void mtx_snarf (MAILSTREAM *stream)
 void mtx_expunge (MAILSTREAM *stream)
 {
   struct stat sbuf;
+  off_t pos = 0;
   int ld;
   unsigned long i = 1;
   unsigned long j,k,m,recent;
@@ -998,16 +998,24 @@ void mtx_expunge (MAILSTREAM *stream)
 	m = min (k,LOCAL->buflen);
 	lseek (LOCAL->fd,j,L_SET);
 	read (LOCAL->fd,LOCAL->buf,(unsigned int) m);
-				/* write to destination position */
-	lseek (LOCAL->fd,j - delta,L_SET);
+	pos = j - delta;	/* write to destination position */
+	lseek (LOCAL->fd,pos,L_SET);
 	write (LOCAL->fd,LOCAL->buf,(unsigned int) m);
+	pos += m;		/* new position */
 	j += m;			/* next chunk, perhaps */
       } while (k -= m);		/* until done */
       elt->data1 -= delta;	/* note the new address of this text */
     }
+    else pos = elt->data1 + k;	/* preserved but no deleted messages */
   }
   if (n) {			/* truncate file after last message */
-    ftruncate (LOCAL->fd,LOCAL->filesize -= delta);
+    if (pos != (LOCAL->filesize -= delta)) {
+      sprintf (LOCAL->buf,"Calculated size mismatch %ld != %ld, delta = %ld",
+	       pos,LOCAL->filesize,delta);
+      mm_log (LOCAL->buf,WARN);
+      LOCAL->filesize = pos;	/* fix it then */
+    }
+    ftruncate (LOCAL->fd,LOCAL->filesize);
     sprintf (LOCAL->buf,"Expunged %ld messages",n);
 				/* output the news */
     mm_log (LOCAL->buf,(long) NIL);
@@ -1097,7 +1105,8 @@ long mtx_append (MAILSTREAM *stream,char *mailbox,char *flags,char *date,
   if (date) {			/* want to preserve date? */
 				/* yes, parse date into an elt */
     if (!mail_parse_date (&elt,date)) {
-      mm_log ("Bad date in append",ERROR);
+      sprintf (tmp,"Bad date in append: %s",date);
+      mm_log (tmp,ERROR);
       return NIL;
     }
   }
@@ -1140,13 +1149,13 @@ long mtx_append (MAILSTREAM *stream,char *mailbox,char *flags,char *date,
 				/* add remainder of header */
   sprintf (tmp+26,",%ld;%010lo%02o\015\012",size,uf,f);
 				/* write header */
-  if ((write (fd,tmp,strlen (tmp)) < 0) || ((write (fd,s,size)) < 0)) {
+  if ((write (fd,tmp,strlen (tmp)) < 0) || ((write (fd,s,size)) < 0) ||
+      fsync (fd)) {
     sprintf (tmp,"Message append failed: %s",strerror (errno));
     mm_log (tmp,ERROR);
     ftruncate (fd,sbuf.st_size);
     ret = NIL;
   }
-  fsync (fd);			/* force out the update */
   tp[0] = sbuf.st_atime;	/* preserve atime and mtime */
   tp[1] = sbuf.st_mtime;
   utime (file,tp);		/* set the times */
@@ -1181,19 +1190,17 @@ int mtx_lock (int fd,char *lock,int op)
 {
   int ld;
   struct stat sbuf;
-				/* get data for this file */
-  if (fstat (fd,&sbuf)) return -1;
+  if (!fstat (fd,&sbuf)) {	/* get data for this file */
 				/* make temporary file name */
-#ifdef __linux__
-  sprintf (lock,"/var/lock/.%hx.%lx",sbuf.st_dev,sbuf.st_ino);
-#else
-  sprintf (lock,"/tmp/.%hx.%lx",sbuf.st_dev,sbuf.st_ino);
-#endif /* __linux__ */
-  if ((ld = open (lock,O_RDWR|O_CREAT,
-		  (int) mail_parameters (NIL,GET_LOCKPROTECTION,NIL))) < 0)
-    return NIL;
-  flock (ld,op);		/* get this lock */
-  return ld;			/* return locking file descriptor */
+    sprintf (lock,"/tmp/.%hx.%lx",sbuf.st_dev,sbuf.st_ino);
+    if (chk_notsymlink (lock,T) &&
+	((ld = open (lock,O_RDWR|O_CREAT,
+		     (int)mail_parameters(NIL,GET_LOCKPROTECTION,NIL))) >= 0)){
+      flock (ld,op);		/* get this lock */
+      return ld;		/* return locking file descriptor */
+    }
+  }
+  return -1;
 }
 
 
@@ -1226,14 +1233,16 @@ unsigned long mtx_size (MAILSTREAM *stream,long m)
 /* Mtx mail generate file string
  * Accepts: temporary buffer to write into
  *	    mailbox name string
- * Returns: local file string
+ * Returns: local file string or NIL if failure
  */
 
 char *mtx_file (char *dst,char *name)
 {
+  char tmp[MAILTMPLEN];
   char *s = mailboxfile (dst,name);
 				/* return our standard inbox */
-  return s ? s : mailboxfile (dst,"INBOX.MTX");
+  return (s && !*s) ? mailboxfile (dst,mtx_isvalid ("~/INBOX.MTX",tmp) ?
+				   "INBOX.MTX" : "mail.txt") : s;
 }
 
 /* Parse flag list
@@ -1334,7 +1343,7 @@ long mtx_parse (MAILSTREAM *stream)
     }
     LOCAL->buf[i] = '\0';	/* tie off buffer just in case */
     if (!((s = strchr (LOCAL->buf,'\015')) && (s[1] == '\012'))) {
-      sprintf (tmp,"Unable to find end of line at %ld in %ld bytes: %s",
+      sprintf (tmp,"Unable to find CRLF at %ld in %ld bytes, text: %s",
 	       curpos,i,LOCAL->buf);
       mm_log (tmp,ERROR);
       mtx_close (stream);
@@ -1459,7 +1468,7 @@ long mtx_copy_messages (MAILSTREAM *stream,char *mailbox)
       do {			/* read from source position */
 	j = min (k,LOCAL->buflen);
 	read (LOCAL->fd,LOCAL->buf,(unsigned int) j);
-	if (write (fd,LOCAL->buf,(unsigned int) j) < 0) {
+	if ((write (fd,LOCAL->buf,(unsigned int) j) < 0) || fsync (fd)) {
 	  sprintf (LOCAL->buf,"Unable to write message: %s",strerror (errno));
 	  mm_log (LOCAL->buf,ERROR);
 	  ftruncate (fd,sbuf.st_size);
@@ -1469,7 +1478,6 @@ long mtx_copy_messages (MAILSTREAM *stream,char *mailbox)
 	}
       } while (k -= j);		/* until done */
     }
-  fsync (fd);			/* force out the update */
   tp[0] = sbuf.st_atime;	/* preserve atime and mtime */
   tp[1] = sbuf.st_mtime;
   utime (file,tp);		/* set the times */
