@@ -10,9 +10,9 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	5 January 1993
- * Last Edited:	9 October 1994
+ * Last Edited:	15 May 1996
  *
- * Copyright 1994 by the University of Washington
+ * Copyright 1996 by the University of Washington
  *
  *  Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -40,7 +40,6 @@
 #include <stdio.h>
 #include <errno.h>
 extern int errno;		/* just in case */
-#include <sys/file.h>
 #include <sys/stat.h>
 #include "smtp.h"
 #include "nntp.h"
@@ -171,7 +170,7 @@ void nntp_find_all_bboards (MAILSTREAM *stream,char *pat)
       patx = pat;		/* use entire specification */
     }
 				/* ask server for all active newsgroups */
-    if (!(smtp_send (LOCAL->nntpstream,"LIST","ACTIVE") == NNTPGLIST)) return;
+    if (!(nntp_send (LOCAL->nntpstream,"LIST","ACTIVE") == NNTPGLIST)) return;
 				/* process data until we see final dot */
     while ((s = tcp_getline (LOCAL->nntpstream->tcpstream)) && *s != '.') {
 				/* tie off after newsgroup name */
@@ -314,7 +313,7 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
   }
   if (nstream) {		/* now try to open newsgroup */
     if ((!stream->halfopen) &&	/* open the newsgroup if not halfopen */
-	((smtp_send (nstream,"GROUP",mb.mailbox) != NNTPGOK) ||
+	((nntp_send (nstream,"GROUP",mb.mailbox) != NNTPGOK) ||
 	 ((k = strtol (nstream->reply + 4,&s,10)) < 0) ||
 	 ((i = strtol (s,&s,10)) < 0) || ((j = strtol (s,&s,10)) < 0) ||
 	 (nmsgs = i | j ? 1 + j - i : 0) < 0)) {
@@ -340,8 +339,8 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
       if (nmsgs) {		/* find what messages exist */
 	LOCAL->number = (unsigned long *) fs_get (nmsgs*sizeof(unsigned long));
 	sprintf (tmp,"%ld-%ld",i,j);
-	if ((smtp_send (nstream,"LISTGROUP",mb.mailbox) == NNTPGOK) ||
-	    (smtp_send (nstream,"XHDR Date",tmp) == NNTPHEAD)) {
+	if ((nntp_send (nstream,"LISTGROUP",mb.mailbox) == NNTPGOK) ||
+	    (nntp_send (nstream,"XHDR Date",tmp) == NNTPHEAD)) {
 	  for (i = 0; (s = tcp_getline (nstream->tcpstream)) && strcmp (s,".");
 	       i++) {		/* initialize c-client/NNTP map */
 	    if (i < nmsgs) LOCAL->number[i] = atol (s);
@@ -349,8 +348,9 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
 	  }
 	  if (s) fs_give ((void **) &s);
 	  if (i != nmsgs) {	/* found holes? */
-	    nmsgs = i;		/* set new size */
-	    fs_resize ((void **) &LOCAL->number,nmsgs*sizeof (unsigned long));
+	    if (nmsgs = i)
+	      fs_resize ((void **) &LOCAL->number,nmsgs*sizeof(unsigned long));
+	    else fs_give ((void **) &LOCAL->number);
 	  }
 	}
 				/* assume c-client/NNTP map is entire range */
@@ -368,8 +368,10 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
 				/* read .newsrc entries */
       mail_recent (stream,newsrc_read (LOCAL->name,stream,LOCAL->number));
 				/* notify if empty bboard */
-      if (!(stream->nmsgs || stream->silent))
-	mm_log ("Newsgroup is empty",WARN);
+      if (!(stream->nmsgs || stream->silent)) {
+	sprintf (tmp,"Newsgroup %s is empty",LOCAL->name);
+	mm_log (tmp,WARN);
+      }
     }
   }
   return LOCAL ? stream : NIL;	/* if stream is alive, return to caller */
@@ -492,7 +494,7 @@ char *nntp_fetchheader (MAILSTREAM *stream,long msgno)
   long m = msgno - 1;
   if (!LOCAL->header[m]) {	/* fetch header if don't have already */
     sprintf (tmp,"%ld",LOCAL->number[m]);
-    if (smtp_send (LOCAL->nntpstream,"HEAD",tmp) == NNTPHEAD)
+    if (nntp_send (LOCAL->nntpstream,"HEAD",tmp) == NNTPHEAD)
       LOCAL->header[m] = nntp_slurp (stream);
 				/* failed, mark as deleted */
     else mail_elt (stream,msgno)->deleted = T;
@@ -528,7 +530,7 @@ char *nntp_fetchtext_work (MAILSTREAM *stream,long msgno)
   long m = msgno - 1;
   if (!LOCAL->body[m]) {	/* fetch body if don't have already */
     sprintf (tmp,"%ld",LOCAL->number[m]);
-    if (smtp_send (LOCAL->nntpstream,"BODY",tmp) == NNTPBODY)
+    if (nntp_send (LOCAL->nntpstream,"BODY",tmp) == NNTPBODY)
       LOCAL->body[m] = nntp_slurp (stream);
 				/* failed, mark as deleted */
     else mail_elt (stream,msgno)->deleted = T;
@@ -549,7 +551,10 @@ char *nntp_slurp (MAILSTREAM *stream)
   while (s = tcp_getline (LOCAL->nntpstream->tcpstream)) {
     if (*s == '.') {		/* possible end of text? */
       if (s[1]) t = s + 1;	/* pointer to true start of line */
-      else break;		/* end of data */
+      else {
+	fs_give ((void **) &s);	/* free the line */
+	break;			/* end of data */
+      }
     }
     else t = s;			/* want the entire line */
 				/* ensure have enough room */
@@ -780,9 +785,9 @@ void nntp_search (MAILSTREAM *stream,char *criteria)
 long nntp_ping (MAILSTREAM *stream)
 {
   /* Kludge alert: SMTPSOFTFATAL is 421 which is used in NNTP to mean ``No
-   * next article in this group''.  Hopefully, no NNTP server will choke on
-   * a bogus command. */
-  return (smtp_send (LOCAL->nntpstream,"NOOP",NIL) != SMTPSOFTFATAL);
+   * next article in this group''.  Hopefully, no NNTP server will send this
+   * in response to a STAT */
+  return (nntp_send (LOCAL->nntpstream,"STAT",NIL) != SMTPSOFTFATAL);
 }
 
 
