@@ -10,9 +10,9 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	2 August 1994
- * Last Edited:	28 August 1995
+ * Last Edited:	2 May 1996
  *
- * Copyright 1995 by the University of Washington
+ * Copyright 1996 by the University of Washington
  *
  *  Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -176,7 +176,7 @@ void nntp_find_all_bboards (MAILSTREAM *stream,char *pat)
       patx = pat;		/* use entire specification */
     }
 				/* ask server for all active newsgroups */
-    if (!(smtp_send (LOCAL->nntpstream,"LIST","ACTIVE") == NNTPGLIST)) return;
+    if (!(nntp_send (LOCAL->nntpstream,"LIST","ACTIVE") == NNTPGLIST)) return;
 				/* process data until we see final dot */
     while ((s = tcp_getline (LOCAL->nntpstream->tcpstream)) && *s != '.') {
 				/* tie off after newsgroup name */
@@ -319,7 +319,7 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
   }
   if (nstream) {		/* now try to open newsgroup */
     if ((!stream->halfopen) &&	/* open the newsgroup if not halfopen */
-	((smtp_send (nstream,"GROUP",mb.mailbox) != NNTPGOK) ||
+	((nntp_send (nstream,"GROUP",mb.mailbox) != NNTPGOK) ||
 	 ((k = strtol (nstream->reply + 4,&s,10)) < 0) ||
 	 ((i = strtol (s,&s,10)) < 0) || ((j = strtol (s,&s,10)) < 0) ||
 	 (nmsgs = i | j ? 1 + j - i : 0) < 0)) {
@@ -345,8 +345,8 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
       if (nmsgs) {		/* find what messages exist */
 	LOCAL->number = (unsigned long *) fs_get (nmsgs*sizeof(unsigned long));
 	sprintf (tmp,"%ld-%ld",i,j);
-	if ((smtp_send (nstream,"LISTGROUP",mb.mailbox) == NNTPGOK) ||
-	    (smtp_send (nstream,"XHDR Date",tmp) == NNTPHEAD)) {
+	if ((nntp_send (nstream,"LISTGROUP",mb.mailbox) == NNTPGOK) ||
+	    (nntp_send (nstream,"XHDR Date",tmp) == NNTPHEAD)) {
 	  for (i = 0; (s = tcp_getline (nstream->tcpstream)) && strcmp (s,".");
 	       i++) {		/* initialize c-client/NNTP map */
 	    if (i < nmsgs) LOCAL->number[i] = atol (s);
@@ -354,8 +354,9 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
 	  }
 	  if (s) fs_give ((void **) &s);
 	  if (i != nmsgs) {	/* found holes? */
-	    nmsgs = i;		/* set new size */
-	    fs_resize ((void **) &LOCAL->number,nmsgs*sizeof (unsigned long));
+	    if (nmsgs = i)
+	      fs_resize ((void **) &LOCAL->number,nmsgs*sizeof(unsigned long));
+	    else fs_give ((void **) &LOCAL->number);
 	  }
 	}
 				/* assume c-client/NNTP map is entire range */
@@ -373,8 +374,10 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
 				/* read .newsrc entries */
       mail_recent (stream,newsrc_read (LOCAL->name,stream,LOCAL->number));
 				/* notify if empty bboard */
-      if (!(stream->nmsgs || stream->silent))
-	mm_log ("Newsgroup is empty",WARN);
+      if (!(stream->nmsgs || stream->silent)) {
+	sprintf (tmp,"Newsgroup %s is empty",LOCAL->name);
+	mm_log (tmp,WARN);
+      }
     }
   }
   return LOCAL ? stream : NIL;	/* if stream is alive, return to caller */
@@ -497,7 +500,7 @@ char *nntp_fetchheader (MAILSTREAM *stream,long msgno)
   long m = msgno - 1;
   if (!LOCAL->header[m]) {	/* fetch header if don't have already */
     sprintf (tmp,"%ld",LOCAL->number[m]);
-    if (smtp_send (LOCAL->nntpstream,"HEAD",tmp) == NNTPHEAD)
+    if (nntp_send (LOCAL->nntpstream,"HEAD",tmp) == NNTPHEAD)
       LOCAL->header[m] = nntp_slurp (stream);
 				/* failed, mark as deleted */
     else mail_elt (stream,msgno)->deleted = T;
@@ -533,7 +536,7 @@ char *nntp_fetchtext_work (MAILSTREAM *stream,long msgno)
   long m = msgno - 1;
   if (!LOCAL->body[m]) {	/* fetch body if don't have already */
     sprintf (tmp,"%ld",LOCAL->number[m]);
-    if (smtp_send (LOCAL->nntpstream,"BODY",tmp) == NNTPBODY)
+    if (nntp_send (LOCAL->nntpstream,"BODY",tmp) == NNTPBODY)
       LOCAL->body[m] = nntp_slurp (stream);
 				/* failed, mark as deleted */
     else mail_elt (stream,msgno)->deleted = T;
@@ -790,7 +793,7 @@ long nntp_ping (MAILSTREAM *stream)
   /* Kludge alert: SMTPSOFTFATAL is 421 which is used in NNTP to mean ``No
    * next article in this group''.  Hopefully, no NNTP server will send this
    * in response to a STAT */
-  return (smtp_send (LOCAL->nntpstream,"STAT",NIL) != SMTPSOFTFATAL);
+  return (nntp_send (LOCAL->nntpstream,"STAT",NIL) != SMTPSOFTFATAL);
 }
 
 
