@@ -10,9 +10,9 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	6 June 1994
- * Last Edited:	15 February 1995
+ * Last Edited:	22 October 1996
  *
- * Copyright 1995 by the University of Washington
+ * Copyright 1996 by the University of Washington
  *
  *  Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -870,6 +870,7 @@ long pop3_send_num (MAILSTREAM *stream,char *command,unsigned long n)
 long pop3_send (MAILSTREAM *stream,char *command,char *args)
 {
   char tmp[MAILTMPLEN];
+  if (!LOCAL->tcpstream) return pop3_fake (stream,"No-op dead stream");
 				/* build the complete command */
   if (args) sprintf (tmp,"%s %s",command,args);
   else strcpy (tmp,command);
@@ -888,6 +889,7 @@ long pop3_send (MAILSTREAM *stream,char *command,char *args)
 long pop3_reply (MAILSTREAM *stream)
 {
   char *s;
+  if (!LOCAL->tcpstream) return pop3_fake (stream,"No-op dead stream");
 				/* flush old reply */
   if (LOCAL->response) fs_give ((void **) &LOCAL->response);
   				/* get reply */
