@@ -10,9 +10,9 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 May 1990
- * Last Edited:	15 December 1995
+ * Last Edited:	15 October 1996
  *
- * Copyright 1995 by the University of Washington
+ * Copyright 1996 by the University of Washington
  *
  *  Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -41,7 +41,6 @@ extern int errno;		/* just in case */
 #include "mail.h"
 #include "osdep.h"
 #include <pwd.h>
-#include <sys/file.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include "tenex2.h"
@@ -1211,15 +1210,17 @@ int tenex_lock (int fd,char *lock,int op)
 {
   int ld;
   struct stat sbuf;
-				/* get data for this file */
-  if (fstat (fd,&sbuf)) return -1;
+  if (!fstat (fd,&sbuf)) {	/* get data for this file */
 				/* make temporary file name */
-  sprintf (lock,"/tmp/.%hx.%lx",sbuf.st_dev,sbuf.st_ino);
-  if ((ld = open (lock,O_RDWR|O_CREAT,
-		  (int) mail_parameters (NIL,GET_LOCKPROTECTION,NIL))) < 0)
-    return -1;
-  flock (ld,op);		/* get this lock */
-  return ld;			/* return locking file descriptor */
+    sprintf (lock,"/tmp/.%hx.%lx",sbuf.st_dev,sbuf.st_ino);
+    if (chk_notsymlink (lock,T) &&
+	((ld = open (lock,O_RDWR|O_CREAT,
+		     (int)mail_parameters(NIL,GET_LOCKPROTECTION,NIL))) >= 0)){
+      flock (ld,op);		/* get this lock */
+      return ld;		/* return locking file descriptor */
+    }
+  }
+  return -1;
 }
 
 
