@@ -10,9 +10,9 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	9 May 1991
- * Last Edited:	7 September 1994
+ * Last Edited:	15 May 1996
  *
- * Copyright 1994 by the University of Washington
+ * Copyright 1996 by the University of Washington
  *
  *  Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -41,7 +41,6 @@ extern int errno;		/* just in case */
 #include "mail.h"
 #include "osdep.h"
 #include <pwd.h>
-#include <sys/file.h>
 #include <sys/stat.h>
 #include "dummy.h"
 #include "misc.h"
@@ -89,12 +88,6 @@ DRIVER dummydriver = {
 
 				/* prototype stream */
 MAILSTREAM dummyproto = {&dummydriver};
-
-
-				/* driver parameters */
-static long mbx_protection = 0600;
-static long sub_protection = 0600;
-static long lock_protection = 0666;
 
 /* Dummy validate mailbox
  * Accepts: mailbox name
@@ -104,14 +97,13 @@ static long lock_protection = 0666;
 DRIVER *dummy_valid (name)
 	char *name;
 {
-  char tmp[MAILTMPLEN];
+  char *s,tmp[MAILTMPLEN];
   struct stat sbuf;
 				/* must be valid local mailbox */
   return (name && *name && (*name != '*') && (*name != '{') &&
-				/* INBOX is always accepted */
-	  ((!strcmp (ucase (strcpy (tmp,name)),"INBOX")) ||
-				/* so is empty file */
-	   (!(stat (dummy_file (tmp,name),&sbuf) || sbuf.st_size))))
+	  (s = mailboxfile (tmp,name)) &&
+				/* INBOX or empty file accepted */
+	  (!*s || (!(stat (s,&sbuf) || sbuf.st_size))))
     ? &dummydriver : NIL;
 }
 
@@ -126,30 +118,7 @@ void *dummy_parameters (function,value)
 	long function;
 	void *value;
 {
-  switch ((int) function) {
-  case SET_MBXPROTECTION:
-    mbx_protection = (long) value;
-    break;
-  case GET_MBXPROTECTION:
-    value = (void *) mbx_protection;
-    break;
-  case SET_SUBPROTECTION:
-    sub_protection = (long) value;
-    break;
-  case GET_SUBPROTECTION:
-    value = (void *) sub_protection;
-    break;
-  case SET_LOCKPROTECTION:
-    lock_protection = (long) value;
-    break;
-  case GET_LOCKPROTECTION:
-    value = (void *) lock_protection;
-    break;
-  default:
-    value = NIL;		/* error case */
-    break;
-  }
-  return value;
+  return NIL;
 }
 
 /* Dummy find list of subscribed mailboxes
@@ -208,16 +177,19 @@ void dummy_find_all (stream,pat)
   if (s = strrchr (pat,'/')) {	/* directory specified in pattern? */
     strncpy (file,pat,i = (++s) - pat);
     file[i] = '\0';		/* tie off prefix */
-    t = dummy_file (tmp,pat);	/* make fully-qualified file name */
+				/* make fully-qualified file name */
+    if (!(t = dummy_file (tmp,pat))) return;
 				/* tie off directory name */
     if (s = strrchr (t,'/')) *s = '\0';
   }
   else t = myhomedir ();	/* use home directory to search */
   if (dirp = opendir (t)) {	/* now open that directory */
-    while (d = readdir (dirp)) {/* for each directory entry */
-      strcpy (file + i,d->d_name);
-      if (pmatch (file,pat)) mm_mailbox (file);
-    }
+    while (d = readdir (dirp))	/* for each directory entry */
+      if ((d->d_name[0] != '.') ||
+	  (d->d_name[1] && ((d->d_name[1] != '.') || d->d_name[2]))) {
+	strcpy (file + i,d->d_name);
+	if (pmatch (file,pat)) mm_mailbox (file);
+      }
     closedir (dirp);		/* flush directory */
   }
 				/* always an INBOX */
@@ -247,10 +219,12 @@ void dummy_find_all_bboards (stream,pat)
   file[i] = '\0';		/* tie off prefix */
   sprintf (tmp,"%s/%s",pw->pw_dir,(file[1] == '/') ? file + 2 : file + 1);
   if (dirp = opendir (tmp)) {	/* now open that directory */
-    while (d = readdir (dirp)) {/* for each directory entry */
-      strcpy (file + i,d->d_name);
-      if (pmatch (file + 1,pat)) mm_bboard (file + 1);
-    }
+    while (d = readdir (dirp))	/* for each directory entry */
+      if ((d->d_name[0] != '.') ||
+	  (d->d_name[1] && ((d->d_name[1] != '.') || d->d_name[2]))) {
+	strcpy (file + i,d->d_name);
+	if (pmatch (file + 1,pat)) mm_bboard (file + 1);
+      }
     closedir (dirp);		/* flush directory */
   }
 }
@@ -385,31 +359,33 @@ long dummy_rename (stream,old,new)
 MAILSTREAM *dummy_open (stream)
 	MAILSTREAM *stream;
 {
-  char tmp[MAILTMPLEN];
+  int fd;
+  char err[MAILTMPLEN],tmp[MAILTMPLEN];
   struct stat sbuf;
-  int fd = -1;
 				/* OP_PROTOTYPE call */
   if (!stream) return &dummyproto;
-  if ((strcmp (ucase (strcpy (tmp,stream->mailbox)),"INBOX")) &&
-      ((fd = open (dummy_file (tmp,stream->mailbox),O_RDONLY,NIL)) < 0))
-    sprintf (tmp,"%s: %s",strerror (errno),stream->mailbox);
-  else {			/* open as empty file */
-    if (fd >= 0) {		/* if got a file */
-      fstat (fd,&sbuf);		/* sniff at its size */
-      close (fd);
-      if (sbuf.st_size) sprintf (tmp,"Not a mailbox: %s",stream->mailbox);
-      else fd = -1;		/* a-OK */
-    }
-    if (fd < 0) {		/* no file, right? */
-      if (!stream->silent) {	/* only if silence not requested */
-	mail_exists (stream,0);	/* say there are 0 messages */
-	mail_recent (stream,0);
-      }
-      return stream;		/* return success */
+  err[0] = '\0';		/* no error message yet */
+				/* can we open the file? */
+  if ((fd = open (dummy_file (tmp,stream->mailbox),O_RDONLY,NIL)) < 0) {
+				/* no, error unless INBOX */
+    if (strcmp (ucase (strcpy (tmp,stream->mailbox)),"INBOX"))
+      sprintf (err,"%s: %s",strerror (errno),stream->mailbox);
+  }
+  else {			/* file had better be empty then */
+    fstat (fd,&sbuf);		/* sniff at its size */
+    close (fd);
+    if (sbuf.st_size)		/* bogus format if non-empty */
+      sprintf (err,"%s (file %s) is not in valid mailbox format",
+	       stream->mailbox,tmp);
+  }
+  if (!stream->silent) {	/* only if silence not requested */
+    if (err[0]) mm_log (err,ERROR);
+    else {
+      mail_exists (stream,0);	/* say there are 0 messages */
+      mail_recent (stream,0);	/* and certainly no recent ones! */
     }
   }
-  if (!stream->silent) mm_log (tmp,ERROR);
-  return NIL;			/* always fails */
+  return err[0] ? NIL : stream;	/* return success if no error */
 }
 
 
@@ -645,7 +621,7 @@ long dummy_append (stream,mailbox,flags,date,message)
 	char *mailbox;
 	char *flags;
 	char *date;
-	STRING *message;
+	 		   STRING *message;
 {
   struct stat sbuf;
   int fd = -1;
@@ -698,5 +674,5 @@ char *dummy_file (dst,name)
 {
   char *s = mailboxfile (dst,name);
 				/* return our standard inbox */
-  return s ? s : strcpy (dst,sysinbox ());
+  return (s && !*s) ? strcpy (dst,sysinbox ()) : s;
 }
