@@ -10,9 +10,9 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 August 1988
- * Last Edited:	7 September 1994
+ * Last Edited:	20 May 1996
  *
- * Copyright 1994 by the University of Washington
+ * Copyright 1996 by the University of Washington
  *
  *  Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -34,6 +34,8 @@
  */
 
 
+#define SUBSCRIPTIONTEMP(t) sprintf (t,"%s/.mlbxlsttmp",myhomedir ())
+
 /* Function prototypes */
 
 #include "env.h"
@@ -42,6 +44,11 @@ long env_init  ();
 char *myusername  ();
 char *sysinbox  ();
 char *lockname  ();
+long chk_notsymlink  ();
 MAILSTREAM *user_flags  ();
 void dorc  ();
-void grim_pid_reap  ();
+void grim_pid_reap_status  ();
+#define grim_pid_reap(pid,killreq) \
+  grim_pid_reap_status (pid,killreq,NIL)
+long safe_write  ();
+long safe_writev  ();
