@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 November 1989
- * Last Edited:	3 June 1996
+ * Last Edited:	15 October 1996
  *
  * Copyright 1996 by the University of Washington
  *
@@ -1270,8 +1270,8 @@ long mail_parse_date (elt,s)
 				/* skip over possible day of week */
   if (isalpha (*s) && isalpha (s[1]) && isalpha (s[2]) && (s[3] == ',') &&
       (s[4] == ' ')) s += 5;
-				/* parse first number (probable month) */
-  if (!(s && (m = strtol ((const char *) s,&s,10)))) return NIL;
+  while (*s == ' ') s++;	/* parse first number (probable month) */
+  if (!(m = strtol ((const char *) s,&s,10))) return NIL;
 
   switch (*s) {			/* different parse based on delimiter */
   case '/':			/* mm/dd/yy format */
@@ -1279,6 +1279,7 @@ long mail_parse_date (elt,s)
 	  (y = strtol ((const char *) ++s,&s,10)) && *s == '\0')) return NIL;
     break;
   case ' ':			/* dd mmm yy format */
+    while (s[1] == ' ') s++;	/* slurp extra whitespace */
   case '-':			/* dd-mmm-yy format */
     d = m;			/* so the number we got is a day */
 				/* make sure string long enough! */
@@ -1299,7 +1300,7 @@ long mail_parse_date (elt,s)
     case (('O'-'A') * 1024) + (('C'-'A') * 32) + ('T'-'A'): m = 10; break;
     case (('N'-'A') * 1024) + (('O'-'A') * 32) + ('V'-'A'): m = 11; break;
     case (('D'-'A') * 1024) + (('E'-'A') * 32) + ('C'-'A'): m = 12; break;
-    default: return NIL;
+    default: return NIL;	/* unknown month */
     }
     if ((s[4] == *s) &&	(y = strtol ((const char *) s+5,&s,10)) &&
 	(*s == '\0' || *s == ' ')) break;
@@ -1324,25 +1325,26 @@ long mail_parse_date (elt,s)
     elt->hours = d; elt->minutes = m; elt->seconds = y;
     switch (*s) {		/* time zone specifier? */
     case ' ':			/* numeric time zone */
+      while (s[1] == ' ') s++;	/* slurp extra whitespace */
       if (!isalpha (s[1])) {	/* treat as '-' case if alphabetic */
 				/* test for sign character */
 	if ((elt->zoccident = (*++s == '-')) || (*s == '+')) s++;
 				/* validate proper timezone */
-	if (!(isdigit (*s) && isdigit (s[1]) && isdigit (s[2]) &&
-	      isdigit (s[3])) || (s[4] && (s[4] != ' '))) return NIL;
-	elt->zhours = (*s - '0') * 10 + (s[1] - '0');
-	elt->zminutes = (s[2] - '0') * 10 + (s[3] - '0');
-	break;
+	if (isdigit(*s) && isdigit(s[1]) && isdigit(s[2]) && isdigit(s[3])) {
+	  elt->zhours = (*s - '0') * 10 + (s[1] - '0');
+	  elt->zminutes = (s[2] - '0') * 10 + (s[3] - '0');
+	}
+	return T;		/* all done! */
       }
 				/* falls through */
 
     case '-':			/* symbolic time zone */
-      if (!(ms = *++s)) return NIL;
-      if (*++s) {		/* multi-character? */
+      if (!(ms = *++s)) ms = 'Z';
+      else if (*++s) {		/* multi-character? */
 	ms -= 'A'; ms *= 1024;	/* yes, make compressed three-byte form */
 	ms += ((*s++ - 'A') * 32);
 	if (*s) ms += *s++ - 'A';
-	if (*s) return NIL;	/* more than three characters */
+	if (*s) ms = '\0';	/* more than three characters */
       }
       /* This is not intended to be a comprehensive list of all possible
        * timezone strings.  Such a list would be impractical.  Rather, this
@@ -1442,9 +1444,8 @@ long mail_parse_date (elt,s)
       elt->zminutes = 0;	/* never a fractional hour */
       break;
     case '\0':			/* no time zone */
-      break;
-    default:
-      return NIL;
+    default:			/* bogus time zone */
+      break;			/* ignore both cases */
     }
   }
   else {			/* make sure all time fields zero */
