@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 November 1989
- * Last Edited:	7 February 1996
+ * Last Edited:	3 June 1996
  *
  * Copyright 1996 by the University of Washington
  *
@@ -457,9 +457,9 @@ DRIVER {
 				/* create mailbox */
   long (*create) ();
 				/* delete mailbox */
-  long (*delete) ();
+  long (*mbxdel) ();
 				/* rename mailbox */
-  long (*rename) ();
+  long (*mbxren) ();
 
 				/* open mailbox */
   MAILSTREAM *(*open) ();
