@@ -9,7 +9,7 @@
  *		Seattle, WA  98195
  *
  * Date:	11 May 1989
- * Last Edited:	11 July 1994
+ * Last Edited:	7 June 1995
  *
  * Copyright 1994 by the University of Washington
  *
@@ -67,31 +67,4 @@ extern int errno;		/* just in case */
 #include "strpbrk.c"
 #include "strstr.c"
 #include "strtok.c"
-
-/* Write current time in RFC 822 format
- * Accepts: destination string
- */
-
-char *days[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
-
-void rfc822_date (date)
-	char *date;
-{
-  int zone;
-  char *zonename,*timezone();
-  struct tm *t;
-  struct timeval tv;
-  struct tzone {
-    int	tz_minuteswest;		/* minutes west of Greenwich */
-    int	tz_dsttime;		/* type of dst correction */
-  };
-  struct tzone tz;
-  gettimeofday (&tv,&tz);	/* get time and timezone poop */
-  t = localtime (&tv.tv_sec);	/* convert to individual items */
-  zone = (t->tm_isdst ? 60 : 0) -tz.tz_minuteswest;
-  zonename = timezone (tz.tz_minuteswest,tz.tz_dsttime);
-				/* and output it */
-  sprintf (date,"%s, %d %s %d %02d:%02d:%02d %+03d%02d (%s)",
-	   days[t->tm_wday],t->tm_mday,months[t->tm_mon],t->tm_year+1900,
-	   t->tm_hour,t->tm_min,t->tm_sec,zone/60,abs (zone) % 60,zonename);
-}
+#include "tz_nul.c"
