@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	27 July 1988
- * Last Edited:	14 March 1996
+ * Last Edited:	8 April 1996
  *
  * Sponsorship:	The original version of this work was developed in the
  *		Symbolic Systems Resources Group of the Knowledge Systems
@@ -740,11 +740,11 @@ void rfc822_parse_content_header (body,name,s)
     if (!(strcmp (name+1,"D") || body->id)) body->id = cpystr (s);
     break;
   case 'D':			/* possible Content-Description */
-    if (!(strcmp (name+1,"ESCRIPTION")) || body->description)
+    if (!(strcmp (name+1,"ESCRIPTION") || body->description))
       body->description = cpystr (s);
     break;
   case 'M':			/* possible Content-MD5 */
-    if (!(strcmp (name+1,"D5")) || body->md5) body->md5 = cpystr (s);
+    if (!(strcmp (name+1,"D5") || body->md5)) body->md5 = cpystr (s);
     break;
   case 'T':			/* possible Content-Type/Transfer-Encoding */
     if (!(strcmp (name+1,"YPE") || body->type || body->subtype ||
@@ -879,7 +879,9 @@ void rfc822_parse_adrlist (lst,string,host)
       }
     }
     else if (string) {		/* bad mailbox */
-      sprintf (tmp,"Invalid mailbox list: %.80s",string);
+      rfc822_skipws (&string);	/* skip WS */
+      if (!*string) strcpy (tmp,"Missing address after comma");
+      else sprintf (tmp,"Invalid mailbox list: %.80s",string);
       mm_log (tmp,PARSE);
       string = NIL;
       (adr = mail_newaddr ())->mailbox = cpystr ("INVALID_ADDRESS");
