@@ -10,9 +10,9 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 August 1988
- * Last Edited:	9 April 1996
+ * Last Edited:	21 February 1997
  *
- * Copyright 1996 by the University of Washington
+ * Copyright 1997 by the University of Washington
  *
  *  Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -199,9 +199,11 @@ TCPSTREAM *tcp_open (host,service,port)
 	 (errno == EINTR));
   if (i > 0) {			/* success, make sure really connected */
     fcntl (sock,F_SETFL,flgs);	/* restore blocking status */
+#ifndef SOLARISKERNELBUG
 				/* get socket status */
     while ((i = read (sock,tmp,0)) < 0 && errno == EINTR);
     if (!i) i = 1;		/* make success if the read is OK */
+#endif
   }
   if (i <= 0) {			/* timeout or error? */
     sprintf (tmp,"Can't connect to %.80s,%d: %s",hostname,port,
@@ -565,8 +567,8 @@ char *tcp_clienthost (dst)
   struct sockaddr_in from;
   int fromlen = sizeof (from);
   if (getpeername (0,(struct sockaddr *) &from,&fromlen)) return "UNKNOWN";
-  strcpy (dst,(hn = gethostbyaddr ((char *) &from.sin_addr,
+  strncpy (dst,(hn = gethostbyaddr ((char *) &from.sin_addr,
 				   sizeof (struct in_addr),from.sin_family)) ?
-	  hn->h_name : inet_ntoa (from.sin_addr));
+	   hn->h_name : inet_ntoa (from.sin_addr),80);
   return dst;
 }
