@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: bind.c,v 4.4 1994/03/17 00:56:12 mikes Exp $";
+static char rcsid[] = "$Id: bind.c,v 4.18 1996/07/09 23:07:31 mikes Exp $";
 #endif
 /*
  * Program:	Key binding routines
@@ -15,28 +15,16 @@ static char rcsid[] = "$Id: bind.c,v 4.4 1994/03/17 00:56:12 mikes Exp $";
  *
  * Please address all bugs and comments to "pine-bugs@cac.washington.edu"
  *
- * Copyright 1991-1994  University of Washington
  *
- *  Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee to the University of
- * Washington is hereby granted, provided that the above copyright notice
- * appears in all copies and that both the above copyright notice and this
- * permission notice appear in supporting documentation, and that the name
- * of the University of Washington not be used in advertising or publicity
- * pertaining to distribution of the software without specific, written
- * prior permission.  This software is made available "as is", and
- * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
- * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
- * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
- * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
- * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Pine and Pico are trademarks of the University of Washington.
- * No commercial use of these trademarks may be made without prior
- * written permission of the University of Washington.
+ * Pine and Pico are registered trademarks of the University of Washington.
+ * No commercial use of these trademarks may be made without prior written
+ * permission of the University of Washington.
+ * 
+ * Pine, Pico, and Pilot software and its included text are Copyright
+ * 1989-1996 by the University of Washington.
+ * 
+ * The full text of our legal notices is contained in the file called
+ * CPYRIGHT, included with this distribution.
  *
  */
 /*	This file is for functions having to do with key bindings,
@@ -46,9 +34,10 @@ static char rcsid[] = "$Id: bind.c,v 4.4 1994/03/17 00:56:12 mikes Exp $";
 								*/
 
 #include	<stdio.h>
+#include	"osdep.h"
+#include	"pico.h"
 #include	"estruct.h"
 #include	"edef.h"
-#include	"pico.h"
 
 #ifdef	ANSI
     int arraylen(char **);
@@ -148,21 +137,54 @@ char **array;
  */
 whelp(f, n)
 {
-    if(Pmaster){
-	(*Pmaster->helper)(Pmaster->composer_help, "Help on the Pine Composer", 1);
-	curwp->w_flag |= WFMODE;
-    }
-    else
-      pico_help(helptext);
+    if(term.t_mrow == 0){  /* blank keymenu in effect */
+	if(km_popped == 0){
+	    /* cause keymenu display */
+	    km_popped = 2;
+	    if(!Pmaster)
+	      sgarbf = TRUE;
 
-        sgarbf = TRUE;
+	    return(TRUE);
+	}
+    }
+
+    if(Pmaster){
+	(*Pmaster->helper)(Pmaster->composer_help,
+			   Pmaster->headents
+			     ? "Help on the Pine Composer"
+			     : "Help on Signature Editor",
+			   1);
+	ttresize();
+	picosigs();			/* restore any altered handlers */
+	curwp->w_flag |= WFMODE;
+	if(km_popped)  /* this will unpop us */
+	  curwp->w_flag |= WFHARD;
+    }
+    else{
+	int mrow_was_zero = 0;
+
+	/* always want keyhelp showing during help */
+	if(term.t_mrow == 0){
+	    mrow_was_zero++;
+	    term.t_mrow = 2;
+	}
+
+	pico_help(helptext);
+	/* put it back the way it was */
+	if(mrow_was_zero)
+	  term.t_mrow = 0;
+    }
+
+    sgarbf = TRUE;
 }
 
 static KEYMENU menu_scroll[] = {
-    {NULL, NULL},		{NULL, NULL},	{NULL, NULL},
-    {NULL, NULL},		{NULL, NULL},	{NULL, NULL},
-    {"^X", "Exit Help"},	{NULL, NULL},	{NULL, NULL},
-    {NULL, NULL},		{NULL, NULL},	{NULL, NULL}
+    {NULL, NULL, KS_NONE},		{NULL, NULL, KS_NONE},
+    {NULL, NULL, KS_NONE},		{NULL, NULL, KS_NONE},
+    {NULL, NULL, KS_NONE},		{NULL, NULL, KS_NONE},
+    {"^X", "Exit Help", KS_NONE},	{NULL, NULL, KS_NONE},
+    {NULL, NULL, KS_NONE},		{NULL, NULL, KS_NONE},
+    {NULL, NULL, KS_NONE},		{NULL, NULL, KS_NONE}
 };
 #define	PREV_KEY	3
 #define	NEXT_KEY	9
@@ -302,15 +324,16 @@ int	textlen;
 
 
 /*
- * normal - given a char and list of function key to command key mappings, 
- *          return, depending on gmode, the right command.  The list is
- *          an array of (Fkey, command-key) pairs.  sc is the index in the
- *          array that means to ignore fkey vs. command key mapping
+ * normalize_cmd - given a char and list of function key to command key
+ *		   mappings, return, depending on gmode, the right command.
+ *		   The list is an array of (Fkey, command-key) pairs.
+ *		   sc is the index in the array that means to ignore fkey
+ *		   vs. command key mapping
  *
  *          rules:  1. if c not in table (either fkey or command), let it thru
  *                  2. if c matches, but in other mode, punt it
  */
-normal(c, list, sc)
+normalize_cmd(c, list, sc)
 int c, sc;
 int list[][2];
 {
@@ -331,6 +354,17 @@ int list[][2];
 	}
     }
 
+#ifdef _WINDOWS
+    /*
+     * Menu's get alpha key commands assigned to them.  Well, if we are in PF
+     * key mode then the alpha key commands don't work.  So, we flag them with
+     * a bit (MENU defined in estruct.h) so we can recognize them as different
+     * from a keyboard press.  Here we clear that flag so the command can be
+     * processed.
+     */
+    c &= ~MENU;
+#endif
+
     return(c);
 }
 
@@ -351,3 +385,29 @@ int (*a)(), (*b)();
 	kp++;
     }
 }
+
+
+/*
+ * bindtokey - bind function f to command c
+ */
+bindtokey(c, f)
+int c, (*f)();
+{
+    KEYTAB *kp, *ktab = (Pmaster) ? &keytab[0] : &pkeytab[0];
+
+    for(kp = ktab; kp->k_fp; kp++)
+      if(kp->k_code == c){
+	  kp->k_fp = f;			/* set to new function */
+	  break;
+      }
+
+    /* not found? create new binding */
+    if(!kp->k_fp && kp < &ktab[NBINDS]){
+	kp->k_code     = c;		/* assign new code and function */
+	kp->k_fp       = f;
+	(++kp)->k_code = 0;		/* and null out next slot */
+	kp->k_fp       = NULL;
+    }
+
+    return(TRUE);
+}
