@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: browse.c,v 4.59 1996/03/16 23:32:58 mikes Exp $";
+static char rcsid[] = "$Id: browse.c,v 4.62 1996/05/29 00:05:04 mikes Exp $";
 #endif
 /*
  * Program:	Routines to support file browser in pico and Pine composer
@@ -142,6 +142,7 @@ static	KEYMENU menu_browse[] = {
 };
 #define	QUIT_KEY	1
 #define	EXEC_KEY	2
+#define	GOTO_KEY	6
 #define	SELECT_KEY	7
 #define	PICO_KEY	11
 
@@ -310,6 +311,11 @@ int   flags;
 	return(0);
     }
 
+    if(gmode&MDGOTO){
+	/* fix up function key mapping table */
+	/* fix up key menu labels */
+    }
+
     /* build contents of cell structures */
     if((gmp = getfcells(dir)) == NULL)
       return(-1);
@@ -829,6 +835,77 @@ int   flags;
 	    PaintBrowser(gmp, 0, &crow, &ccol);
 	    break;
 
+	  case 'g':                             /* jump to a directory */
+	  case 'G':
+
+	    if(!(gmode&MDGOTO))
+	      goto Default;
+
+	    i = 0;
+	    child[0] = '\0';
+ 
+	    while(!i){
+ 
+		status = mlreply("Directory to go to: ", child, NLINE, QNORML,
+				 NULL);
+ 
+		switch(status){
+		  case HELPCH:
+		    emlwrite("\007No help yet!", NULL);
+		    /* remove break and sleep after help text is installed */
+		    sleep(3);
+		    break;
+		  case (CTRL|'L'):
+		    PaintBrowser(gmp, 0, &crow, &ccol);
+		  break;
+		  case ABORT:
+		    emlwrite("Goto cancelled", NULL);
+		    i++;
+		    break;
+		  case FALSE:
+		  case TRUE:
+		    i++;
+ 
+		    if(*child == '\0')
+		      strcpy(child, gethomedir(NULL));
+ 
+		    if(!compresspath(gmp->dname, child, NLINE)){
+			emlwrite("Invalid Directory: %s", child);
+			break;
+		    }
+
+		    if((gmode&MDSCUR) && homeless(child)){
+			emlwrite("Restricted mode browsing limited to home directory",NULL);
+			break;
+		    }
+ 
+		    if((gmode&MDTREE) && !in_oper_tree(child)){
+			emlwrite("Attempt to Goto directory denied", NULL);
+			break;
+		    }
+ 
+		    if(isdir(child, (long *) NULL)){
+			if((mp = getfcells(child)) == NULL){
+			    /* getfcells should explain what happened */
+			    i++;
+			    break;
+			}
+ 
+			zotmaster(&gmp);
+			gmp = mp;
+			PaintBrowser(gmp, 0, &crow, &ccol);
+		    }
+		    else
+		      emlwrite("\007Not a directory: \"%s\"", child);
+ 
+		    break;
+		  default:
+		    break;
+		}
+	    }
+	    BrowserKeys();
+	    break;
+ 
 	  case 'c':					/* copy */
 	  case 'C':
 	    if(gmp->current->mode == FIODIR){
@@ -1050,10 +1127,8 @@ int   flags;
 	  Selected:
 
 	    if((toupper(new_c) == 'S' && (gmode&MDBRONLY))
-	       || (toupper(new_c) == 'V' && !(gmode&MDBRONLY))){
-		(*term.t_beep)();
-		goto Default;
-	    }
+	       || (toupper(new_c) == 'V' && !(gmode&MDBRONLY)))
+	      goto Default;
 
 	    if(gmp->current->mode == FIODIR){
 		*child = '\0';
@@ -1553,6 +1628,8 @@ BrowserKeys()
 {
     menu_browse[QUIT_KEY].name  = (gmode&MDBRONLY) ? "Q" : "E";
     menu_browse[QUIT_KEY].label = (gmode&MDBRONLY) ? "Quit" : "Exit Brwsr";
+    menu_browse[GOTO_KEY].name  = (gmode&MDGOTO) ? "G" : NULL;
+    menu_browse[GOTO_KEY].label = (gmode&MDGOTO) ? "Goto" : NULL;
     if(gmode & MDBRONLY){
 	menu_browse[EXEC_KEY].name  = "L";
 	menu_browse[EXEC_KEY].label = "Launch";
@@ -1771,7 +1848,7 @@ char *s1, *s2;
 
     while(*s1){
 	j = 0;
-	while(toupper(s1[j]) == toupper(s2[j]))
+	while(toupper((unsigned char)s1[j]) == toupper((unsigned char)s2[j]))
 	  if(s2[++j] == '\0')			/* bingo! */
 	    return(1);
 
@@ -1922,7 +1999,7 @@ LikelyASCII(file)
     char *file;
 {
 #define	LA_TEST_BUF	1024
-#define	LA_LINE_LIMIT	128
+#define	LA_LINE_LIMIT	300
 #if defined(DOS) || defined(OS2)
 #define	MODE	"rb"
 #else
