@@ -1,5 +1,5 @@
 /*
- * $Id: dos_gen.c,v 4.19 1996/03/15 07:41:11 hubert Exp $
+ * $Id: dos_gen.c,v 4.23 1996/05/31 00:01:14 mikes Exp $
  *
  * Program:	Operating system dependent routines - MS DOS Generic
  *
@@ -59,7 +59,7 @@ static unsigned char okinfname[32] = {
       0,    0, 			/* ^@ - ^G, ^H - ^O  */
       0,    0,			/* ^P - ^W, ^X - ^_  */
       0x80, 0x17,		/* SP - ' ,  ( - /   */
-      0xff, 0xe0,		/*  0 - 7 ,  8 - ?   */
+      0xff, 0xe4,		/*  0 - 7 ,  8 - ?   */
       0x7f, 0xff,		/*  @ - G ,  H - O   */
       0xff, 0xe9,		/*  P - W ,  X - _   */
       0x7f, 0xff,		/*  ` - g ,  h - o   */
@@ -362,9 +362,9 @@ int  len;
       return(0);
     
     /* return the full path of given file */
-    if(isalpha(name[0]) && name[1] == ':'){	/* have drive spec? */
-	if(name[2] != '\\'){			/* including path? */
-	    dr = toupper(name[0]) - 'A' + 1;
+    if(isalpha((unsigned char)name[0]) && name[1] == ':'){/* have drive spec? */
+	if(name[2] != '\\'){				  /* including path? */
+	    dr = toupper((unsigned char)name[0]) - 'A' + 1;
 	    if(_getdcwd(dr, file, _MAX_PATH) != NULL){
 		if(file[strlen(file)-1] != '\\')
 		  strcat(file, "\\");
@@ -629,6 +629,17 @@ ffclose()
 }
 
 
+/*
+ * ffelbowroom - make sure the destination's got enough room to receive
+ *		 what we're about to write...
+ */
+ffelbowroom(fn)
+char    *fn;
+{
+    return(TRUE);
+}
+
+
 /*
  * worthit - generic sort of test to roughly gage usefulness of using 
  *           optimized scrolling.
