@@ -1,5 +1,5 @@
 /*
- * $Id: efunc.h,v 4.37 1996/05/31 00:06:19 mikes Exp $
+ * $Id: efunc.h,v 4.38 1996/07/09 23:55:32 mikes Exp $
  *
  * Program:	Pine's composer and pico's function declarations
  *
@@ -238,6 +238,7 @@ extern	int yank(int, int);
 
 /* region.c */
 extern	int killregion(int, int);
+extern	int deleteregion(int, int);
 extern	int markregion(int);
 
 /* search.c */
@@ -453,6 +454,7 @@ extern	int yank();
 
 /* region.c */
 extern	int killregion();
+extern	int deleteregion();
 extern	int markregion();
 
 /* search.c */
