@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: file.c,v 4.14 1994/10/11 23:46:51 mikes Exp $";
+static char rcsid[] = "$Id: file.c,v 4.45 1996/05/31 00:06:51 mikes Exp $";
 #endif
 /*
  * Program:	High level file input and output routines
@@ -15,28 +15,16 @@ static char rcsid[] = "$Id: file.c,v 4.14 1994/10/11 23:46:51 mikes Exp $";
  *
  * Please address all bugs and comments to "pine-bugs@cac.washington.edu"
  *
- * Copyright 1991-1994  University of Washington
  *
- *  Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee to the University of
- * Washington is hereby granted, provided that the above copyright notice
- * appears in all copies and that both the above copyright notice and this
- * permission notice appear in supporting documentation, and that the name
- * of the University of Washington not be used in advertising or publicity
- * pertaining to distribution of the software without specific, written
- * prior permission.  This software is made available "as is", and
- * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
- * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
- * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
- * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
- * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Pine and Pico are trademarks of the University of Washington.
- * No commercial use of these trademarks may be made without prior
- * written permission of the University of Washington.
+ * Pine and Pico are registered trademarks of the University of Washington.
+ * No commercial use of these trademarks may be made without prior written
+ * permission of the University of Washington.
+ * 
+ * Pine, Pico, and Pilot software and its included text are Copyright
+ * 1989-1996 by the University of Washington.
+ * 
+ * The full text of our legal notices is contained in the file called
+ * CPYRIGHT, included with this distribution.
  *
  */
 /*
@@ -56,8 +44,10 @@ static char rcsid[] = "$Id: file.c,v 4.14 1994/10/11 23:46:51 mikes Exp $";
 
 #ifdef	ANSI
     int ifile(char *);
+    int insmsgchar(int);
 #else
     int ifile();
+    int insmsgchar();
 #endif
 
 
@@ -87,6 +77,11 @@ int f, n;
 	  return(0);
 	}
 
+	if((gmode&MDTREE) && !in_oper_tree(fname)){
+	  emlwrite("Can't read file from outside of %s", opertree);
+	  return(0);
+	}
+
         return(readin(fname, TRUE));
 }
 
@@ -99,7 +94,7 @@ static char *inshelptext[] = {
   "\tType in a file name to have it inserted into your editing",
   "\tbuffer between the line that the cursor is currently on",
   "\tand the line directly below it.  You may abort this by ",
-  "~\ttyping the ~F~3 (~^~C) key after exiting help.",
+  "~\ttyping the ~F~2 (~^~C) key after exiting help.",
   " ",
   "End of Insert File Help",
   " ",
@@ -111,7 +106,7 @@ static char *writehelp[] = {
   " ",
   "\tType in a file name to have it written out, thus saving",
   "\tyour buffer, to a file.  You can abort this by typing ",
-  "~\tthe ~F~3 (~^~C) key after exiting help.",
+  "~\tthe ~F~2 (~^~C) key after exiting help.",
   " ",
   "End of Write File Help",
   " ",
@@ -130,41 +125,145 @@ static char *writehelp[] = {
 insfile(f, n)
 int f, n;
 {
-        register int    s;
-        char fname[NFILEN], dir[NFILEN];
-        int	retval, bye = 0;
-        char	*prompt;
-	char     pbuf[40];
-        register int	availen;
-	KEYMENU menu_ins[2];
+    register int s;
+    char	 fname[NFILEN], dir[NFILEN];
+    int		 retval, bye = 0, msg = 0;
+    char	 prompt[64];
+    EXTRAKEYS    menu_ins[5];
+    
+    if (curbp->b_mode&MDVIEW) /* don't allow this command if */
+      return(rdonly()); /* we are in read only mode */
 
-	prompt = pbuf;
-	strcat(strcat(strcpy(prompt, "Insert file from "),
-				    gmode&MDCURDIR ? "current" : "home"),
-				    " directory: ");
-        availen = term.t_ncol - strlen(prompt);
+    fname[0] = '\0';
+    while(!bye){
+	/* set up keymenu stuff */
+	if(!msg){
+	    int last_menu = 0;
 
-	if (curbp->b_mode&MDVIEW)	/* don't allow this command if	*/
-		return(rdonly());	/* we are in read only mode	*/
+	    menu_ins[last_menu].name  = "^T";
+	    menu_ins[last_menu].key   = (CTRL|'T');
+	    menu_ins[last_menu].label = "To Files";
+	    KS_OSDATASET(&menu_ins[last_menu], KS_NONE);
 
-        fname[0] = '\0';
-	menu_ins[0].name  = "^T";
-	menu_ins[0].label = "To Files";
-	menu_ins[1].name  = NULL;
-        while(!bye){
-	    s = mlreplyd(prompt, fname, NFILEN, QDEFLT, menu_ins);
-            if (s != TRUE){
-		switch(s){
-		  case (CTRL|'T'):
+	    if(Pmaster && Pmaster->msgntext){
+		menu_ins[++last_menu].name  = "^W";
+		menu_ins[last_menu].key     = (CTRL|'W');
+		menu_ins[last_menu].label   = msg ? "InsertFile" : "InsertMsg";
+		KS_OSDATASET(&menu_ins[last_menu], KS_NONE);
+	    }
+
+#if	!defined(DOS) && !defined(MAC)
+	    if(Pmaster && Pmaster->upload){
+		menu_ins[++last_menu].name = "^Y";
+		menu_ins[last_menu].key    = (CTRL|'Y');
+		menu_ins[last_menu].label  = "RcvUpload";
+		KS_OSDATASET(&menu_ins[last_menu], KS_NONE);
+	    }
+#endif	/* !(DOS || MAC) */
+
+	    if(gmode & MDCMPLT){
+		menu_ins[++last_menu].name = msg ? "" : "TAB";
+		menu_ins[last_menu].key    = (CTRL|'I');
+		menu_ins[last_menu].label  = msg ? "" : "Complete";
+		KS_OSDATASET(&menu_ins[last_menu], KS_NONE);
+	    }
+
+	    menu_ins[++last_menu].name = NULL;
+	}
+
+	sprintf(prompt, "%s to insert from %s %s: ",
+		msg ? "Number of message" : "File",
+		(msg || (gmode&MDCURDIR)) ? "current"
+					  : (gmode&MDTREE) ? opertree
+							   : "home",
+		msg ? "folder" : "directory");
+	s = mlreplyd(prompt, fname, NFILEN, QDEFLT, msg ? NULL : menu_ins);
+	/* something to read and it was edited or the default accepted */
+        if(fname[0] && (s == TRUE || s == FALSE)){
+	    bye++;
+	    if(msg){
+		if((*Pmaster->msgntext)(atol(fname), insmsgchar))
+		  emlwrite("Message %s included", fname);
+	    }
+	    else{
+		bye++;
+		if(gmode&MDSCUR){
+		    emlwrite("Can't insert file in restricted mode",NULL);
+		}
+		else{
+		    if(gmode&MDTREE)
+		      compresspath(opertree, fname, NFILEN);
+
+		    fixpath(fname, NFILEN);
+		    if((gmode&MDTREE) && !in_oper_tree(fname))
+		      emlwrite("Can't insert file from outside of %s",
+			        opertree);
+		    else
+		      retval = ifile(fname);
+		}
+	    }
+	}
+	else{
+	    switch(s){
+	      case (CTRL|'I') :
+		{
+		    char *fn, *p;
+		    int   l = NFILEN;;
+
+		    dir[0] = '\0';
+		    if(*fname && (p = strrchr(fname, C_FILESEP))){
+			fn = p + 1;
+			l -= fn - fname;
+			if(p == fname)
+			  strcpy(dir, S_FILESEP);
+#ifdef	DOS
+			else if(fname[0] == C_FILESEP
+				 || (isalpha((unsigned char)fname[0])
+				     && fname[1] == ':')){
+#else
+			else if (fname[0] == C_FILESEP || fname[0] == '~') {
+#endif
+			    strncpy(dir, fname, p - fname);
+			    dir[p-fname] = '\0';
+			}
+			else
+			  sprintf(dir, "%s%c%.*s", 
+				  (gmode&MDCURDIR) ? "."
+					  : (gmode&MDTREE) ? opertree
+							   : gethomedir(NULL),
+				  C_FILESEP, p - fname, fname);
+		    }
+		    else{
+			fn = fname;
+			strcpy(dir, gmode&MDCURDIR ? "."
+					    : gmode&MDTREE ? opertree
+							   : gethomedir(NULL));
+		    }
+
+		    if(!pico_fncomplete(dir, fn, l - 1))
+		      (*term.t_beep)();
+		}
+
+		break;
+	      case (CTRL|'W'):
+		msg = !msg;			/* toggle what to insert */
+		break;
+	      case (CTRL|'T'):
+		if(msg){
+		    emlwrite("Can't select messages yet!", NULL);
+		}
+		else{
 		    if(*fname && isdir(fname, NULL))
 		      strcpy(dir, fname);
 		    else
-		      strcpy(dir, gmode&MDCURDIR ? "." : gethomedir(NULL));
+		      strcpy(dir, (gmode&MDCURDIR) ? "."
+					  : (gmode&MDTREE) ? opertree
+							   : gethomedir(NULL));
 
 		    fname[0] = '\0';
 		    if((s = FileBrowse(dir, fname, NULL, FB_READ)) == 1){
 			if(gmode&MDSCUR){
-			    emlwrite("Can't insert file in restricted mode",
+			    emlwrite("Can't insert in restricted mode",
 				     NULL);
 			    sleep(2);
 			}
@@ -183,40 +282,86 @@ int f, n;
 			update(); 		/* redraw on return */
 			continue;
 		    }
-		    break;
-		  case HELPCH:
-		    if(Pmaster){
-			(*Pmaster->helper)(Pmaster->ins_help,
-					   "Help for Insert File", 1);
+		}
+
+		break;
+
+#if	!defined(DOS) && !defined(MAC)
+	      case (CTRL|'Y') :
+		if(Pmaster && Pmaster->upload){
+		    char tfname[NFILEN];
+
+		    if(gmode&MDSCUR){
+			emlwrite(
+			      "\007Restricted mode disallows uploaded command",
+			      NULL);
+			return(0);
 		    }
-		    else
-		      pico_help(inshelptext, "Help for Insert File", 1);
-		  case (CTRL|'L'):
+
+		    tfname[0] = '\0';
+		    retval = (*Pmaster->upload)(tfname, NULL);
+
 		    refresh(FALSE, 1);
 		    update();
-		    continue;
-		  default:
-                    retval = s;
-		    bye++;
-		}
-            }
-            else{
-		bye++;
-		if(gmode&MDSCUR){
-		    emlwrite("Can't insert file in restricted mode",NULL);
-		}
-		else{
-		    fixpath(fname, NFILEN);
-		    retval = ifile(fname);
-		}
-            }
-        }
-        curwp->w_flag |= WFMODE|WFHARD;
 
-        return(retval);
+		    if(retval){
+			retval = ifile(tfname);
+			bye++;
+		    }
+		    else
+		      sleep(3);			/* problem, show error! */
+
+		    if(tfname[0])		/* clean up temp file */
+		      unlink(tfname);
+		}
+		else
+		  (*term.t_beep)();		/* what? */
+
+		break;
+#endif	/* !(DOS || MAC) */
+	      case HELPCH:
+		if(Pmaster){
+		    (*Pmaster->helper)(Pmaster->ins_help,
+				       "Help for Insert File", 1);
+		}
+		else
+		  pico_help(inshelptext, "Help for Insert File", 1);
+	      case (CTRL|'L'):
+		refresh(FALSE, 1);
+		update();
+		continue;
+	      default:
+		ctrlg(FALSE, 0);
+                retval = s;
+		bye++;
+	    }
+        }
+    }
+    curwp->w_flag |= WFMODE|WFHARD;
+    
+    return(retval);
 }
 
 
+insmsgchar(c)
+    int c;
+{
+    if(c == '\n'){
+	char *p;
+
+	lnewline();
+	for(p = Pmaster->quote_str; p && *p; p++)
+	  if(!linsert(1, *p))
+	    return(0);
+    }
+    else if(c != '\r')			/* ignore CR (likely CR of CRLF) */
+      return(linsert(1, c));
+
+    return(1);
+}
+
+
+
 /*
  * Read file "fname" into the current
  * buffer, blowing away any text found there. Called
@@ -243,18 +388,28 @@ int	lockfl;		/* check for file locks? */
 	CELL            ac;
 
         bp = curbp;                             /* Cheap.               */
+	bp->b_linecnt = -1;			/* Must be recalculated */
 	ac.a = 0;
         if ((s=bclear(bp)) != TRUE)             /* Might be old.        */
                 return (s);
         bp->b_flag &= ~(BFTEMP|BFCHG);
 	/* removed 'C' mode detection */
         strcpy(bp->b_fname, fname);
-        if ((s=ffropen(fname)) == FIOERR)       /* Hard file open.      */
-                goto out;
-        if (s == FIOFNF) {                      /* File not found.      */
-                emlwrite("New file", NULL);
-                goto out;
-        }
+	if((gmode&MDTREE) && !in_oper_tree(fname)){
+	    emlwrite("Can't read file from outside of %s", opertree);
+	    s = FIOERR;
+	    goto out;
+	}
+
+        if ((s=ffropen(fname)) != FIOSUC){      /* Hard file open.      */
+	    if(s == FIOFNF)                     /* File not found.      */
+	      emlwrite("New file", NULL);
+	    else
+	      fioperr(s, fname);
+
+	    goto out;
+	}
+
         emlwrite("Reading file", NULL);
         nline = 0;
 	lflag = FALSE;
@@ -325,7 +480,7 @@ int f, n;
         char            fname[NFILEN];
 	char		shows[128], *bufp;
 	long		l;		/* length returned from fexist() */
-	KEYMENU		menu_write[2];
+	EXTRAKEYS	menu_write[3];
 
 	if(curbp->b_fname[0] != 0)
 	  strcpy(fname, curbp->b_fname);
@@ -334,19 +489,73 @@ int f, n;
 
 	menu_write[0].name  = "^T";
 	menu_write[0].label = "To Files";
-	menu_write[1].name  = NULL;
-	for(;;){
+	menu_write[0].key   = (CTRL|'T');
+	menu_write[1].name  = "TAB";
+	menu_write[1].label = "Complete";
+	menu_write[1].key   = (CTRL|'I');
+	menu_write[2].name  = NULL;
+	for(;!(gmode & MDTOOL);){
 	    s = mlreplyd("File Name to write : ", fname, NFILEN,
 			 QDEFLT|QFFILE, menu_write);
 
-	    fixpath(fname, NFILEN);		/*  fixup ~ in file name  */
-
 	    switch(s){
 	      case FALSE:
-		if(strlen(fname) == 0)		/* no file name to write to */
-		  return(s);
+		if(!fname[0]){			/* no file name to write to */
+		    ctrlg(FALSE, 0);
+		    return(s);
+		}
 	      case TRUE:
-		break;
+		if(gmode&MDTREE)
+		  compresspath(opertree, fname, NFILEN);
+
+		fixpath(fname, NFILEN);		/*  fixup ~ in file name  */
+		if((gmode&MDTREE) && !in_oper_tree(fname)){
+		    emlwrite("Can't write outside of %s", opertree);
+		    sleep(2);
+		    continue;
+		}
+		else
+		  break;
+	      case (CTRL|'I'):
+		{
+		    char *fn, *p, dir[NFILEN];
+		    int   l = NFILEN;
+
+		    dir[0] = '\0';
+		    if(*fname && (p = strrchr(fname, C_FILESEP))){
+			fn = p + 1;
+			l -= fn - fname;
+			if(p == fname)
+			  strcpy(dir, S_FILESEP);
+#ifdef	DOS
+			else if(fname[0] == C_FILESEP
+				 || (isalpha((unsigned char)fname[0])
+				     && fname[1] == ':')){
+#else
+			else if (fname[0] == C_FILESEP || fname[0] == '~') {
+#endif
+			    strncpy(dir, fname, p - fname);
+			    dir[p-fname] = '\0';
+			}
+			else
+			  sprintf(dir, "%s%c%.*s", 
+				  (gmode&MDCURDIR) ? "."
+					  : gmode&MDTREE ? opertree
+							 : gethomedir(NULL),
+				  C_FILESEP, p - fname, fname);
+		    }
+		    else{
+			fn = fname;
+			strcpy(dir, gmode&MDCURDIR ? "."
+					: gmode&MDTREE ? opertree
+						       : gethomedir(NULL));
+		    }
+
+		    if(!pico_fncomplete(dir, fn, l - 1))
+		      (*term.t_beep)();
+		}
+
+		continue;
 	      case (CTRL|'T'):
 		/* If we have a file name, break up into path and file name.*/
 		*shows = 0;
@@ -370,8 +579,8 @@ int f, n;
 		}
 
 		/* If we did not end up with a valid directory, use home. */
-		if (!isdir (shows, NULL))
-		  strcpy(shows, gethomedir(NULL));
+		if (!*shows || !isdir (shows, NULL))
+		  strcpy(shows, (gmode&MDTREE) ? opertree : gethomedir(NULL));
 
 		s = FileBrowse(shows, fname, NULL, FB_SAVE);
 		strcat(shows, S_FILESEP);
@@ -401,13 +610,7 @@ int f, n;
 	    if((s=fexist(fname, "w", &l)) == FIOSUC){ /* exists.  overwrite? */
 
 		sprintf(shows, "File \"%s\" exists, OVERWRITE", fname);
-		if((s=mlyesno(shows, FALSE)) != TRUE){
-		    if((bufp = strrchr(fname,C_FILESEP)) == NULL)
-		      fname[0] = '\0';
-		    else
-		      *++bufp = '\0';
-		}
-		else
+		if((s=mlyesno(shows, FALSE)) == TRUE)
 		  break;
 	    }
 	    else if(s == FIOFNF){
@@ -505,8 +708,8 @@ char    *fn;
         register int    nline;
 	char     line[80];
 
-        if ((s=ffwopen(fn)) != FIOSUC)          /* Open writes message. */
-                return (-1);
+        if (!ffelbowroom(fn) || (s=ffwopen(fn)) != FIOSUC)
+                return (-1);			/* Open writes message. */
 
         lp = lforw(curbp->b_linep);             /* First line.          */
         nline = 0;                              /* Number of lines.     */
@@ -544,8 +747,7 @@ int f, n;
         if ((s=ffwopen(fn)) != FIOSUC)          /* Open writes message. */
                 return(NULL);
 
-#ifndef _WINDOWS	/* For some reason, in windows changing the mode */
-			/* _always_ causes the file close to fail. */
+#ifdef	MODE_READONLY
 	chmod(fn, MODE_READONLY);		/* fix access rights */
 #endif
 
@@ -595,13 +797,13 @@ char    fname[];
         bp = curbp;                             /* Cheap.               */
         bp->b_flag |= BFCHG;			/* we have changed	*/
 	bp->b_flag &= ~BFTEMP;			/* and are not temporary*/
+	bp->b_linecnt = -1;			/* must be recalculated */
 	ac.a = 0;
-        if ((s=ffropen(fname)) == FIOERR)       /* Hard file open.      */
-                goto out;
-        if (s == FIOFNF) {                      /* File not found.      */
-		emlwrite("No such file: %s",fname);
-		return(FALSE);
-        }
+        if ((s=ffropen(fname)) != FIOSUC){      /* Hard file open.      */
+		fioperr(s,fname);
+                return(FALSE);
+	}
+
         emlwrite("Inserting %s.", fname);
 
 	/* back up a line and save the mark here */
@@ -662,3 +864,74 @@ out:
                 return (FALSE);
         return (TRUE);
 }
+
+
+
+/*
+ * pico_fncomplete - pico's function to complete the given file name
+ */
+int pico_fncomplete(dir, fn, len)
+char *dir, *fn;
+int   len;
+{
+    char *p, *dlist, tmp[NFILEN], dtmp[NFILEN];
+    int   n, i, match = -1;
+#ifdef	DOS
+#define	FILECMP(x, y)	(toupper((unsigned char)(x))\
+				== toupper((unsigned char)(y)))
+#else
+#define	FILECMP(x, y)	((x) == (y))
+#endif
+
+    strcpy(dtmp, dir);
+    pfnexpand(dir = dtmp, NFILEN);
+    if(*fn && (dlist = p = getfnames(dir, fn, &n, NULL))){
+	memset(tmp, 0, sizeof(tmp));
+	while(n--){			/* any names in it */
+	    for(i = 0; fn[i] && FILECMP(p[i], fn[i]); i++)
+	      ;
+
+	    if(!fn[i]){			/* match or more? */
+		if(tmp[0]){
+		    for(; p[i] && FILECMP(p[i], tmp[i]); i++)
+		      ;
+
+		    match = !p[i] && !tmp[i];
+		    tmp[i] = '\0';	/* longest common string */
+		}
+		else{
+		    match = 1;		/* may be it!?! */
+		    strcpy(tmp,  p);
+		}
+	    }
+
+	    p += strlen(p) + 1;
+	}
+
+	free(dlist);
+    }
+
+    if(match >= 0){
+	strncpy(fn, tmp, len);
+	fn[len] = '\0';
+	if(match == 1){
+	    strcat(dir, S_FILESEP);
+	    strcat(dir, fn);
+	    if(isdir(dir, NULL))
+	      strcat(fn, S_FILESEP);
+	}
+    }
+
+    return(match == 1);
+
+}
+
+
+/*
+ * in_oper_tree - returns true if file "f" does reside in opertree
+ */
+in_oper_tree(f)
+char *f;
+{
+    return(!strncmp(opertree, f, strlen(opertree)));
+}
