@@ -14,38 +14,42 @@
  *
  * Please address all bugs and comments to "pine-bugs@cac.washington.edu"
  *
- * Copyright 1991-1994  University of Washington
- *
- *  Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee to the University of
- * Washington is hereby granted, provided that the above copyright notice
- * appears in all copies and that both the above copyright notice and this
- * permission notice appear in supporting documentation, and that the name
- * of the University of Washington not be used in advertising or publicity
- * pertaining to distribution of the software without specific, written
- * prior permission.  This software is made available "as is", and
- * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
- * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
- * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
- * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
- * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Pine and Pico are trademarks of the University of Washington.
- * No commercial use of these trademarks may be made without prior
- * written permission of the University of Washington.
  *
+ * Pine and Pico are registered trademarks of the University of Washington.
+ * No commercial use of these trademarks may be made without prior written
+ * permission of the University of Washington.
+ * 
+ * Pine, Pico, and Pilot software and its included text are Copyright
+ * 1989-1996 by the University of Washington.
+ * 
+ * The full text of our legal notices is contained in the file called
+ * CPYRIGHT, included with this distribution.
  */
 
 
-#undef OWNRECT
+#ifndef MSWIN_H
+#define MSWIN_H
+
+
+/*
+ * Def's to help Win32 port...
+ */
+#ifdef	WIN32
+#define	__far
+#define	_far
+#define	__export
+#define	signal	mswin_signal
+#endif
+
 
 /*
  * Equivalent to a windows handle.
  */
+#ifdef	WIN32
+typedef void	       *WINHAND;
+#else
 typedef unsigned int	WINHAND;
+#endif
 
 /*
  * Mouse input events. 
@@ -55,27 +59,62 @@ typedef struct {
 	int		button;
 	int		nRow;
 	int		nColumn;
-	unsigned long	flags;
+	int		keys;
+	int		flags;
 } MEvent;
 
 
+/*
+ * These define how mouse events get queued.
+ */
+#define MSWIN_MF_REPLACING	0x1000
+#define MSWIN_MF_REPEATING	0x2000
+
+
+typedef struct {
+	int		ch;
+	int		rval;
+	char		*name;
+	char		*label;
+	int		id;
+} MDlgButton;
+
+
+
 /*
  * Type of function expected by mswin_allowcopy and mswin_allowcopycut
  * and used in EditDoCopyData
  */
 typedef	int (*getc_t)(int pos);
 
+/*
+ * Type used by line up/down event handlers to move the body of the
+ * displayed text.
+ */
+typedef int (*scroll_t)(int action, long count);
 
-#define MEVENT_NONE		0x0000
-#define MEVENT_MOUSEUP		0x0001
-#define MEVENT_MOUSEDOWN	0x0002
-#define MEVENT_ANY		0x0003
+/*
+ * Callback used to fetch text for display in alternate window.  Text
+ * can be returned as either a pointer to a null terminated block of
+ * text (a string), with CRLF deliminating lines.  OR as a pointer to
+ * an array of pointers to lines, each line being a null terminated
+ * string. 
+ */
+typedef int (*gettext_t)(char *title, void **text, long *len, int *format);
 
-#define MEVENT_KEY_LBUTTON	0x0001		/* = MK_LBUTTON */
-#define MEVENT_KEY_MBUTTON	0x0010		/* = MK_MBUTTON */
-#define MEVENT_KEY_RBUTTON	0x0002		/* = MK_RBUTTON */
-#define MEVENT_KEY_SHIFT	0x0004		/* = MK_SHIFT */
-#define MEVENT_KEY_CONTROL	0x0008		/* = MK_CONTROL */
+
+/*
+ * Callback used for periodic callback.
+ */
+typedef void (*callback_t)(void);
+
+#define GETTEXT_TITLELEN	128
+#define GETTEXT_TEXT		1	/* 'text' is pointer to text. */
+#define GETTEXT_LINES		2	/* 'text' is pointer to array of
+					 *   pointers to lines. */
+
+
+#define PATH_MAX		128	/* Max size of a directory path. */
 
 
 #define MSWIN_RANGE_START	0x7001
@@ -101,17 +140,27 @@ typedef	int (*getc_t)(int pos);
 #define MSWIN_KEY_F11		0x7014
 #define MSWIN_KEY_F12		0x7015
 #define MSWIN_KEY_NODATA	0x7016
-#define MSWIN_RANGE_END		0x7016
+#define MSWIN_KEY_SCROLLUPLINE		0x7017
+#define MSWIN_KEY_SCROLLDOWNLINE	0x7018
+#define MSWIN_KEY_SCROLLUPPAGE		0x7019
+#define MSWIN_KEY_SCROLLDOWNPAGE	0x701a
+#define MSWIN_KEY_SCROLLTO		0x701b
+
+#define MSWIN_RANGE_END		0x701b
 
 #define	MSWIN_PASTE_DISABLE	0
 #define	MSWIN_PASTE_FULL	1
 #define	MSWIN_PASTE_LINE	2
 
-#define MSWIN_EXIT_DISABLE	0	/* disable the File/Exit menu. */
-#define MSWIN_EXIT_DENY		1	/* Enable the menu, but deny exit.*/
-#define MSWIN_EXIT_SENDCHAR	2	/* Enable the menu and exit by
-					 * sending a character to the app.*/
 
+#define MSWIN_CURSOR_ARROW	0
+#define MSWIN_CURSOR_BUSY	1
+
+/*
+ * Flags for mswin_displaytext
+ */
+#define MSWIN_DT_NODELETE	0x0001	/* Don't delete text when 
+					 * window closes. */
 
 
 
@@ -123,17 +172,31 @@ typedef	int (*getc_t)(int pos);
 void			AssertFail	(char *str, char *file, int line, int msgbox);
 
 
-int		WinGetMEvent (MEvent * pMouse);
-void		WinIdle (void);
+WINHAND		mswin_gethinstance();
+WINHAND		mswin_gethwnd ();
+int		mswin_getmouseevent (MEvent * pMouse);
 int		mswin_setdebug (int debug, FILE *debugfile);
+int		mswin_setdndcallback (int (*cb)());
+int		mswin_cleardndcallback (void);
 int		mswin_setresizecallback (int (*cb)());
 int		mswin_clearresizecallback (int (*cb)());
-int		mswin_sethelptext (char *pHelpText);
+int		mswin_sethelptext (char *title, char *pHelpText, 
+			size_t len, char **pHelpLines);
 int		mswin_setclosetext (char *pCloseText);
 int		mswin_setwindow (char *fontName, char *fontSize, 
 			char *fontStyle, char *windowPosition);
 int		mswin_getwindow (char *fontName, char *fontSize, 
 			char *fontStyle, char *windowPosition);
+void		mswin_setscrollrange (long max);
+void		mswin_setscrollpos (long pos);
+long		mswin_getscrollpos (void);
+long		mswin_getscrollto (void);
+void		mswin_setscrolcallback (scroll_t);
+int		mswin_setprintfont (char *fontName, char *fontSize, 
+			char *fontStyle);
+int		mswin_getprintfont (char *fontName, char *fontSize, 
+			char *fontStyle);
+int		mswin_yeild (void);
 int		mswin_charavail (void);
 int		mswin_getc (void);
 int		mswin_getc_fast (void);
@@ -156,32 +219,73 @@ int		mswin_eeop (void);
 int		mswin_beep (void);
 int		mswin_pause (int);
 int		mswin_flush (void);
+void		mswin_setcursor (int);
+int		mswin_messagebox (char *msg, int);
 int		mswin_allowpaste (int);
 int		mswin_allowcopycut (getc_t);
 int		mswin_allowcopy (getc_t);
-int		mswin_allowexit (int allow, char *denyReason, int quitCmd);
-WINHAND		mswin_inst2task (WINHAND hInst);
-int		mswin_exec_and_wait (char *whatsit, char *command);
-int		mswin_ontask_del (WINHAND hTask, char *path);
-int		mswin_print_ready (char *docDesc);
+int		mswin_newmailicon (void);
+void		mswin_newmailtext (char *t);
+void		mswin_newmaildone (void);
+void		mswin_menuitemclear (void);
+void		mswin_menuitemadd (int key, char *label, int menuitem, 
+			int flags);
+int		mswin_setwindowmenu (int menu);
+int		mswin_print_ready (WINHAND hWnd, char *docDesc);
 int		mswin_print_done (void);
 char *		mswin_print_error (int errorcode);
 int		mswin_print_char (int c);
 int		mswin_print_text (char *text);
 int		mswin_savefile (char *dir, char *fName, int nMaxFName);
 int		mswin_openfile (char *dir, char *fName, int nMaxFName);
+void		mswin_killbuftoclip (getc_t copyfunc);
 int		pico_nfcolor(char *);
 int		pico_nbcolor(char *);
 int		pico_rfcolor(char *);
 int		pico_rbcolor(char *);
 int		mswin_fflush (FILE *f);
+void		mswin_setperiodiccallback (callback_t periodiccb, 
+					long period);
+WINHAND		mswin_inst2task (WINHAND hInst);
+int		mswin_ontask_del (WINHAND hTask, char *path);
+int		mswin_exec_and_wait (char *whatsit, char *command);
+void		mswin_exec_err_msg (char *what, int status, char *buf, 
+			int buflen);
+int		mswin_onexit_del (char *path);
+int		mswin_set_quit_confirm (int);
+void		mswin_showhelpmsg (WINHAND hWnd, char **helplines);
+int		mswin_displaytext (char *title, char *pText, size_t textLen, 
+			char **pLines, int windRef, int flags);
+
 int		mswin_majorver();
 int		mswin_minorver();
 char	       *mswin_compilation_date();
 
 
+int		mswin_usedialog (void);
+int		mswin_dialog (char *prompt, char *string, int field_len, 
+			int append_current, int passwd, 
+			MDlgButton *button_list, char **help, unsigned flags);
+int		mswin_select (char *prompt, MDlgButton *button_list, 
+			int dflt, int on_ctrl_C, char **help, unsigned flags);
+int		mswin_yesno (char *prompt);
+
+
+#ifdef	MSC_MALLOC
 /*
- * Memory management stuff, also from mswin.c
+ * These definitions will disable the SEGHEAP allocation routines, in
+ * favor of the compliler libraries usual allocators.  This is useful
+ * when external debugging tools and the SEGHEAP debugging routines step
+ * on each other...
+ */
+#define MemAlloc(X) malloc(X)
+#define MemFree(X) free(X)
+#define MemRealloc(X,Y) realloc(X,Y)
+#define MemFreeAll()
+#define	MemDebug(X,Y)
+#else
+/*
+ * Memory management stuff, from msmem.c
  */
 typedef unsigned long		MemSize;
 typedef void __far	      *	MemPtr;
@@ -200,14 +304,20 @@ int		_MemFree (void __far *block, char __far *file, int line);
 void __far *	_MemRealloc (void __far *block, MemSize size, 
 					char __far * file, int line);
 MemSize		MemBlkSize (MemPtr block);
+void		MemFreeAll (void);
+void            MemFailSoon (MemSize);
+#endif
+
 
 
 /*
  * Signals that are not defined by MS C
  */
+#define SIGHUP	    1	/* Terminal hangup. */
 #define SIGINT      2   /* Ctrl-C sequence */
 #define SIGILL      4   /* illegal instruction - invalid function image */
 #define SIGSEGV     11  /* segment violation */
+#define SIGALRM     14  /* alarm clock */
 #define SIGTERM     15  /* Software termination signal from kill */
 #define SIGABRT     22  /* abnormal termination triggered by abort call */
 #define SIGWINCH    28	/* Change in window size. */
@@ -215,3 +325,4 @@ MemSize		MemBlkSize (MemPtr block);
 
 #define fflush	mswin_fflush
 
+#endif /* MSWIN_H */
