@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: os_unix.c,v 4.106 1996/04/10 05:23:36 hubert Exp $";
+static char rcsid[] = "$Id: os_unix.c,v 4.116 1996/07/12 18:13:23 mikes Exp $";
 #endif
 /*
  * Program:	Operating system dependent routines - Ultrix 4.1
@@ -56,7 +56,7 @@ static char rcsid[] = "$Id: os_unix.c,v 4.106 1996/04/10 05:23:36 hubert Exp $";
 #endif
 #if	defined(POSIX)
 #include	<termios.h>
-#if	defined(a32) || defined(cvx) || defined(osf)
+#if	defined(a32) || defined(a41) || defined(cvx) || defined(osf)
 #include	<sys/ioctl.h>
 #endif
 #else
@@ -81,7 +81,7 @@ static char rcsid[] = "$Id: os_unix.c,v 4.106 1996/04/10 05:23:36 hubert Exp $";
 #include	<sys/file.h>
 #include	<sys/types.h>
 #include	<sys/time.h>
-#if	defined(a32)
+#if	defined(a32) || defined(a41) || defined(dpx)
 #include	<sys/select.h>
 #endif
 
@@ -427,16 +427,13 @@ typahead()
 
 
 /*
- * GetKey - Read in a key.
- * Do the standard keyboard preprocessing. Convert the keys to the internal
- * character set.  Resolves escape sequences and returns no-op if global
- * timeout value exceeded.
+ * ReadyForKey - return true if there's no timeout or we're told input
+ *		 is available...
  */
-GetKey()
+ReadyForKey(timeout)
+    int timeout;
 {
-    int    c, status, cc;
-
-    if(timeout){
+    if(timeout > 0){
 	/*
 	 * simple use of select/poll here to handle requested timeouts
 	 * while waiting for keyboard input...
@@ -463,7 +460,7 @@ GetKey()
 #endif
 	if(rv < 0){
 	    if(errno == EINTR){		/* interrupted? */
-		return(NODATA);		/* return like we timed out */
+		return(0);		/* return like we timed out */
 	    }
 	    else{
 		emlwrite("\007Problem reading from keyboard!", NULL);
@@ -471,9 +468,26 @@ GetKey()
 	    }
 	}
 	else if(rv == 0)
-	  return(NODATA);		/* we really did time out */
+	  return(0);			/* we really did time out */
     }
 
+    return(1);
+}
+
+
+/*
+ * GetKey - Read in a key.
+ * Do the standard keyboard preprocessing. Convert the keys to the internal
+ * character set.  Resolves escape sequences and returns no-op if global
+ * timeout value exceeded.
+ */
+GetKey()
+{
+    int    c, status, cc;
+
+    if(!ReadyForKey(timeout))
+      return(NODATA);
+
     switch(status = kbseq(pico_kbesc, term.t_getchar, &c)){
       case 0: 	/* regular character */
 	break;
@@ -481,7 +495,7 @@ GetKey()
       case K_DOUBLE_ESC:
 	c = (*term.t_getchar)();
 
-	if(isdigit(c)){
+	if(isdigit((unsigned char)c)){
 	    int n = 0, i = c - '0';
 
 	    if(!strchr("012", c))
@@ -490,7 +504,7 @@ GetKey()
 	    while(n++ < 2){
 		c = (*term.t_getchar)();
 					/* remember Horner? */
-		if(!isdigit(c) || (n == 1 && i == 2 && c > '5'))
+		if(!isdigit((unsigned char)c) || (n == 1 && i == 2 && c > '5'))
 		  return(c);		/* bogus literal char value */
 
 		i = (i * 10) + (c - '0');
@@ -500,11 +514,12 @@ GetKey()
 	    break;
 	}
 	else{
-	    if(islower(c))	/* canonicalize c */
-	      c = toupper(c);
+	    if(islower((unsigned char)c))	/* canonicalize c */
+	      c = toupper((unsigned char)c);
 
-	    return((isalpha(c) || c == '@' || (c >= '[' && c <= '_'))
-		   ? (CTRL | c) : c);
+	    return((isalpha((unsigned char)c) || c == '@'
+		    || (c >= '[' && c <= '_'))
+		    ? (CTRL | c) : ((c == ' ') ? (CTRL | '@') : c));
 	}
 
 #ifdef MOUSE
@@ -560,15 +575,21 @@ GetKey()
 	return(status);
 
       case K_SWALLOW_TIL_Z:
-	status = NODATA;
+	status = BADESC;
       case K_SWALLOW_UP:
       case K_SWALLOW_DOWN:
       case K_SWALLOW_LEFT:
       case K_SWALLOW_RIGHT:
-	for(cc = 0; cc != 'z' && cc != '~';)
-	  cc = (*term.t_getchar)() & 0x7f;
+	do
+	  if(!ReadyForKey(2)){
+	      status = BADESC;
+	      break;
+	  }
+	while(!strchr("~qz", (*term.t_getchar)()));
 
-	return(status);
+	return((status == BADESC)
+		? status
+		: status - (K_SWALLOW_UP - K_PAD_UP));
 	break;
 
       case K_KERMIT:
@@ -859,7 +880,7 @@ alt_editor(f, n)
 	    return(-1);
 	}
 
-	while(isspace(*cp))
+	while(isspace((unsigned char)(*cp)))
 	  if(*cp != '\0')
 	    cp++;
 	  else
@@ -867,7 +888,7 @@ alt_editor(f, n)
 
 	args[i] = cp;
 
-	while(!isspace(*cp))
+	while(!isspace((unsigned char)(*cp)))
 	  if(*cp != '\0')
 	    cp++;
 	  else
@@ -1535,11 +1556,10 @@ int  len;
 		int i, off = strlen(opertree);
 
 		for(shft = strchr(name, '\0'); shft >= name; shft--)
-		  shft[off] = *shft;
-		
-		off--;
-		for(shft=name, i=0; i < off; i++)
-		  shft[i] = opertree[i];
+		  shft[off+1] = *shft;
+
+		strncpy(name, opertree, off);
+		name[off] = '/';
 	    }
 	    else{
 		for(shft = strchr(name, '\0'); shft >= name; shft--)
@@ -1787,8 +1807,9 @@ ffclose()
 {
     extern FILE *ffp;
 
-    if (fclose(ffp) != FALSE) {
-        emlwrite("Error closing file", NULL);
+    fflush(ffp);
+    if (fclose(ffp) == EOF) {
+        emlwrite("\007Error closing file: %s", errstr(errno));
         return(FIOERR);
     }
 
@@ -1796,6 +1817,104 @@ ffclose()
 }
 
 
+#if defined(asv) || defined(sco)
+
+/*
+ *	Tim Rice	tim@trr.metro.net	Mon Jun  3 16:57:26 PDT 1996
+ *
+ *	a quick and dirty trancate()
+ *	Altos System V (5.3.1) does not have one
+ *	neither does SCO Open Server Enterprise 3.0
+ *
+ */
+
+truncate(fn, size)
+char    *fn ;
+long size ;
+{
+	int    fdes;
+	int    rc = -1 ;
+
+	if((fdes = open(fn, O_RDWR)) != -1)
+	{
+		rc = chsize(fdes, size) ;
+
+		if(close(fdes))
+			return(-1) ;
+	}
+	return(rc) ;
+}
+
+#endif /* defined(asv) || defined(sco) */
+
+
+/*
+ * ffelbowroom - make sure the destination's got enough room to receive
+ *		 what we're about to write...
+ */
+ffelbowroom(fn)
+char    *fn;
+{
+    register LINE *lp;
+    register long  n;
+    struct   stat  sbuf;
+    FILE    *fp;
+    int	     x, vapor = FALSE;
+    char    *s, *err_str = NULL;
+#define	EXTEND_BLOCK	1024
+
+    /* Figure out how much room do we need */
+    /* first, what's total */
+    for(n=0L, lp=lforw(curbp->b_linep); lp != curbp->b_linep; lp=lforw(lp))
+      n += llength(lp) + 1;
+
+    if(stat(fn, &sbuf) < 0){
+	if(errno == ENOENT)
+	  vapor = TRUE;
+	else
+	  return(FALSE);
+    }
+    else
+      n -= sbuf.st_size;
+
+    if(n > 0L){			/* must be growing file, extend it */
+	if(s = (char *) malloc(EXTEND_BLOCK * sizeof(char))){
+	    memset(s, 0, EXTEND_BLOCK);
+	    if(fp = fopen(fn, "a")){
+		for( ; n > 0; n -= EXTEND_BLOCK){
+		    x = (n < EXTEND_BLOCK) ? EXTEND_BLOCK - n : EXTEND_BLOCK;
+		    if(fwrite(s, sizeof(char), x, fp) != x){
+			err_str = errstr(errno);
+			break;
+		    }
+		}
+		    
+		if(fclose(fp) == EOF)
+		  err_str = errstr(errno);
+	    }
+	    else
+	      err_str = errstr(errno);
+
+	    free(s);
+	}
+	else
+	  err_str = "No memory to extend file";
+    }
+
+    if(err_str){		/* clean up */
+	if(vapor)
+	  unlink(fn);
+	else
+	  truncate(fn, sbuf.st_size);
+
+	emlwrite("No room for file: %s", err_str);
+	return(FALSE);
+    }
+
+    return(TRUE);
+}
+
+
 /*
  * P_open - run the given command in a sub-shell returning a file pointer
  *	    from which to read the output
@@ -1904,11 +2023,11 @@ QcompType *s1, *s2;
 
     a = *(char **)s1;
     b = *(char **)s2;
-    while(toupper(*a) == toupper(*b++))
+    while(toupper((unsigned char)(*a)) == toupper((unsigned char)(*b++)))
 	if(*a++ == '\0')
 	  return(0);
 
-    return(toupper(*a) - toupper(*--b));
+    return(toupper((unsigned char)(*a)) - toupper((unsigned char)(*--b)));
 }
 
 
