@@ -1,5 +1,5 @@
 /*
- * $Id: os_unix.h,v 4.34 1996/04/11 04:09:12 hubert Exp $
+ * $Id: os_unix.h,v 4.35 1996/06/12 20:47:11 mikes Exp $
  *
  * Program:	Operating system dependent routines - Ultrix 4.1
  *
@@ -147,7 +147,7 @@ typedef struct
 #endif
 
 /* memmove() is a built-in for AIX 3.2 xlc. */
-#if	defined (a32) || defined(COHERENT)
+#if	defined (a32) || defined(a41) || defined(COHERENT)
 #define bcopy(a,b,s) memmove (b, a, s)
 #endif
 
