@@ -1,5 +1,5 @@
 /*
- * $Id: pico.h,v 4.71 1996/03/22 23:40:29 hubert Exp $
+ * $Id: pico.h,v 4.75 1996/07/09 22:49:32 mikes Exp $
  *
  * Program:	pico.h - definitions for Pine's composer library
  *
@@ -300,6 +300,7 @@ extern	void	kbdestroy();
 #define	P_HIBITIGN	MDHBTIGN	/* ignore chars with hi bit set    */
 #define	P_DOTFILES	MDDOTSOK	/* browser displays dot files	   */
 #define	P_ABOOK		MDHDRONLY	/* called as address book editor   */
+#define	P_ALLOW_GOTO	MDGOTO		/* support "Goto" in file browser  */
 
 /*
  * definitions for various PICO modes 
@@ -328,13 +329,14 @@ extern	void	kbdestroy();
 #define	MDMOUSE		0x00200000	/* allow mouse (part. in xterm) */
 #define	MDONECOL	0x00400000	/* single column browser        */
 #define	MDHDRONLY	0x00800000	/* header editing exclusively   */
+#define	MDGOTO		0x01000000	/* support "Goto" in file browser */
 
 /*
  * Main defs 
  */
 #ifdef	maindef
 PICO	*Pmaster = NULL;		/* composer specific stuff */
-char	*version = "2.7";		/* PICO version number */
+char	*version = "2.9";		/* PICO version number */
 
 #else
 extern	PICO *Pmaster;			/* composer specific stuff */
@@ -383,8 +385,8 @@ extern	char *version;			/* pico version! */
 #define K_SWALLOW_TIL_Z		0x0827
 #define K_SWALLOW_UP		0x0828
 #define K_SWALLOW_DOWN		0x0829
-#define K_SWALLOW_LEFT		0x0830
-#define K_SWALLOW_RIGHT		0x0831
+#define K_SWALLOW_RIGHT		0x0830
+#define K_SWALLOW_LEFT		0x0831
 #define K_KERMIT		0x0832
 #define NODATA			0x08FF
  
