@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: spell.c,v 4.8 1994/03/24 18:22:57 mikes Exp $";
+static char rcsid[] = "$Id: spell.c,v 4.14 1996/04/26 16:03:18 hubert Exp $";
 #endif
 /*
  * Program:	spell.c
@@ -15,37 +15,26 @@ static char rcsid[] = "$Id: spell.c,v 4.8 1994/03/24 18:22:57 mikes Exp $";
  *
  * Please address all bugs and comments to "pine-bugs@cac.washington.edu"
  *
- * Copyright 1991-1994  University of Washington
  *
- *  Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee to the University of
- * Washington is hereby granted, provided that the above copyright notice
- * appears in all copies and that both the above copyright notice and this
- * permission notice appear in supporting documentation, and that the name
- * of the University of Washington not be used in advertising or publicity
- * pertaining to distribution of the software without specific, written
- * prior permission.  This software is made available "as is", and
- * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
- * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
- * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
- * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
- * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- * Pine and Pico are trademarks of the University of Washington.
- * No commercial use of these trademarks may be made without prior
- * written permission of the University of Washington.
+ * Pine and Pico are registered trademarks of the University of Washington.
+ * No commercial use of these trademarks may be made without prior written
+ * permission of the University of Washington.
+ * 
+ * Pine, Pico, and Pilot software and its included text are Copyright
+ * 1989-1996 by the University of Washington.
+ * 
+ * The full text of our legal notices is contained in the file called
+ * CPYRIGHT, included with this distribution.
  *
  */
 
 #include <stdio.h>
 #include "osdep.h"
+#include "pico.h"
 #include "estruct.h"
 #ifdef	SPELLER
 #include "edef.h"
-#include "pico.h"
+
 
 #ifdef	ANSI
     void chword(char *, char *);
@@ -111,10 +100,14 @@ spell(f, n)
     setimark(0, 1);
     emlwrite("Checking spelling..."); 		/* greetings */
 
+    if(Pmaster && Pmaster->alt_spell)
+      return(alt_editor(1, 0));			/* f == 1 means fork speller */
+
     if((fn = writetmp(0, 0)) == NULL){
 	emlwrite("Can't write temp file for spell checker");
 	return(-1);
     }
+
     if((sp = (char *)getenv("SPELL")) == NULL)
       sp = SPELLER;
 
@@ -215,7 +208,7 @@ chword(wb, cb)
   char *wb;					/* word buffer */
   char *cb;					/* changed buffer */
 {
-    ldelete(strlen(wb), 0);			/* not saved in kill buffer */
+    ldelete((long) strlen(wb), 0);		/* not saved in kill buffer */
     while(*cb != '\0')
       linsert(1, *cb++);
 
@@ -257,9 +250,9 @@ movetoword(w)
 	 * We want to minimize the number of substrings that we report
 	 * as matching a misspelled word...
 	 */
-	if(off == 0 || !isalpha(lgetc(lp, off - 1).c)){
+	if(off == 0 || !isalpha((unsigned char)lgetc(lp, off - 1).c)){
 	    off += strlen(w);
-	    if((!isalpha(lgetc(lp, off).c) || off == llength(lp)) 
+	    if((!isalpha((unsigned char)lgetc(lp, off).c) || off == llength(lp))
 	       && lgetc(lp, 0).c != '>'){
 		ret = TRUE;
 		break;
