@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: word.c,v 4.20 1996/03/15 07:41:11 hubert Exp $";
+static char rcsid[] = "$Id: word.c,v 4.23 1996/05/15 00:38:41 mikes Exp $";
 #endif
 /*
  * Program:	Word at a time routines
@@ -59,7 +59,7 @@ wrapword()
       return(FALSE);
 
     for(bp = cnt = i = 0; cnt < llength(curwp->w_dotp) && !bp; cnt++, i++){
-	if(isspace(lgetc(curwp->w_dotp, cnt).c)){
+	if(isspace((unsigned char) lgetc(curwp->w_dotp, cnt).c)){
 	    first = 0;
 	    if(lgetc(curwp->w_dotp, cnt).c == TAB)
 	      while(i+1 & 0x07)
@@ -82,6 +82,20 @@ wrapword()
     if(!lnewline())			/* break the line */
       return(FALSE);
 
+    /* clean up trailing whitespace from line above ... */
+    if(backchar(FALSE, 1)){
+	while(llength(curwp->w_dotp) > 0 && backchar(FALSE, 1)
+	      && isspace((unsigned char) lgetc(curwp->w_dotp, curwp->w_doto).c)
+	      && (cnt > 0 || cnt < -1)){
+	    forwdel(FALSE, 1);
+	    if(cnt < 0)
+	      ++cnt;
+	}
+
+	gotoeol(FALSE, 1);
+	forwchar(FALSE, 1);		/* goto first char of next line */
+    }
+
     /*
      * if there's a line below, it doesn't start with whitespace 
      * and there's room for this line...
@@ -89,7 +103,7 @@ wrapword()
     if(!(curbp->b_flag & BFWRAPOPEN)
        && lforw(curwp->w_dotp) != curbp->b_linep 
        && llength(lforw(curwp->w_dotp)) 
-       && !isspace(lgetc(lforw(curwp->w_dotp), 0).c)
+       && !isspace((unsigned char) lgetc(lforw(curwp->w_dotp), 0).c)
        && (llength(curwp->w_dotp) + llength(lforw(curwp->w_dotp)) < fillcol)){
 	gotoeol(0, 1);			/* then pull text up from below */
 	if(lgetc(curwp->w_dotp, curwp->w_doto - 1).c != ' ')
@@ -99,8 +113,9 @@ wrapword()
 	gotobol(0, 1);
     }
 
-    curbp->b_flag &= ~BFWRAPOPEN;           /* don't open new line next wrap */
-    if(!forwchar(0, cnt < 0 ? cnt-1 : cnt)) /* restore dot (account for NL)  */
+    curbp->b_flag &= ~BFWRAPOPEN;	/* don't open new line next wrap */
+					/* restore dot (account for NL)  */
+    if(cnt && !forwchar(0, cnt < 0 ? cnt-1 : cnt))
       return(FALSE);
 
     return(TRUE);
@@ -359,7 +374,24 @@ delbword(f, n)
 inword()
 {
     return(curwp->w_doto < llength(curwp->w_dotp)
-	   && isalnum(lgetc(curwp->w_dotp, curwp->w_doto).c));
+	   && isalnum((unsigned char)lgetc(curwp->w_dotp, curwp->w_doto).c));
+}
+
+
+/*
+ * Return TRUE if whatever starts the line matches the quote string
+ */
+quote_match(q, l)
+    char *q;
+    LINE *l;
+{
+    register int i;
+
+    for(i = 0; i <= llength(l) && q[i]; i++)
+      if(q[i] != lgetc(l, i).c)
+	return(0);
+
+    return(1);
 }
 
 
@@ -473,8 +505,9 @@ int f, n;	/* deFault flag and Numeric argument */
 	curwp->w_dotp = lforw(bqline);
 	curwp->w_doto = 0;
 	do
-	  forwdel(FALSE, (quote_len > llength(curwp->w_dotp))
-			   ? llength(curwp->w_dotp) : quote_len);
+	  if(quote_match(quote, curwp->w_dotp))
+	    forwdel(FALSE, (quote_len > llength(curwp->w_dotp))
+			     ? llength(curwp->w_dotp) : quote_len);
 	while((curwp->w_dotp = lforw(curwp->w_dotp)) != lback(eqline));
 
 	/* got to the line we started on */
