@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: context.c,v 4.36 1994/10/11 16:37:08 mikes Exp $";
+static char rcsid[] = "$Id: context.c,v 4.49 1996/04/26 16:06:16 hubert Exp $";
 #endif
 /*
  * Program:	Mailbox Context Management
@@ -12,35 +12,22 @@ static char rcsid[] = "$Id: context.c,v 4.36 1994/10/11 16:37:08 mikes Exp $";
  *		Seattle, WA  98195
  *		Internet: MRC@CAC.Washington.EDU
  *
- * Date:	27 February 1993
- * Last Edited:	27 February 1993
  *
- * Copyright 1993-1994 by the University of Washington
- * 
+ * Pine and Pico are registered trademarks of the University of Washington.
+ * No commercial use of these trademarks may be made without prior written
+ * permission of the University of Washington.
+ *
+ * Pine, Pico, and Pilot software and its included text are Copyright
+ * 1989-1996 by the University of Washington.
+ *
+ * The full text of our legal notices is contained in the file called
+ * CPYRIGHT, included with this distribution.
+ *
  *
  * USENET News reading additions in part by L Lundblade / NorthWestNet, 1993
  * lgl@nwnet.net
- *
- *  Permission to use, copy, modify, and distribute this software and its
- * documentation for any purpose and without fee is hereby granted, provided
- * that the above copyright notice appears in all copies and that both the
- * above copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the University of Washington not be
- * used in advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  This software is made
- * available "as is", and
- * THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
- * WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
- * NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
- * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
- * (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
  */
 
-#include "../c-client/misc.h"
 #include "headers.h"
 
 char *current_context = NIL;	/* current context for FIND */
@@ -111,7 +98,9 @@ context_fqn_to_an (f, context_string, fqn)
      */
 
     if((ret=((!prefix) ? fqn : strstr(fqn, prefix))) != NULL){
-	fqn = ret;
+	if(prefix && !strcmp(fqn = ret, prefix))
+	  return;			/* no folder name! */
+
 	fqnlen = strlen(fqn);
     }
 
@@ -289,8 +278,8 @@ context_isambig (s)
     char c = (*s == '*') ? *(s+1) : *s;
     if(c == '\0' && *s == '*')	/* special case! */
       return(1);
-#ifdef	DOS
-    else if (c != '{' && isalpha(*s) && *(s+1) == ':')
+#if defined(DOS) || defined(OS2)
+    else if (c != '{' && isalpha((unsigned char)*s) && *(s+1) == ':')
       return(0);
 #endif
     else
@@ -306,7 +295,8 @@ context_isambig (s)
        * The good news is, THIS MUST AND WILL BE REMOVED
        * WITH THE COMING IMAP4 SUPPORT!!!!
        */
-      return(!(c == '{' || c == '/' || c == ':' || c == '[' || c == '#'));
+      return(!(c == '{' || c == '/' || c == ':' || c == '[' || c == '#'
+	       || strucmp(s, ps_global->inbox_name) == 0));
 }
 
 /* Context Manager find list of subscribed mailboxes
@@ -476,20 +466,7 @@ context_create (context, stream, mailbox)
   else
     context_apply (tmp,context,mailbox);/* build fully-qualified name */
 				/* try to do the create */
-  if(!stream){			/* stream not specified? */
-     /* This code is called only when the target of a create is a
-	remote mailbox.  We depend upon this, because mail_open
-	requires a local mailbox to exist even if all you want is
-	a prototype back.  */
-     if (!(stream = mail_open (NIL,tmp,OP_PROTOTYPE))){
-	 sprintf (tmp,"Unable to infer network server from context: %s",
-		  mailbox);
-	 mm_log (tmp,ERROR);
-	 return NIL;
-     }
-  }
-
-  return mail_create (stream,tmp);
+  return(cntxt_allowed(tmp) ? mail_create(stream,tmp) : 0L);
 }
 
 
@@ -519,7 +496,7 @@ context_open (context, old, name, opt)
       s = tmp;
   }
   else s = name;		/* fully-qualified name */
-  return mail_open (old,s,opt);	/* do the normal c-client action */
+  return(cntxt_allowed(s) ? mail_open(old,s,opt) : (MAILSTREAM *)NULL);
 }
 
 
@@ -553,7 +530,8 @@ context_rename (context, stream, old, new)
       t = tmp;
   }
   else t = new;		/* fully-qualified name */
-  return mail_rename (stream,s,t);
+  return((cntxt_allowed(s) && cntxt_allowed(t)) ? mail_rename(stream,s,t)
+						 : 0L);
 }
 
 
@@ -578,7 +556,7 @@ context_delete (context, stream, name)
       s = tmp;
   }
   else s = name;		/* fully-qualified name */
-  return mail_delete (stream,s);
+  return(cntxt_allowed(s) ? mail_delete(stream,s) : 0L);
 }
 
 
@@ -605,7 +583,7 @@ context_append (context, stream, name, msg)
       s = tmp;
   }
   else s = name;		/* fully-qualified name */
-  return mail_append (stream,s,msg);
+  return(cntxt_allowed(s) ? mail_append(stream,s,msg) : 0L);
 }
 
 
@@ -633,7 +611,7 @@ long context_append_full (context, stream, name, flags, date, msg)
       s = tmp;
   }
   else s = name;		/* fully-qualified name */
-  return mail_append_full (stream,s,flags,date,msg);
+  return(cntxt_allowed(s) ? mail_append_full(stream,s,flags,date,msg) : 0L);
 }
 
 
@@ -662,7 +640,7 @@ long context_copy (context, stream, sequence, name)
 	strcpy(s = tmp, "INBOX");	/* presume "inbox" ala c-client */
   }
 
-  return (s ? mail_copy(stream, sequence, s) : 0L);
+  return((s && cntxt_allowed(s)) ? mail_copy(stream,sequence,s) : 0L);
 }
 
 
@@ -758,6 +736,9 @@ context_same_driver(context, name, stream)
   }
   else s = name;		/* fully-qualified name */
 
+  if(!cntxt_allowed(s))
+    return((MAILSTREAM *)NULL);
+
   /* IF we're given a stream to compare *and* it's not remote *and*
    * the given mailbox name isn't remote *and* we know the driver
    * the given mailbox will be in *and* the drivers are the same
@@ -773,7 +754,7 @@ context_same_driver(context, name, stream)
 	      || (*stream->mailbox == '*' && *(stream->mailbox + 1) == '{'))
 	     || (*s == '{' || (*s == '*' && *(s+1) == '{')))
 	&& ((proto_stream = mail_open(NIL,s,OP_PROTOTYPE))
-	    || (proto_stream = default_driver(context)))
+	    || (*s != '#' && (proto_stream = default_proto())))
 	&& !strcmp(stream->dtb->name, proto_stream->dtb->name));
   ps_global->try_to_create = 0;
 
