@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: mailpart.c,v 4.113 1996/03/25 20:20:57 mikes Exp $";
+static char rcsid[] = "$Id: mailpart.c,v 4.124 1996/05/29 21:47:03 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -101,6 +101,8 @@ ATDISP_S *first_attline PROTO((ATDISP_S *));
 void	  attachment_screen_redrawer PROTO(());
 int	  attachment_screen_updater PROTO((struct pine *, ATDISP_S *, \
 					   ATT_SCREEN_S *));
+int	  init_att_progress PROTO((char *, MAILSTREAM *, BODY *));
+long	  save_att_piped PROTO((int));
 int	  save_att_percent PROTO(());
 int	  df_trigger_cmp PROTO((long, char *));
 int	  df_trigger_cmp_text PROTO((char *, char *));
@@ -321,8 +323,8 @@ attachment_screen(ps, msgmap)
 	clear_mfunc(mouse_in_content);
 #endif
 
-        if(ch <= 0xff && isupper(ch))
-          ch = tolower(ch);
+        if(ch <= 0xff && isupper((unsigned char)ch))
+          ch = tolower((unsigned char)ch);
 
 	if(km_popped)
 	  switch(ch){
@@ -772,10 +774,57 @@ first_attline(p)
 }
 
 
+int
+init_att_progress(msg, stream, body)
+    char       *msg;
+    MAILSTREAM *stream;
+    BODY	*body;
+{
+    if(save_att_length = body->size.bytes){
+	/* if there are display filters, factor in extra copy */
+	if(body->type == TYPETEXT && ps_global->VAR_DISPLAY_FILTERS)
+	  save_att_length += body->size.bytes;
+
+	/* if remote folder and segment not cached, factor in IMAP fetch */
+	if(stream && stream->mailbox && IS_REMOTE(stream->mailbox)
+	   && !((body->type == TYPETEXT && body->contents.text)
+		|| (body->type == TYPEMESSAGE && body->contents.msg.text)
+		|| body->contents.binary))
+	  save_att_length += body->size.bytes;
+
+	gf_filter_init();			/* reset counters */
+	pine_gets_bytes(1);
+	save_att_piped(1);
+	return(busy_alarm(1, msg, save_att_percent, 1));
+    }
+
+    return(0);
+}
+
+
+long
+save_att_piped(reset)
+    int reset;
+{
+    static long x;
+    long	y;
+
+    if(reset){
+	x = y = 0L;
+    }
+    else if((y = gf_bytes_piped()) >= x){
+	x = y;
+	y = 0;
+    }
+
+    return(x + y);
+}
+
+
 int
 save_att_percent()
 {
-    int i = (int) (((pine_gets_bytes(0) + gf_bytes_piped()) * 100)
+    int i = (int) (((pine_gets_bytes(0) + save_att_piped(0)) * 100)
 							   / save_att_length);
     return(min(i, 100));
 }
@@ -871,8 +920,8 @@ save_attachment(qline, msgno, a)
 			    FB_SAVE);
 
 	    if(r == 1){
-		strcat(full_filename, "/");
-		strcat(full_filename, filename);
+		build_path (tmp_20k_buf, full_filename, filename);
+		strcpy (full_filename, tmp_20k_buf);
 	    }
 	    else
 	      continue;
@@ -932,14 +981,12 @@ save_attachment(qline, msgno, a)
 	    tfp = temp_nam(NULL, "pd");
 	    dprint(1, (debugfile, "Download attachment called!\n"));
 	    if(store = so_get(FileStar, tfp, WRITE_ACCESS)){
+		sprintf(prompt_buf, "Saving to \"%.50s\"", full_filename);
+		we_cancel = init_att_progress(prompt_buf,
+					      ps_global->mail_stream,
+					      a->body);
+
 		gf_set_so_writec(&pc, store);
-
-		if(save_att_length = 2 * a->body->size.bytes){
-		    pine_gets_bytes(1);
-		    sprintf(prompt_buf, "Saving to \"%.50s\"", full_filename);
-		    we_cancel = busy_alarm(1, prompt_buf, save_att_percent, 1);
-		}
-
 		if(err = detach(ps_global->mail_stream, msgno, a->body,
 				a->number, &len, pc, NULL))
 		  q_status_message2(SM_ORDER | SM_DING, 3, 5,
@@ -1083,17 +1130,13 @@ save_attachment(qline, msgno, a)
 	return;
     }
 
+    sprintf(prompt_buf, "Saving to \"%.50s\"", full_filename);
+    we_cancel = init_att_progress(prompt_buf, ps_global->mail_stream, a->body);
+
     gf_set_so_writec(&pc, store);
-
-    if(save_att_length = 2 * a->body->size.bytes){
-	pine_gets_bytes(1);
-	sprintf(prompt_buf, "Saving to \"%.50s\"", full_filename);
-	we_cancel = busy_alarm(1, prompt_buf, save_att_percent, 1);
-    }
-
     err = detach(ps_global->mail_stream,msgno,a->body,a->number,&len,pc,NULL);
 
-    if(save_att_length && we_cancel)
+    if(we_cancel)
       cancel_busy_alarm(0);
 
     if(!err){
@@ -1140,7 +1183,7 @@ display_attachment(msgno, a)
     STORE_S *store;
     gf_io_t  pc;
     char    *err;
-    int      we_cancel =0;
+    int      we_cancel = 0;
 #if !defined(DOS) && !defined(OS2)
     char     prefix[8];
 #endif
@@ -1186,7 +1229,23 @@ display_attachment(msgno, a)
 
     /*------ Write the image to a temporary file ------*/
 #if defined(DOS) || defined(OS2)
-    filename = temp_nam(NULL, "im");
+    /*
+     *  Dos applications are particular about the file name extensions
+     *  Try to look up the expected extension from the mime.types file.
+     */
+    {  char ext[32];
+       char mtype[128];
+       
+     ext[0] = '\0';
+     strcpy (mtype, body_type_names(a->body->type));
+     if (a->body->subtype) {
+	 strcat (mtype, "/");
+	 strcat (mtype, a->body->subtype);
+     }
+
+     set_mime_extension_by_type (ext, mtype);
+     filename = temp_nam_ext(NULL, "im", ext);
+   }
 #else
     sprintf(prefix, "img-%.3s", (a->body->subtype) ? a->body->subtype : "unk");
     filename = temp_nam(NULL, prefix);
@@ -1199,22 +1258,23 @@ display_attachment(msgno, a)
         return(1);
     }
 
-    if(save_att_length = 2 * a->body->size.bytes){
+
+    if(a->body->size.bytes){
 	char msg_buf[128];
 
-	pine_gets_bytes(1);
 	sprintf(msg_buf, "Decoding %s%.50s%s%s",
 		a->description ? "\"" : "",
 		a->description ? a->description : "attachment number ",
 		a->description ? "" : a->number,
 		a->description ? "\"" : "");
-	we_cancel = busy_alarm(1, msg_buf, save_att_percent, 1);
+	we_cancel = init_att_progress(msg_buf, ps_global->mail_stream,
+				      a->body);
     }
 
     gf_set_so_writec(&pc, store);
     err = detach(ps_global->mail_stream,msgno,a->body,a->number,NULL,pc,NULL);
 
-    if(save_att_length && we_cancel)
+    if(we_cancel)
       cancel_busy_alarm(0);
 
     if(err){
@@ -1617,7 +1677,7 @@ pipe_attachment(msgno, a)
 				   (flags&PIPE_RESET) ? NULL : &resultfilename,
 				   NULL, flags)){
 		gf_io_t  pc;		/* wire up a generic putchar */
-		gf_set_writec(&pc, syspipe->ofilep, 0L, FileStar);
+		gf_set_writec(&pc, syspipe->out.f, 0L, FileStar);
 
 		/*------ Write the image to a temporary file ------*/
 		if(raw){		/* pipe raw text */
@@ -1650,8 +1710,10 @@ pipe_attachment(msgno, a)
 				      (void *)NULL);
 #endif	/* DOS */
 
-		    if(!capture)
+		    if(capture)
 		      we_cancel = busy_alarm(1, NULL, NULL, 0);
+		    else
+		      suspend_busy_alarm();
 
 		    if(!err
 		       && !(contents = mail_fetchbody(ps_global->mail_stream,
@@ -1683,8 +1745,12 @@ pipe_attachment(msgno, a)
 			mail_gc(ps_global->mail_stream, GC_TEXTS);
 		    }
 #endif
-		    if(we_cancel)
-		      cancel_busy_alarm(0);
+		    if(capture){
+			if(we_cancel)
+			  cancel_busy_alarm(0);
+		    }
+		    else
+		      resume_busy_alarm();
 		}
 		else{
 		    /* BUG: there's got to be a better way */
@@ -1779,8 +1845,10 @@ detach(stream, msg_no, body, part_no, len, pc, aux_filters)
     gf_io_t        gc;
     SourceType     src = CharStar;
     static char    err_string[100];
-#if	defined(DOS) && !defined(WIN32)
+#ifdef	DOS
+#ifndef	WIN32
     char	  *tmpfile_name = NULL;
+#endif
     extern unsigned char  *xlate_to_codepage;
 #endif
 
@@ -1808,6 +1876,8 @@ detach(stream, msg_no, body, part_no, len, pc, aux_filters)
     if(!ps_global->print)
       we_cancel = busy_alarm(1, NULL, NULL, 0);
 
+    gf_filter_init();			/* prepare for filtering! */
+
     /*
      * go grab the requested body part
      */
@@ -1820,8 +1890,6 @@ detach(stream, msg_no, body, part_no, len, pc, aux_filters)
 
     rv = (length) ? length : 1L;
 
-    gf_filter_init();			/* prepare to filter it! */
-
     switch(body->encoding) {		/* handle decoding */
       case ENC7BIT:
       case ENC8BIT:
@@ -1876,7 +1944,7 @@ detach(stream, msg_no, body, part_no, len, pc, aux_filters)
     if(body->type == TYPETEXT || (body->type == TYPEMESSAGE
 				  && !strucmp(body->subtype, "rfc822"))){
 	gf_link_filter(gf_nvtnl_local);
-#if	defined(DOS) && !defined(WIN32)
+#ifdef	DOS
 	/*
 	 * When detaching a text part AND it's US-ASCII OR it
 	 * matches what the user's defined as our charset,
@@ -1938,7 +2006,8 @@ detach(stream, msg_no, body, part_no, len, pc, aux_filters)
 		for(p = aux_filters; *p; p++)	/* count aux_filters */
 		  ;
 
-		p = aux = fs_get(((p - aux_filters) + 3) * sizeof(filter_t));
+		p = aux = (filter_t *)fs_get(((p - aux_filters) + 3)
+					      * sizeof(filter_t));
 		*p++ = gf_local_nvtnl;
 		for(; *aux_filters; p++, aux_filters++)
 		  *p = *aux_filters;
@@ -2185,7 +2254,7 @@ int
 df_trigger_cmp_lwsp(s1, s2)
     char *s1, *s2;
 {
-    while(*s1 && isspace(*s1))
+    while(*s1 && isspace((unsigned char)*s1))
       s1++;
 
     return((*s1) ? (!strncmp(s1, s2, strlen(s2)) ? 1 : -1) : 0);
@@ -2220,7 +2289,7 @@ df_valid_command(cmd)
      * copy cmd to build expanded path if necessary.
      */
     for(i = 0; cpath[i] = (*cmd)[i]; i++)
-      if(isspace((*cmd)[i])){
+      if(isspace((unsigned char)(*cmd)[i])){
 	  cpath[i] = '\0';		/* tie off command's path*/
 	  break;
       }
@@ -2307,6 +2376,7 @@ dfilter(rawcmd, input_so, output_pc, aux_filters)
 			      gf_link_filter(*aux_filters++);
 
 			    status = gf_pipe(gc, output_pc);
+			    fclose(fp);
 			}
 			else
 			  status = "Can't read result of display filter";
@@ -2401,9 +2471,9 @@ expand_filter_tokens(filter, env, tmpf, resultf, key)
 
 	    rl = fs_get(strlen(to_l) + strlen(cc_l) + strlen(bcc_l) + 3);
 	    sprintf(rl, "%s %s %s", to_l, cc_l, bcc_l);
-	    fs_give((void *)&to_l);
-	    fs_give((void *)&cc_l);
-	    fs_give((void *)&bcc_l);
+	    fs_give((void **)&to_l);
+	    fs_give((void **)&cc_l);
+	    fs_give((void **)&bcc_l);
 	    for(to_l = rl; *to_l; to_l++)	/* to_l overloaded! */
 	      if(*to_l == ',')			/* space delim'd list */
 		*to_l = ' ';
@@ -2418,7 +2488,7 @@ expand_filter_tokens(filter, env, tmpf, resultf, key)
 	if(rlp){
 	    rplstr(cmd + (rlp - filter), 12, rl ? rl : "");
 	    if(rl)
-	      fs_give((void *)&rl);
+	      fs_give((void **)&rl);
 	}
 
 	if(skp = strstr(cmd, "_PREPENDKEY_")){
