@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: newmail.c,v 4.37 1994/08/18 06:33:44 mikes Exp $";
+static char rcsid[] = "$Id: newmail.c,v 4.68 1996/07/09 22:20:17 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -16,28 +16,17 @@ static char rcsid[] = "$Id: newmail.c,v 4.37 1994/08/18 06:33:44 mikes Exp $";
 
    Please address all bugs and comments to "pine-bugs@cac.washington.edu"
 
-   Copyright 1989-1994  University of Washington
 
-    Permission to use, copy, modify, and distribute this software and its
-   documentation for any purpose and without fee to the University of
-   Washington is hereby granted, provided that the above copyright notice
-   appears in all copies and that both the above copyright notice and this
-   permission notice appear in supporting documentation, and that the name
-   of the University of Washington not be used in advertising or publicity
-   pertaining to distribution of the software without specific, written
-   prior permission.  This software is made available "as is", and
-   THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
-   WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
-   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
-   NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
-   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-   LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
-   (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
-   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
-  
-   Pine and Pico are trademarks of the University of Washington.
-   No commercial use of these trademarks may be made without prior
-   written permission of the University of Washington.
+   Pine and Pico are registered trademarks of the University of Washington.
+   No commercial use of these trademarks may be made without prior written
+   permission of the University of Washington.
+
+   Pine, Pico, and Pilot software and its included text are Copyright
+   1989-1996 by the University of Washington.
+
+   The full text of our legal notices is contained in the file called
+   CPYRIGHT, included with this distribution.
+
 
    Pine is in part based on The Elm Mail System:
     ***********************************************************************
@@ -60,11 +49,11 @@ static char rcsid[] = "$Id: newmail.c,v 4.37 1994/08/18 06:33:44 mikes Exp $";
 #include "headers.h"
 
 
-#ifdef ANSI
-void new_mail_mess(MAILSTREAM *, char *, long, long);
-#else
-void new_mail_mess();
-#endif
+/*
+ * Internal prototypes
+ */
+void new_mail_mess PROTO((MAILSTREAM *, char *, long, long, int));
+void fixup_flags PROTO((MAILSTREAM *, MSGNO_S *, long));
 
 
 static long mailbox_mail_since_command = 0L,
@@ -73,10 +62,9 @@ static long mailbox_mail_since_command = 0L,
 /*----------------------------------------------------------------------
      new_mail() - check for new mail in the inbox
  
-  Input:  cursor_moved -- address of flag that is set if the cursor was
-                          moved due to checkpointing.
-          force       -- flag indicating we should check for new mail no matter
+  Input:  force       -- flag indicating we should check for new mail no matter
           time_for_check_point -- 0: GoodTime, 1: BadTime, 2: VeryBadTime
+	  do_status_message -- flag telling us to q a new mail status message
 
   Result: returns -1 if there was no new mail. Otherwise returns the
           sorted message number of the smallest message number that
@@ -95,8 +83,8 @@ static long mailbox_mail_since_command = 0L,
  
  --- */
 long
-new_mail(cursor_moved, force, time_for_check_point)
-     int *cursor_moved, force, time_for_check_point;
+new_mail(force, time_for_check_point, do_status_msg)
+    int force, time_for_check_point, do_status_msg;
 {
     static time_t last_check = 0;
     static time_t last_check_point_call = 0;
@@ -105,20 +93,29 @@ new_mail(cursor_moved, force, time_for_check_point)
     MAILSTREAM   *stream;
     register struct pine *pine_state;
     int           checknow = 0;
+    int           act_on_deferred = 0;
+    int           deferring_sort;
 
-    dprint(9, (debugfile, "new mail called (%p %d %d)\n",
-               cursor_moved, force, time_for_check_point));
+    dprint(9, (debugfile, "new mail called (%d %d %d)\n",
+               force, time_for_check_point, do_status_msg));
     pine_state = ps_global;  /*  this gets called out of the composer which
                               *  doesn't know about pine_state
                               */
     now = time(0);
 
+    deferring_sort = pine_state->sort_is_deferred;	/* for this time */
+    pine_state->sort_is_deferred = 0;			/* for next time */
+    if(!deferring_sort && pine_state->unsorted_newmail)
+      act_on_deferred++;
+
     /*
      * only check every 15 seconds, unless we're compelled to
      */
     if(!(stream = pine_state->mail_stream)
+       || (timeout == 0 && !force)
        || (now-last_check_point_call <= 15 && time_for_check_point != 0
-	   && !pine_state->mail_box_changed && !force))
+	   && !pine_state->mail_box_changed && !force
+	   && !act_on_deferred))
       return(-1);
     else if(force || now-last_check >= timeout-2){ /* 2: check each timeout */
 	checknow++;
@@ -129,10 +126,10 @@ new_mail(cursor_moved, force, time_for_check_point)
 
     if(!check_point(time_for_check_point == 0 ? GoodTime:
 	      time_for_check_point == 1 ? BadTime : VeryBadTime) && checknow){
-	if(F_ON(F_SHOW_DELAY_CUE, ps_global)){
+	if(F_ON(F_SHOW_DELAY_CUE, ps_global) && !ps_global->in_init_seq){
 	    StartInverse();
-	    PutLine0(0, 1, "*");	/* Show something indicate delay*/
-	    MoveCursor(ps_global->ttyo->screen_rows -3, 0);
+	    PutLine0(0, 1, "*");	/* Show something to indicate delay */
+	    MoveCursor(ps_global->ttyo->screen_rows -FOOTER_ROWS(ps_global),0);
 	    fflush(stdout);
 	}
 
@@ -149,22 +146,19 @@ new_mail(cursor_moved, force, time_for_check_point)
 	now = time(0);
 	dprint(7, (debugfile, "Ping complete: %s\n", ctime(&now)));
 #endif
-	if(F_ON(F_SHOW_DELAY_CUE, ps_global)){
+	if(F_ON(F_SHOW_DELAY_CUE, ps_global) && !ps_global->in_init_seq){
 	    PutLine0(0, 1, " ");
 	    EndInverse();
 	    fflush(stdout);
 	}
     }
 
-    if(F_ON(F_SHOW_DELAY_CUE, ps_global) && cursor_moved)
-      *cursor_moved = 1;		/* if we got here, the cursor moved. */
-
     if(checknow && pine_state->inbox_stream 
        && stream != pine_state->inbox_stream){
 	if(F_ON(F_SHOW_DELAY_CUE, ps_global)){
 	    StartInverse();
-	    PutLine0(0, 1, "*"); 	/* Show something indicate delay*/
-	    MoveCursor(ps_global->ttyo->screen_rows -3, 0);
+	    PutLine0(0, 1, "*"); 	/* Show something to indicate delay */
+	    MoveCursor(ps_global->ttyo->screen_rows -FOOTER_ROWS(ps_global),0);
 	    fflush(stdout);
 	}
 
@@ -190,7 +184,7 @@ new_mail(cursor_moved, force, time_for_check_point)
     /*-------------------------------------------------------------
        Mail box state changed, could be additions or deletions.
       -------------------------------------------------------------*/
-    if(pine_state->mail_box_changed) {
+    if(pine_state->mail_box_changed || act_on_deferred) {
         dprint(7, (debugfile,
         "New mail, %s,  new_mail_count:%d  expunge count:%d,  max_msgno:%d\n",
                    pine_state->mail_stream == pine_state->inbox_stream ?
@@ -199,81 +193,45 @@ new_mail(cursor_moved, force, time_for_check_point)
                    pine_state->expunge_count,
                    mn_get_total(pine_state->msgmap)));
 
-        if(ps_global->new_mail_count > 0) {
+	if(pine_state->mail_box_changed)
+	  fixup_flags(pine_state->mail_stream, pine_state->msgmap,
+		    pine_state->new_mail_count);
+
+	/*
+	 * Lastly, worry about sorting if we got something new, otherwise
+	 * it was taken care of inside mm_expunge...
+	 */
+	if((pine_state->new_mail_count > 0L || pine_state->unsorted_newmail)
+	   && !deferring_sort
+	   && !any_lflagged(pine_state->msgmap, MN_HIDE)){
+	    /*
+	     * When new mail arrived, mm_exists cleared the cache.
+	     * However, if we deferred the sort at that time and are
+	     * sorting now, we need to clear the cache again.
+	     */
+	    if(pine_state->unsorted_newmail)
+	      clear_index_cache();
+
+	    sort_current_folder(1, mn_get_sort(pine_state->msgmap),
+				mn_get_revsort(pine_state->msgmap));
+	}
+	else if(pine_state->new_mail_count > 0L)
+	  pine_state->unsorted_newmail = 1;
+
+        if(ps_global->new_mail_count > 0 || mailbox_mail_since_command) {
             mailbox_mail_since_command += ps_global->new_mail_count;
 
             new_mail_mess(pine_state->mail_stream,
                       pine_state->mail_stream == pine_state->inbox_stream ?
                       NULL :  pine_state->cur_folder,
                       mailbox_mail_since_command,
-		      mn_get_total(pine_state->msgmap));
+		      mn_get_total(pine_state->msgmap),
+		      do_status_msg);
         }
-
-	/*
-	 * Straighten out any local flag problems here.  We can't
-	 * take care of them in the mm_exists or mm_expunged callbacks
-	 * since the flags themselves are in an MESSAGECACHE and we're
-	 * not allowed to reenter c-client from a callback...
-	 */
-	if(pine_state->new_mail_count > 0L
-	   && any_lflagged(pine_state->msgmap, MN_HIDE)){
-	    rv = mn_get_total(pine_state->msgmap)
-		   - pine_state->new_mail_count + 1L;
-	    while(rv > 0L && rv <= mn_get_total(pine_state->msgmap))
-	      set_lflag(pine_state->mail_stream, pine_state->msgmap, rv++,
-			MN_HIDE, 1);
-	}
-
-	/*
-	 * Also, worry about the case where expunged away all of the 
-	 * zoomed messages.  Unhide everything in that case...
-	 */
-	if(mn_get_total(pine_state->msgmap) > 0L){
-	    if(any_lflagged(pine_state->msgmap, MN_HIDE)
-					  >= mn_get_total(pine_state->msgmap)){
-		for(rv = 1L; rv <= mn_get_total(pine_state->msgmap); rv++)
-		  set_lflag(pine_state->mail_stream, pine_state->msgmap, rv,
-			    MN_HIDE, 0);
-
-		mn_set_cur(pine_state->msgmap,
-			   mn_get_total(pine_state->msgmap));
-	    }
-	    else if(any_lflagged(pine_state->msgmap, MN_HIDE)){
-		/*
-		 * if we got here, there are some hidden messages and
-		 * some not.  Make sure the current message is one
-		 * that's not...
-		 */
-		for(rv = mn_get_cur(pine_state->msgmap);
-		    rv <= mn_get_total(pine_state->msgmap);
-		    rv++){
-		    if(!get_lflag(pine_state->mail_stream, pine_state->msgmap,
-				 rv, MN_HIDE)){
-			mn_set_cur(pine_state->msgmap, rv);
-			break;
-		    }
-		}
-
-		for(rv = mn_get_cur(pine_state->msgmap); rv > 0L; rv--){
-		    if(!get_lflag(pine_state->mail_stream, pine_state->msgmap,
-				 rv, MN_HIDE)){
-			mn_set_cur(pine_state->msgmap, rv);
-			break;
-		    }
-		}
-	    }
-	}
-
-	/*
-	 * Lastly, worry about sorting if we got something new, otherwise
-	 * it was taken care of inside mm_expunge...
-	 */
-	if(pine_state->new_mail_count > 0L)
-	  sort_current_folder();
     }
 
-    if(pine_state->inbox_changed &&
-                       pine_state->inbox_stream != pine_state->mail_stream) {
+    if(pine_state->inbox_changed
+       && pine_state->inbox_stream != pine_state->mail_stream) {
         /*--  New mail for the inbox, queue up the notification           --*/
         /*-- If this happens then inbox is not current stream that's open --*/
         dprint(7, (debugfile,
@@ -282,20 +240,34 @@ new_mail(cursor_moved, force, time_for_check_point)
                    pine_state->inbox_expunge_count,
                    mn_get_total(pine_state->inbox_msgmap)));
 
-        if(pine_state->inbox_new_mail_count > 0) {
+	fixup_flags(pine_state->inbox_stream, pine_state->inbox_msgmap,
+		    pine_state->inbox_new_mail_count);
+
+        if(pine_state->inbox_new_mail_count > 0 || inbox_mail_since_command) {
             inbox_mail_since_command       += ps_global->inbox_new_mail_count;
             ps_global->inbox_new_mail_count = 0;
             new_mail_mess(pine_state->inbox_stream,NULL,
                           inbox_mail_since_command,
-			  mn_get_total(pine_state->inbox_msgmap));
+			  mn_get_total(pine_state->inbox_msgmap),
+			  do_status_msg);
         }
     }
 
-    rv = pine_state->new_mail_count + pine_state->expunge_count;
-    pine_state->inbox_changed    = 0;
-    pine_state->mail_box_changed = 0;
+    rv = mailbox_mail_since_command
+	 + inbox_mail_since_command + pine_state->expunge_count;
+    if(do_status_msg){
+	pine_state->inbox_changed    = 0;
+	pine_state->mail_box_changed = 0;
+    }
+
     ps_global->new_mail_count    = 0L;
 
+    if(pine_state->shown_newmail
+       && !(mailbox_mail_since_command || inbox_mail_since_command)){
+	icon_text(NULL);
+	pine_state->shown_newmail = 0;
+    }
+
     dprint(6, (debugfile, "******** new mail returning %ld  ********\n", 
 	       rv ? rv : -1));
     return(rv ? rv : -1);
@@ -315,15 +287,16 @@ status_message code will fit what it can on the screen and truncation on
 the right is about what we want which is what will happen.
   ----*/
 void
-new_mail_mess(stream, folder, number, max_num)
+new_mail_mess(stream, folder, number, max_num, do_status_msg)
      MAILSTREAM *stream;
      long        number, max_num;
      char       *folder;
+     int         do_status_msg;
 
 {
-    ENVELOPE *e;
-    char      subject[200], from[MAX_ADDRESS+20], intro[50 + MAXFOLDER];
-    static char   *carray[] = { "regarding",
+    ENVELOPE	*e;
+    char	 subject[200], from[2*MAX_SCREEN_COLS], intro[50 + MAXFOLDER];
+    static char *carray[] = { "regarding",
 				"concerning",
 				"about",
 				"as to",
@@ -337,6 +310,13 @@ new_mail_mess(stream, folder, number, max_num)
 				"subject:"
     };
 
+#ifdef _WINDOWS
+    mswin_newmailicon ();
+#endif
+
+    if(!do_status_msg)
+      return;
+
     if(!folder) {
         if(number > 1)
           sprintf(intro, "%ld new messages!", number);
@@ -350,29 +330,103 @@ new_mail_mess(stream, folder, number, max_num)
           sprintf(intro, "Mail saved to folder \"%s\"", folder);
     }
 
-    if((e = mail_fetchstructure(stream, max_num, NULL)) && e->from)
-      sprintf(from, " %srom %s%s%s", 
-	     (number > 1L) ? "Most recent f" : "F",
-	     (e->from->personal) ? e->from->personal : e->from->mailbox,
-	     (e->from->personal) ? "" : "@",
-	     (e->from->personal) ? "" : e->from->host);
+    if((e = mail_fetchstructure(stream, max_num, NULL)) && e->from){
+	sprintf(from, " %srom ", (number > 1L) ? "Most recent f" : "F");
+	if(e->from->personal)
+	  istrncpy(from + ((number > 1L) ? 18 : 6),
+		   (char *) rfc1522_decode((unsigned char *) tmp_20k_buf,
+					   e->from->personal, NULL),
+		   ps_global->ttyo->screen_cols);
+	else
+	  sprintf(from + ((number > 1L) ? 18 : 6), "%s@%s", 
+		  e->from->mailbox, e->from->host);
+    }
     else
       from[0] = '\0';
 
     if(number <= 1L) {
-        if(e && e->subject)
-	  sprintf(subject, " %s %.150s", carray[(unsigned)random()%12],
-		  e->subject);
+        if(e && e->subject){
+	    sprintf(subject, " %s ", carray[(unsigned)random()%12]);
+	    istrncpy(subject + strlen(subject),
+		     (char *) rfc1522_decode((unsigned char *) tmp_20k_buf,
+					     e->subject, NULL), 100);
+	}
 	else
 	  strcpy(subject, " with no subject");
 
         if(!from[0])
-          subject[1] = toupper(subject[1]);
+          subject[1] = toupper((unsigned char)subject[1]);
 
-        q_status_message3(0, 2, 4, "\007%s%s%s", intro, from, subject);
+        q_status_message3(SM_ASYNC | SM_DING, 0, 60,
+			  "%s%s%s", intro, from, subject);
     }
     else
-      q_status_message2(0, 2, 4, "\007%s%s", intro, from);
+      q_status_message2(SM_ASYNC | SM_DING, 0, 60, "%s%s", intro, from);
+
+    sprintf(tmp_20k_buf, "%s%s", intro, from);
+    icon_text(tmp_20k_buf);
+    ps_global->shown_newmail = 1;
+}
+
+
+
+/*----------------------------------------------------------------------
+  Straighten out any local flag problems here.  We can't take care of
+  them in the mm_exists or mm_expunged callbacks since the flags
+  themselves are in an MESSAGECACHE and we're not allowed to reenter
+  c-client from a callback...
+
+ Args: stream -- mail stream to operate on
+       msgmap -- messages in that stream to fix up
+       new_msgs -- number of new messages
+
+ Result: returns with local flags as they should be
+
+  ----*/
+void
+fixup_flags(stream, msgmap, new_msgs)
+    MAILSTREAM *stream;
+    MSGNO_S    *msgmap;
+    long	new_msgs;
+{
+    long i;
+
+    if(new_msgs > 0L && any_lflagged(msgmap, MN_HIDE)){
+	i = mn_get_total(msgmap) - new_msgs + 1L;
+	while(i > 0L && i <= mn_get_total(msgmap))
+	  set_lflag(stream, msgmap, i++, MN_HIDE, 1);
+    }
+
+    /*
+     * Also, worry about the case where expunged away all of the 
+     * zoomed messages.  Unhide everything in that case...
+     */
+    if(mn_get_total(msgmap) > 0L){
+	if(any_lflagged(msgmap, MN_HIDE) >= mn_get_total(msgmap)){
+	    for(i = 1L; i <= mn_get_total(msgmap); i++)
+	      set_lflag(stream, msgmap, i, MN_HIDE, 0);
+
+	    mn_set_cur(msgmap, mn_get_total(msgmap));
+	}
+	else if(any_lflagged(msgmap, MN_HIDE)){
+	    /*
+	     * if we got here, there are some hidden messages and
+	     * some not.  Make sure the current message is one
+	     * that's not...
+	     */
+	    for(i = mn_get_cur(msgmap); i <= mn_get_total(msgmap); i++)
+	      if(!get_lflag(stream, msgmap, i, MN_HIDE)){
+		  mn_set_cur(msgmap, i);
+		  break;
+	      }
+
+	    for(i = mn_get_cur(msgmap); i > 0L; i--)
+	      if(!get_lflag(stream, msgmap, i, MN_HIDE)){
+		  mn_set_cur(msgmap, i);
+		  break;
+	      }
+	}
+    }
 }
 
 
@@ -387,9 +441,9 @@ new_mail_mess(stream, folder, number, max_num)
   Result: returns 1 if checkpoint was written, 
                   0 if not.
 
-NOTE: mail_check will also notice new mail arrival, so it's impetive that
+NOTE: mail_check will also notice new mail arrival, so it's imperative that
 code exist after this function is called that can deal with updating the 
-various pieces of pines state associated with the message count and such.
+various pieces of pine's state associated with the message count and such.
 
 Only need to checkpoint current stream because there are no changes going
 on with other streams when we're not manipulating them.
@@ -414,7 +468,7 @@ check_point(timing)
                timing == BadTime  ? "BadTime" :
                timing == VeryBadTime  ? "VeryBadTime" : "DoItNow"));
 
-    if(!ps_global->mail_stream || ps_global->mail_stream->readonly ||
+    if(!ps_global->mail_stream || ps_global->mail_stream->rdonly ||
 							     check_count == 0)
 	 return(0);
 
@@ -455,7 +509,7 @@ check_point(timing)
 	if(F_ON(F_SHOW_DELAY_CUE, ps_global)){
 	    StartInverse();
 	    PutLine0(0, 0, "**");	/* Show something indicate delay*/
-	    MoveCursor(ps_global->ttyo->screen_rows -3, 0);
+	    MoveCursor(ps_global->ttyo->screen_rows -FOOTER_ROWS(ps_global),0);
 	    fflush(stdout);
 	}
 
@@ -557,10 +611,10 @@ streams_died()
         inbox = 1;
     }
     if(rv == 1) 
-      q_status_message1(1, 3, 6,
-                        "\007MAIL FOLDER \"%s\" CLOSED DUE TO ACCESS ERROR",
-                        pretty_fn(inbox? ps_global->inbox_name :
-                                         ps_global->cur_folder));
+      q_status_message1(SM_ORDER | SM_DING, 3, 6,
+                        "MAIL FOLDER \"%s\" CLOSED DUE TO ACCESS ERROR",
+                        pretty_fn(inbox ? ps_global->inbox_name
+				  	: ps_global->cur_folder));
     return(rv);
 }
         
