@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: os-a32.h,v 5.40 1996/03/15 06:43:47 hubert Exp $
+  $Id: os-a32.h,v 5.46 1996/06/05 17:12:38 mikes Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -65,13 +65,24 @@
 /*----------------------------------------------------------------------
    Define this if you want to allow the users to change their From header
    line when they send out mail.  The users will still have to configure
-   either default-composer-hdrs or custumized-hdrs to get at the From
+   either default-composer-hdrs or customized-hdrs to get at the From
    header, even if this is set.
  ----*/
 /* #define ALLOW_CHANGING_FROM  /* comment out to not allow changing From */
 
 
 
+/*----------------------------------------------------------------------
+   Define this if you want to allow users to turn on the feature that
+   enables sending to take place in a fork()'d child.  This may reduce
+   the time on the user's wall clock it takes to post mail.
+   NOTE: You'll also have to make sure the appropriate osdep/postreap.*
+         file is included in the os-*.ic file for your system.
+ ----*/
+#define BACKGROUND_POST  /* comment out to disable posting from child */
+
+
+
 /*----------------------------------------------------------------------
     Turn this on if you want to disable the keyboard lock function.
  ----*/
@@ -122,6 +133,14 @@
 
 
 
+/*----------------------------------------------------------------------
+ When scrolling screens, the number of lines from top and bottom of
+ the screen to initiate single-line scrolling.
+ ----*/
+#define	DF_MARGIN	"0"
+
+
+
 /*----------------------------------------------------------------------
  Default fill column for pine composer and maximum fill column.  The max
  is used to stop people from setting their custom fill column higher than
@@ -211,6 +230,11 @@
 
 
 
+/*----------------- locale.h -------------------------------------------*/
+#include <locale.h>  /* To make matching and sorting work right */
+
+
+
 /*----------------- time.h ---------------------------------------------*/
 #include <time.h>
 /* plain time.h isn't enough on some systems */
@@ -226,10 +250,8 @@
 /* #define SigType int      /* value returned by sig handlers is int */
 
 #define POSIX_SIGNALS    /* use POSIX signal semantics (ttyin.c) */
+/* #define SYSV_SIGNALS    /* use System-V signal semantics (ttyin.c) */
 
-/*--- For AT&T-derived systems ----------------------------------------*/
-#define signal(a,b) sigset(a, b) /* sigset is just like signal */
-extern void (*sigset())();       /* declare sigset if not declared already */
 
 
 
@@ -257,14 +279,32 @@ typedef unsigned short usign16_t;
 
 
 
-#include <termio.h>     /* this is needed for struct winsize */
+/*
+ * Choose one of the following three terminal drivers
+ */
+
+/*--------- Good 'ol BSD -----------------------------------------------*/
+/* #include <sgtty.h>      /* BSD-based systems */
+
 /*--------- System V terminal driver -----------------------------------*/
 /* #define HAVE_TERMIO     /* this is for pure System V */
+/* #include <termio.h>     /* Sys V */
+
+/*--------- POSIX terminal driver --------------------------------------*/
+#include <termio.h>     /* this is needed for struct winsize on aix */
 #define HAVE_TERMIOS    /* this is an alternative */
+#include <termios.h>    /* POSIX */
+
+
+
+/*-------- Use poll system call instead of select ----------------------*/
 /* #define USE_POLL        /* use the poll() system call instead of select() */
+
+
+
+/*-------- Use terminfo database instead of termcap --------------------*/
 /* #define USE_TERMINFO    /* use terminfo instead of termcap */
-/* #include <sys/stream.h> /* for ptem.h */
-/* #include <sys/ptem.h>   /* for struct winsize */
+
 
 
 /*------------ AIX 3.2 needs this for types passed select() ------------*/
@@ -278,7 +318,9 @@ typedef unsigned short usign16_t;
 
 
 /*-------- Is window resizing available? -------------------------------*/
+#if defined(TIOCGWINSZ) && defined(SIGWINCH)
 #define RESIZING  /* SIGWINCH and friends */
+#endif
 
 
 
