@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: os-gen.h,v 5.18 1994/10/10 22:29:35 mikes Exp $
+  $Id: os-gen.h,v 5.42 1996/06/05 17:12:38 mikes Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -14,28 +14,17 @@
 
    Please address all bugs and comments to "pine-bugs@cac.washington.edu"
 
-   Copyright 1989-1994  University of Washington
 
-    Permission to use, copy, modify, and distribute this software and its
-   documentation for any purpose and without fee to the University of
-   Washington is hereby granted, provided that the above copyright notice
-   appears in all copies and that both the above copyright notice and this
-   permission notice appear in supporting documentation, and that the name
-   of the University of Washington not be used in advertising or publicity
-   pertaining to distribution of the software without specific, written
-   prior permission.  This software is made available "as is", and
-   THE UNIVERSITY OF WASHINGTON DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED,
-   WITH REGARD TO THIS SOFTWARE, INCLUDING WITHOUT LIMITATION ALL IMPLIED
-   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, AND IN
-   NO EVENT SHALL THE UNIVERSITY OF WASHINGTON BE LIABLE FOR ANY SPECIAL,
-   INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
-   LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, TORT
-   (INCLUDING NEGLIGENCE) OR STRICT LIABILITY, ARISING OUT OF OR IN CONNECTION
-   WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
-  
-   Pine and Pico are trademarks of the University of Washington.
-   No commercial use of these trademarks may be made without prior
-   written permission of the University of Washington.
+   Pine and Pico are registered trademarks of the University of Washington.
+   No commercial use of these trademarks may be made without prior written
+   permission of the University of Washington.
+
+   Pine, Pico, and Pilot software and its included text are Copyright
+   1989-1996 by the University of Washington.
+
+   The full text of our legal notices is contained in the file called
+   CPYRIGHT, included with this distribution.
+
 
    Pine is in part based on The Elm Mail System:
     ***********************************************************************
@@ -76,13 +65,24 @@
 /*----------------------------------------------------------------------
    Define this if you want to allow the users to change their From header
    line when they send out mail.  The users will still have to configure
-   either default-composer-hdrs or custumized-hdrs to get at the From
+   either default-composer-hdrs or customized-hdrs to get at the From
    header, even if this is set.
  ----*/
 /* #define ALLOW_CHANGING_FROM  /* comment out to not allow changing From */
 
 
 
+/*----------------------------------------------------------------------
+   Define this if you want to allow users to turn on the feature that
+   enables sending to take place in a fork()'d child.  This may reduce
+   the time on the user's wall clock it takes to post mail.
+   NOTE: You'll also have to make sure the appropriate osdep/postreap.*
+         file is included in the os-*.ic file for your system.
+ ----*/
+/* #define BACKGROUND_POST  /* comment out to disable posting from child */
+
+
+
 /*----------------------------------------------------------------------
     Turn this on if you want to disable the keyboard lock function.
  ----*/
@@ -91,8 +91,8 @@
 
 
 /*----------------------------------------------------------------------
-    Turn this on if it's OK to encode sent message text containing
-  "From " at the beginning of any line.
+    Turn this on to trigger QP encoding of sent message text if it contains
+  "From " at the beginning of a line or "." on a line by itself.
  ----*/
 /* #define ENCODE_FROMS */
 
@@ -101,9 +101,11 @@
 /*----------------------------------------------------------------------
     Timeouts (seconds)
  ----*/
-#define NEW_MAIL_TIME      (150) /* How often to check for new mail. There's 
-                                   some expense in doing this so it shouldn't
-                                   be done too frequently.  */
+#define DF_MAILCHECK      "150" /* How often to check for new mail, by
+				   default.  There's some expense in doing
+				   this so it shouldn't be done too
+				   frequently.  (Can be set in config
+				   file now.)  */
 
 /*----------------------------------------------------------------------
     Check pointing (seconds)
@@ -127,7 +129,25 @@
  In scrolling through text, the number of lines from the previous
  screen to overlap when showing the next screen.  Usually set to two.
  ----*/
-#define	OVERLAP		(2) 
+#define	DF_OVERLAP	"2"
+
+
+
+/*----------------------------------------------------------------------
+ When scrolling screens, the number of lines from top and bottom of
+ the screen to initiate single-line scrolling.
+ ----*/
+#define	DF_MARGIN	"0"
+
+
+
+/*----------------------------------------------------------------------
+ Default fill column for pine composer and maximum fill column.  The max
+ is used to stop people from setting their custom fill column higher than
+ that number.  Note that DF_FILLCOL is a string but MAX_FILLCOL is an integer.
+ ----*/
+#define	DF_FILLCOL	"74"
+#define	MAX_FILLCOL	80
 
 
 
@@ -146,6 +166,7 @@
 #define POSTPONED_MAIL            "postponed-mail"
 #define POSTPONED_MSGS            "postponed-msgs"
 #define INTERRUPTED_MAIL          ".pine-interrupted-mail"
+#define DEADLETTER                "dead.letter"
 #define DF_MAIL_DIRECTORY         "mail"
 #define INBOX_NAME                "INBOX"
 #define DF_SIGNATURE_FILE         ".signature"
@@ -164,6 +185,13 @@
 #define DF_ADDRESSBOOK            ".addressbook"
 #define DF_BUGS_FULLNAME          "Pine Developers"
 #define DF_BUGS_ADDRESS           "pine-bugs@cac.washington.edu"
+#define DF_SUGGEST_FULLNAME       "Pine Developers"
+#define DF_SUGGEST_ADDRESS        "pine-suggestions@cac.washington.edu"
+#define DF_PINEINFO_FULLNAME      "Pine-Info News Group"
+#define DF_PINEINFO_ADDRESS       "pine-info@cac.washington.edu"
+#define DF_LOCAL_FULLNAME         "Local Support"
+#define DF_LOCAL_ADDRESS          "postmaster"
+#define DF_KBLOCK_PASSWD_COUNT    "1"
 
 /*----------------------------------------------------------------------
    The default printer when pine starts up for the first time with no printer
@@ -190,23 +218,6 @@
 /*------ If our compiler doesn't understand type void ------------------*/
 /* #define void char     /* no void in compiler */
 
-/******* These are included in c-client/osdep.h. ************************/
-#ifdef notdef
-/*-------- Standard ANSI functions usually defined in stdlib.h ---------*/
-#include <stdlib.h>  /* declarations of many common functions */
-/*---- Declare getenv() if not already declared ------------------------*/
-/* char *getenv();      /* often declared in stdlib.h */
-
-
-/*------- Standard string functions ------------------------------------*/
-#include <string.h>
-/*---- Declare these if not already declared ---------------------------*/
-/* char *strcpy(), *strncpy();  /* usually in string.h */
-
-
-/*------- Include types.h if you have it -------------------------------*/
-#include <sys/types.h>
-#endif /*****************************************************************/
 
 
 /*------- Some more includes that should usually be correct ------------*/
@@ -218,6 +229,11 @@
 
 
 
+/*----------------- locale.h -------------------------------------------*/
+/* #include <locale.h>  /* To make matching and sorting work right */
+
+
+
 /*----------------- time.h ---------------------------------------------*/
 #include <time.h>
 /* plain time.h isn't enough on some systems */
@@ -229,14 +245,18 @@
 #include <signal.h>      /* sometimes both required, sometimes */
 /* #include <sys/signal.h>  /* only one or the other */
 
-#define SigType void     /* value returned by signal() is void * */
-/* #define SigType int      /* value returned by signal() is int * */
+#define SigType void     /* value returned by sig handlers is void */
+/* #define SigType int      /* value returned by sig handlers is int */
 
 /* #define POSIX_SIGNALS    /* use POSIX signal semantics (ttyin.c) */
+/* #define SYSV_SIGNALS    /* use System-V signal semantics (ttyin.c) */
 
-/*--- For AT&T-derviced systems ----------------------------------------*/
-/* #define signal(a,b) sigset(a, b) /* sigset is just like signal */
-/* extern void (*sigset())();       /* declare sigset if not declared already */
+
+
+
+/*-------------- A couple typedef's for integer sizes ------------------*/
+typedef unsigned int usign32_t;
+typedef unsigned short usign16_t;
 
 
 
@@ -246,19 +266,42 @@
 
 
 
+/*-------------- fcntl flag to set non-blocking IO ---------------------*/
+#define	NON_BLOCKING_IO	O_NONBLOCK		/* POSIX style */
+/*#define	NON_BLOCKING_IO	FNDELAY		/* good ol' bsd style  */
+
+
+
 /*------ how help text is referenced (always char ** on Unix) ----------*/
 #define HelpType char **
 #define NO_HELP (char **)NULL
 
 
 
+/*
+ * Choose one of the following three terminal drivers
+ */
+
+/*--------- Good 'ol BSD -----------------------------------------------*/
+/* #include <sgtty.h>      /* BSD-based systems */
+
 /*--------- System V terminal driver -----------------------------------*/
 /* #define HAVE_TERMIO     /* this is for pure System V */
+/* #include <termio.h>     /* Sys V */
+
+/*--------- POSIX terminal driver --------------------------------------*/
 /* #define HAVE_TERMIOS    /* this is an alternative */
+/* #include <termios.h>    /* POSIX */
+
+
+
+/*-------- Use poll system call instead of select ----------------------*/
 /* #define USE_POLL        /* use the poll() system call instead of select() */
+
+
+
+/*-------- Use terminfo database instead of termcap --------------------*/
 /* #define USE_TERMINFO    /* use terminfo instead of termcap */
-/* #include <sys/stream.h> /* for ptem.h */
-/* #include <sys/ptem.h>   /* for struct winsize */
 
 
 
@@ -266,8 +309,11 @@
 #define HAVE_WAIT_UNION  /* the arg to wait is a union wait * */
 
 
+
 /*-------- Is window resizing available? -------------------------------*/
+#if defined(TIOCGWINSZ) && defined(SIGWINCH)
 #define RESIZING  /* SIGWINCH and friends */
+#endif
 
 
 
@@ -278,10 +324,14 @@
 
 /*----- The usual sendmail configuration for sending mail on Unix ------*/
 #define SENDMAIL	"/usr/lib/sendmail"
-#define SENDMAILFLAGS	"-oi -oem -t"	/* ignore dots in incoming mail,
-					   mail back errors, and
-					   get recipients from To, cc,
-					   and bcc header lines.      */
+#define SENDMAILFLAGS	"-bs -odb -oem"	/* send via smtp with backgroud
+					   delivery and mail back errors */
+
+
+/*----------------------------------------------------------------------
+   If no nntp-servers are defined, this program will be used to post news.
+ ----*/
+/* #define SENDNEWS	"/usr/local/bin/inews -h"	/* news posting cmd */
 
 
 /*--------- Program employed by users to change their password ---------*/
