@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: pine.c,v 4.334 1996/05/16 23:43:36 hubert Exp $";
+static char rcsid[] = "$Id: pine.c,v 4.335 1996/07/15 20:32:28 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: pine.c,v 4.334 1996/05/16 23:43:36 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-1996 by the University of Washington.
+   1989-1997 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -46,7 +46,7 @@ static char rcsid[] = "$Id: pine.c,v 4.334 1996/05/16 23:43:36 hubert Exp $";
  * Handy local definitions...
  */
 #define	LEGAL_NOTICE \
-   "Copyright 1989-1996.  PINE is a trademark of the University of Washington."
+   "Copyright 1989-1997.  PINE is a trademark of the University of Washington."
 
 #define	STDIN_FD	0			/* Our normal input desc    */
 #define	STDER_FD	2			/* Another desc tied to tty */ 
@@ -2030,7 +2030,7 @@ phone_home_blurb(dline)
 {
     static char *blurb[] = {
       "SPECIAL OFFER:  Would you like to receive (via email) a brief document",
-      "  entitled \"Getting the most out of Pine 3.94\" ?"
+      "  entitled \"Getting the most out of Pine\" ?"
     };
 
     if(dline+3 < ps_global->ttyo->screen_rows - FOOTER_ROWS(ps_global))
