@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: strings.c,v 4.42 1996/04/08 23:12:53 mikes Exp $";
+static char rcsid[] = "$Id: strings.c,v 4.48 1996/07/09 22:23:27 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -55,6 +55,7 @@ static char rcsid[] = "$Id: strings.c,v 4.42 1996/04/08 23:12:53 mikes Exp $";
       - strindex       Replacement for strchr/index
       - strrindex      Replacement for strrchr/rindex
       - sstrcpy        Copy one string onto another, advancing dest'n pointer
+      - istrncpy       Copy n chars between bufs, making ctrl chars harmless
       - month_abbrev   Return three letter abbreviations for months
       - month_num      Calculate month number from month/year string
       - cannon_date    Formalize format of a some what formatted date
@@ -133,8 +134,8 @@ sqzspaces(string)
 {
     char *p = string;
 
-    while(*string = *p++)		/* while something to copy       */
-      if(!isspace(*string))		/* only really copy if non-blank */
+    while(*string = *p++)		   /* while something to copy       */
+      if(!isspace((unsigned char)*string)) /* only really copy if non-blank */
 	string++;
 }
 
@@ -467,6 +468,33 @@ sstrcpy(d, s)
 }
 
 
+/*----------------------------------------------------------------------
+  copy at most n chars of the source string onto the destination string
+  returning pointer to start of destination and converting any undisplayable
+  characters to harmless character equivalents.
+ ----*/
+char *
+istrncpy(d, s, n)
+    char *d, *s;
+    int n;
+{
+    char *rv = d;
+
+    do
+      if(F_OFF(F_PASS_CONTROL_CHARS, ps_global) && *s && CAN_DISPLAY(*s))
+	if(n-- >= 0){
+	    *d++ = '^';
+
+	    if(n-- >= 0)
+	      *d++ = *s++ + '@';
+	}
+    while(n-- >= 0 && (*d++ = *s++));
+
+    return(rv);
+}
+
+
+
 char *xdays[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", NULL};
 
 char *
@@ -607,7 +635,7 @@ parse_date(given_date, d)
       return;
 
     p = given_date;
-    while(*p && isspace(*p))
+    while(*p && isspace((unsigned char)*p))
       p++;
 
     /* Start with month, weekday or day ? */
@@ -617,16 +645,16 @@ parse_date(given_date, d)
     if(*i != NULL) {
         /* Started with week day */
         d->wkday = i - xdays;
-        while(*p && !isspace(*p) && *p != ',')
+        while(*p && !isspace((unsigned char)*p) && *p != ',')
           p++;
-        while(*p && (isspace(*p) || *p == ','))
+        while(*p && (isspace((unsigned char)*p) || *p == ','))
           p++;
     }
-    if(isdigit(*p)) {
+    if(isdigit((unsigned char)*p)) {
         d->day = atoi(p);
-        while(*p && isdigit(*p))
+        while(*p && isdigit((unsigned char)*p))
           p++;
-        while(*p && (*p == '-' || *p == ',' || isspace(*p)))
+        while(*p && (*p == '-' || *p == ',' || isspace((unsigned char)*p)))
           p++;
     }
     for(month = 1; month <= 12; month++)
@@ -637,83 +665,83 @@ parse_date(given_date, d)
 
     } 
     /* Move over month, (or whatever is there) */
-    while(*p && !isspace(*p) && *p != ',' && *p != '-')
+    while(*p && !isspace((unsigned char)*p) && *p != ',' && *p != '-')
        p++;
-    while(*p && (isspace(*p) || *p == ',' || *p == '-'))
+    while(*p && (isspace((unsigned char)*p) || *p == ',' || *p == '-'))
        p++;
 
     /* Check again for day */
-    if(isdigit(*p) && d->day == -1) {
+    if(isdigit((unsigned char)*p) && d->day == -1) {
         d->day = atoi(p);
-        while(*p && isdigit(*p))
+        while(*p && isdigit((unsigned char)*p))
           p++;
-        while(*p && (*p == '-' || *p == ',' || isspace(*p)))
+        while(*p && (*p == '-' || *p == ',' || isspace((unsigned char)*p)))
           p++;
     }
 
     /*-- Check for time --*/
-    for(q = p; *q && isdigit(*q); q++);
+    for(q = p; *q && isdigit((unsigned char)*q); q++);
     if(*q == ':') {
         /* It's the time (out of place) */
         d->hour = atoi(p);
-        while(*p && *p != ':' && !isspace(*p))
+        while(*p && *p != ':' && !isspace((unsigned char)*p))
           p++;
         if(*p == ':') {
             p++;
             d->minute = atoi(p);
-            while(*p && *p != ':' && !isspace(*p))
+            while(*p && *p != ':' && !isspace((unsigned char)*p))
               p++;
             if(*p == ':') {
                 d->sec = atoi(p);
-                while(*p && !isspace(*p))
+                while(*p && !isspace((unsigned char)*p))
                   p++;
             }
         }
-        while(*p && isspace(*p))
+        while(*p && isspace((unsigned char)*p))
           p++;
     }
     
 
     /* Get the year 0-49 is 2000-2049; 50-100 is 1950-1999 and
                                            101-9999 is 101-9999 */
-    if(isdigit(*p)) {
+    if(isdigit((unsigned char)*p)) {
         d->year = atoi(p);
         if(d->year < 50)   
           d->year += 2000;
         else if(d->year < 100)
           d->year += 1900;
-        while(*p && isdigit(*p))
+        while(*p && isdigit((unsigned char)*p))
           p++;
-        while(*p && (*p == '-' || *p == ',' || isspace(*p)))
+        while(*p && (*p == '-' || *p == ',' || isspace((unsigned char)*p)))
           p++;
     } else {
         /* Something wierd, skip it and try to resynch */
-        while(*p && !isspace(*p) && *p != ',' && *p != '-')
+        while(*p && !isspace((unsigned char)*p) && *p != ',' && *p != '-')
           p++;
-        while(*p && (isspace(*p) || *p == ',' || *p == '-'))
+        while(*p && (isspace((unsigned char)*p) || *p == ',' || *p == '-'))
           p++;
     }
 
     /*-- Now get hours minutes, seconds and ignore tenths --*/
-    for(q = p; *q && isdigit(*q); q++);
+    for(q = p; *q && isdigit((unsigned char)*q); q++);
     if(*q == ':' && d->hour == -1) {
         d->hour = atoi(p);
-        while(*p && *p != ':' && !isspace(*p))
+        while(*p && *p != ':' && !isspace((unsigned char)*p))
           p++;
         if(*p == ':') {
             p++;
             d->minute = atoi(p);
-            while(*p && *p != ':' && !isspace(*p))
+            while(*p && *p != ':' && !isspace((unsigned char)*p))
               p++;
             if(*p == ':') {
                 p++;
                 d->sec = atoi(p);
-                while(*p && !isspace(*p))
+                while(*p && !isspace((unsigned char)*p))
                   p++;
             }
         }
     }
-    while(*p && isspace(*p))
+    while(*p && isspace((unsigned char)*p))
       p++;
 
 
@@ -722,8 +750,11 @@ parse_date(given_date, d)
     d->min_off_gmt = 0;
     if(*p) {
         if((*p == '+' || *p == '-')
-	   && isdigit(p[1]) && isdigit(p[2]) && isdigit(p[3]) && isdigit(p[4])
-	   && !isdigit(p[5])) {
+	   && isdigit((unsigned char)p[1])
+	   && isdigit((unsigned char)p[2])
+	   && isdigit((unsigned char)p[3])
+	   && isdigit((unsigned char)p[4])
+	   && !isdigit((unsigned char)p[5])) {
             char tmp[3];
             d->min_off_gmt = d->hours_off_gmt = (*p == '+' ? 1 : -1);
             p++;
@@ -1134,7 +1165,7 @@ strtoval(s, val, minmum, maxmum, errbuf, varname)
     removing_leading_white_space(p);
     removing_trailing_white_space(p);
     for(; *p; p++)
-      if(isdigit(*p)){
+      if(isdigit((unsigned char)*p)){
 	  i = (i * 10) + (*p - '0');
       }
       else{
@@ -1180,8 +1211,8 @@ get_pair(string, label, value, firstws)
 	if(*p == '\\' && *(p+1) == '"')		/* escaped quote? */
 	  p++;					/* skip it... */
 
-	if(isspace(*p) && !quoted){		/* if space,  */
-	    while(*++p && isspace(*p))		/* move past it */
+	if(isspace((unsigned char)*p) && !quoted){	/* if space,  */
+	    while(*++p && isspace((unsigned char)*p))	/* move past it */
 	      ;
 
 	    if(!firstws || !token)
@@ -1222,13 +1253,13 @@ read_hex(s)
     register int i, c, j;
 
     i = 0;
-    for(j = 0; j < 2 && *s && isxdigit((unsigned char)(*s)); s++,j++){
+    for(j = 0; j < 2 && *s && isxdigit((unsigned char)*s); s++,j++){
 	c = (unsigned char)(*s);
-	if(isupper(c))
-	  c = tolower(c);
+	if(isupper((unsigned char)c))
+	  c = tolower((unsigned char)c);
 
-	i = i*16 + (isdigit(c)  ? c - '0'
-				: c - 'a' + 10);
+	i = i*16 + (isdigit((unsigned char)c)  ? c - '0'
+					       : c - 'a' + 10);
     }
 
     return(i);
@@ -1521,10 +1552,10 @@ cstring_to_hexstring(s)
 		  /* hex number */
 		  case 'x':
 		    s++;
-		    if(isxdigit(*s)){
+		    if(isxdigit((unsigned char)*s)){
 			c = read_hex(s);
 			s++;
-			if(isxdigit(*s))
+			if(isxdigit((unsigned char)*s))
 			  s++;
 		    }
 		    else
@@ -1538,12 +1569,12 @@ cstring_to_hexstring(s)
 
 		  /* octal number */
 		  default:
-		    if(isdigit(*s)){
+		    if(isdigit((unsigned char)*s)){
 			c = read_octal(s);
 			s++;
-			if(isdigit(*s)){
+			if(isdigit((unsigned char)*s)){
 			    s++;
-			    if(isdigit(*s))
+			    if(isdigit((unsigned char)*s))
 			      s++;
 			}
 		    }
@@ -1669,7 +1700,7 @@ rfc1522_decode(d, s, charset)
 	if(rfc1522_valid(sw, &cset, &enc, &txt, &ew)){
 	    /* copy everything between s and sw to destination */
 	    for(i = 0; &s[i] < sw; i++)
-	      if(!isspace(s[i])){ /* if some non-whitespace */
+	      if(!isspace((unsigned char)s[i])){ /* if some non-whitespace */
 		  while(s < sw)
 		    *d++ = (unsigned char) *s++;
 
@@ -1832,7 +1863,7 @@ int
 rfc1522_valenc(c)
     int c;
 {
-    return(!(c == '?' || c == SPACE) && isprint(c));
+    return(!(c == '?' || c == SPACE) && isprint((unsigned char)c));
 }
 
 
