@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: ttyout.c,v 4.71 1996/03/15 07:13:42 hubert Exp $";
+static char rcsid[] = "$Id: ttyout.c,v 4.80 1996/07/11 01:45:32 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -95,16 +95,6 @@ static int   _in_inverse;
 #if	!(defined(DOS) ||defined(OS2))
 /* Beginning of giant ifdef to switch between UNIX and DOS */
 
-#ifdef HAVE_TERMIOS
-#include <termios.h>
-#else
-#ifdef HAVE_TERMIO
-#include <termio.h>
-#else
-#include <sgtty.h>
-#endif
-#endif
-
 #ifdef _
 #include <sys/tty.h>
 #endif
@@ -446,15 +436,14 @@ config_screen(tt, kbesc)
 void
 init_screen()
 {
-    if(_termcap_init != NULL) {
-	/* init using termcap's rule */
-        tputs(_termcap_init, 1, outchar);
-	/* and make sure there are no scrolling surprises! */
-	BeginScroll(0, ps_global->ttyo->screen_rows);
-	/* and make sure icon text starts out consistent */
-	icon_text(NULL);
-	fflush(stdout);
-    }
+    if(_termcap_init)			/* init using termcap's rule */
+      tputs(_termcap_init, 1, outchar);
+
+    /* and make sure there are no scrolling surprises! */
+    BeginScroll(0, ps_global->ttyo->screen_rows - 1);
+    /* and make sure icon text starts out consistent */
+    icon_text(NULL);
+    fflush(stdout);
 }
         
 
@@ -532,9 +521,7 @@ end_screen(message)
       tputs(_termcap_end, 1, outchar);
 
     if(message){
-	StartInverse();
 	printf("%s\r\n", message);
-	EndInverse();
     }
 
     fflush(stdout);
@@ -940,6 +927,7 @@ DeleteChar(n)
 
    Args: top    -- top line of region to scroll
          bottom -- bottom line of region to scroll
+	 (These are zero-origin numbers)
 
  Result: either set scrolling region or
          save values for later scrolling
@@ -1125,7 +1113,11 @@ Writechar(ch, new_esc_len)
 	    break;
 
 	  default :		/* Change remaining control characters to ? */
-	    putchar('?');
+	    if(F_ON(F_PASS_CONTROL_CHARS, ps_global))
+	      putchar(ch);
+	    else
+	      putchar('?');
+
 	    if(esc_len > 0)
 	      esc_len--;
 	    else
@@ -1226,7 +1218,7 @@ int c;
 	/** output the given character.  From tputs... **/
 	/** Note: this CANNOT be a macro!              **/
 
-	putc(c, stdout);
+	putc((unsigned char)c, stdout);
 }
 
 
@@ -1459,10 +1451,10 @@ struct ttyo *ttyo;
 #else
 #ifdef OS2
     get_dimen(&_columns, &_lines);
-#else
+#endif
+
     ttyo->screen_cols = _columns;
     ttyo->screen_rows = _lines;
-#endif
     return(0);
 #endif
 }
@@ -1844,6 +1836,9 @@ void
 icon_text(s)					/* DOS */
     char *s;
 {
+#ifdef _WINDOWS
+    mswin_newmailtext ((s) ? s : ps_global->pine_name);
+#endif
     return;					/* NO OP */
 }
 
